SystemOrganization addCategory: #'Tweak-Tiles-Inspector'!
SystemOrganization addCategory: #'Tweak-Tiles-Types'!
SystemOrganization addCategory: #'Tweak-Tiles-Definition'!
SystemOrganization addCategory: #'Tweak-Tiles-Players'!
SystemOrganization addCategory: #'Tweak-Tiles-Costumes'!
SystemOrganization addCategory: #'Tweak-Tiles-Viewer'!
SystemOrganization addCategory: #'Tweak-Tiles-ValueCostumes'!

!CObject methodsFor: '*Tweak-Tiles' stamp: 'ar 3/6/2004 15:53'!
valueType
	^self value class! !

CButton subclass: #CTileButton
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tweak-Tiles-Costumes'!

CTileButton defineFields: '<?xml  version="1.0" ?>
<fields>
</fields>'!

!CTileButton methodsFor: 'initialize' stamp: 'ar 3/14/2004 21:13'!
defaultCostume
	^CTileButtonCostume! !

CHolder subclass: #CTileViewerPage
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tweak-Tiles-Viewer'!

CTileViewerPage defineFields: '<?xml  version="1.0" ?>
<fields>
	<field toGet="target" toSet="target:" changeEvent="targetChanged">target</field>
	<field toGet="definition" toSet="definition:" changeEvent="definitionChanged">definition</field>
</fields>'!

!CTileViewerPage methodsFor: 'adding' stamp: 'ar 6/21/2005 19:56'!
contents: items
	self removeAll.
	self addAll: (items collect:[:each| 
		each isTilePlayer 
			ifFalse:[each] "leave non-tiles alone"
			ifTrue:[CViewerRow new
			messageTile: each; 
			target: target;
			yourself]]).! !

!CTileViewerPage methodsFor: 'accessing' stamp: 'bf 3/7/2005 18:24'!
definition
	"Answer the definition of the receiver"
	<bewareOf: #definitionChanged>
	^self propertyValueAt: #definition! !

!CTileViewerPage methodsFor: 'accessing' stamp: 'bf 3/7/2005 18:24'!
definition: aValue
	"Modify the receiver's definition"
	^self propertyValueAt: #definition put: aValue with: #definitionChanged! !

!CTileViewerPage methodsFor: 'initialize' stamp: 'bf 3/7/2005 17:59'!
setupCostume
	super setupCostume.
	self layout: CTableLayout new.
	self hResizing: #spaceFill; vResizing: #spaceFill.
	self layout cellPositioning: #topLeft.

	self contents: (definition items collect: [:each| each newFor: target])! !

!CTileViewerPage methodsFor: 'accessing' stamp: 'bf 3/7/2005 17:57'!
target
	"Answer the target of the receiver"
	<bewareOf: #targetChanged>
	^self propertyValueAt: #target! !

!CTileViewerPage methodsFor: 'accessing' stamp: 'bf 3/7/2005 17:57'!
target: aValue
	"Modify the receiver's target"
	^self propertyValueAt: #target put: aValue with: #targetChanged! !

!Number class methodsFor: '*Tweak-Tiles-Types' stamp: 'ar 1/23/2005 18:50'!
defaultValue
	^1! !

!Metaclass methodsFor: '*Tweak-Tiles-Types' stamp: 'bf 6/23/2005 19:33'!
canCoerceType: aType
	^aType includesBehavior: self! !

!Metaclass methodsFor: '*Tweak-Tiles-Types' stamp: 'bf 7/31/2005 17:33'!
defaultValue
	^self soleInstance! !

!Metaclass methodsFor: '*Tweak-Tiles-Types' stamp: 'bf 6/23/2005 19:20'!
read: aString
	"Read a textual representation of some value"

	aString = self soleInstance name
		ifTrue: [^self soleInstance].
	^nil! !

!Metaclass methodsFor: '*Tweak-Tiles-Types' stamp: 'bf 6/23/2005 19:21'!
valueTileMaker
	"the tile class used for representing objects of our type"
	^CValueTile! !

!CPlayer methodsFor: '*Tweak-Tiles' stamp: 'ar 4/4/2006 21:24'!
addVariable
	"The user requested that a variable be added.  Put up a dialog requesting a name, reduce it if necessary to a properly-formatted and unused name, and then actually add the variable."

	| usedNames aName reply |
	usedNames := self class classFields collect: [:f | f name]
		thenSelect: [:nom | nom beginsWith: 'var'].
	aName := Utilities keyLike: 'var1'  satisfying: [:aKey | (usedNames includes: aKey) not].
	reply := self request: 'Variable name? ' translated initialAnswer: aName.
	reply ifNotEmpty: [self addVariableNamed: reply]! !

!CPlayer methodsFor: '*Tweak-Tiles' stamp: 'pe 12/6/2005 17:01'!
addVariableNamed: aName

	self addVariableNamed: aName type: CNumberType.! !

!CPlayer methodsFor: '*Tweak-Tiles' stamp: 'bf 12/9/2005 10:23'!
addVariableNamed: aName type: variableType
	"The user requested that a variable of the given name be added.  Adjust the name as necessary, then actually add the variable."

	^self addVariableNamed: aName type: variableType default: variableType defaultValue! !

!CPlayer methodsFor: '*Tweak-Tiles' stamp: 'bf 12/9/2005 10:22'!
addVariableNamed: aName type: variableType default: defaultValue
	"The user requested that a variable of the given name be added.  Adjust the name as necessary, then actually add the variable."

	| fieldDef uniqueName existing |
	existing := Utilities authorInitialsPerSe.  Utilities setAuthorInitials: 'auto'.
	[
		uniqueName := self uniqueInstanceVariableNameLike: aName excluding: #(#self #true #false #nil #super).
		self becomeUniClass.  self signal: #nameChanged.
		fieldDef := CFieldDefinition new.
		fieldDef name: uniqueName.
		fieldDef toGet: uniqueName asSymbol.
		fieldDef toSet: (uniqueName, ':') asSymbol.
		fieldDef type: variableType.

		self class addField: fieldDef.
		self class compilePropertyFor: uniqueName defaultValue: defaultValue.
		self signal: #variableModified withArguments: {uniqueName}.
	] ensure:[Utilities setAuthorInitials: (existing ifNil: [''])].
! !

!CPlayer methodsFor: '*Tweak-Tiles' stamp: 'sw 5/17/2005 04:54'!
numericValue
	"Answer the numeric value associated with the receiver"

	| aValue aString |
	(aValue := self userDataAt: #numericValue ifAbsent: [nil]) ifNotNil: [^ aValue].

	(aValue := self value) isNumber ifTrue: [^ aValue].
	(aString := text asString) isEmptyOrNil ifTrue: [^ 0].
	
	^ [aString asNumber] ifError: [:a :b | ^ aString asInteger ifNil: [0]]! !

!CPlayer methodsFor: '*Tweak-Tiles' stamp: 'sw 5/17/2005 04:52'!
numericValue: aNumber
	"Answer the numeric value associated with the receiver"

	self userDataAt: #numericValue put: aNumber! !

!CPlayer methodsFor: '*Tweak-Tiles' stamp: 'ar 6/27/2005 13:50'!
tileForSelf
	"Answer a tile representing the receiver"

	| aTile |
	aTile := CValueTile new.
	aTile valueType: CPlayerType.
	aTile hResizing: #shrinkWrap.
	aTile vResizing: #shrinkWrap.
	aTile value: self.
	^ aTile
! !

CPlayer subclass: #CScriptorHeader
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tweak-Tiles-Costumes'!

CScriptorHeader defineFields: '<?xml  version="1.0" ?>
<fields>
	<field toGet="fireButton" toSet="fireButton:" changeEvent="fireButtonChanged">fireButton</field>
	<field toGet="collapseButton" toSet="collapseButton:" changeEvent="collapseButtonChanged">collapseButton</field>
	<field toGet="rcvrLabel" toSet="rcvrLabel:" changeEvent="rcvrLabelChanged">rcvrLabel</field>
	<field toGet="scriptName" toSet="scriptName:" changeEvent="scriptNameChanged">scriptName</field>
	<field toGet="ticker" toSet="ticker:" changeEvent="tickerChanged">ticker</field>
	<field toGet="trigger" toSet="trigger:" changeEvent="triggerChanged">trigger</field>
	<field toGet="closeButton" toSet="closeButton:" changeEvent="closeButtonChanged">closeButton</field>
</fields>'!

!CScriptorHeader methodsFor: 'accessing' stamp: 'ar 3/12/2004 16:25'!
closeButton
	"Answer the closeButton of the receiver"
	<bewareOf: #closeButtonChanged>
	^self propertyValueAt: #closeButton! !

!CScriptorHeader methodsFor: 'accessing' stamp: 'ar 3/12/2004 16:25'!
closeButton: aValue
	"Modify the receiver's closeButton"
	^self propertyValueAt: #closeButton put: aValue with: #closeButtonChanged! !

!CScriptorHeader methodsFor: 'accessing' stamp: 'ar 3/12/2004 16:24'!
collapseButton
	"Answer the collapseButton of the receiver"
	<bewareOf: #collapseButtonChanged>
	^self propertyValueAt: #collapseButton! !

!CScriptorHeader methodsFor: 'accessing' stamp: 'ar 3/12/2004 16:24'!
collapseButton: aValue
	"Modify the receiver's collapseButton"
	^self propertyValueAt: #collapseButton put: aValue with: #collapseButtonChanged! !

!CScriptorHeader methodsFor: 'accessing' stamp: 'ar 3/12/2004 16:24'!
fireButton
	"Answer the fireButton of the receiver"
	<bewareOf: #fireButtonChanged>
	^self propertyValueAt: #fireButton! !

!CScriptorHeader methodsFor: 'accessing' stamp: 'ar 3/12/2004 16:24'!
fireButton: aValue
	"Modify the receiver's fireButton"
	^self propertyValueAt: #fireButton put: aValue with: #fireButtonChanged! !

!CScriptorHeader methodsFor: 'initialize' stamp: 'ar 4/19/2005 19:14'!
initialize
	super initialize.
	self define: #fireButton as: CWindowButton new.
	self define: #collapseButton as: CWindowButton new.
	self define: #rcvrLabel as: CInputField new.
	self define: #scriptName as: CInputField new.
	self define: #ticker as: CTickIndicator new.
	self define: #trigger as: CInputField new.
	self define: #closeButton as: CWindowButton new.
! !

!CScriptorHeader methodsFor: 'initialize' stamp: 'ar 12/19/2004 17:12'!
makeTexture: spacer color: aColor
	| form bb |
	form := ColorForm extent: 32@32 depth: 2.
	form colors: {Color transparent. Color white. aColor. aColor mixed: 0.8 with: Color white}.
	form pixelValueAt: 0@0 put: 2.
	form pixelValueAt: 0@1 put: 1.
	form pixelValueAt: 2@2 put: 2.
	form pixelValueAt: 2@3 put: 1.
	"Note: If necessary (i.e., slow) the following can be improved by copying increasingly more bits at a time, e.g., in the first run we copy 2 px then 4, then 8 etc. The same holds, of course, for the vertical loop."
	bb := BitBlt toForm: form.
	bb combinationRule: Form over.
	bb sourceForm: form.
	bb sourceX: 0; sourceY: 0; width: 4; height: 4.
	bb destX: 0; destY: 0.
	4 to: form width by: 4 do:[:xx|bb destX: xx; copyBits].
	bb sourceX: 0; sourceY: 0; width: form width; height: 4.
	bb destX: 0; destY: 0.
	4 to: form height by: 4 do:[:yy|bb destY: yy; copyBits].
	spacer graphicForm := form.
	spacer graphicFit := #tile.! !

!CScriptorHeader methodsFor: 'accessing' stamp: 'ar 3/12/2004 16:24'!
rcvrLabel
	"Answer the rcvrLabel of the receiver"
	<bewareOf: #rcvrLabelChanged>
	^self propertyValueAt: #rcvrLabel! !

!CScriptorHeader methodsFor: 'accessing' stamp: 'ar 3/12/2004 16:24'!
rcvrLabel: aValue
	"Modify the receiver's rcvrLabel"
	^self propertyValueAt: #rcvrLabel put: aValue with: #rcvrLabelChanged! !

!CScriptorHeader methodsFor: 'accessing' stamp: 'ar 3/12/2004 16:24'!
scriptName
	"Answer the scriptName of the receiver"
	<bewareOf: #scriptNameChanged>
	^self propertyValueAt: #scriptName! !

!CScriptorHeader methodsFor: 'accessing' stamp: 'ar 3/12/2004 16:24'!
scriptName: aValue
	"Modify the receiver's scriptName"
	^self propertyValueAt: #scriptName put: aValue with: #scriptNameChanged! !

!CScriptorHeader methodsFor: 'initialize' stamp: 'ar 4/19/2005 19:14'!
setupCostume
	| leftSpacer rightSpacer |
	super setupCostume.
	self extent: 2@2.
	self layout: CTableLayout new.
	self hResizing: #spaceFill; vResizing: #shrinkWrap.
	self layout listDirection: #leftToRight.
	self layout cellInset: 2@2.

	self fillStyle: #radialGradient.
	self fillColors: CWidgetLibrary default gradientColors.
	self layout layoutInset: 3.

	fireButton font: CTileLibrary default tileFont.
	fireButton textEmphasis: 1.
	fireButton label: '!!'.
	self add: fireButton.

	collapseButton font: (TextStyle named: 'WindowFont') defaultFont.
	collapseButton label: 'O'.
	self add: collapseButton.

	leftSpacer := CPlayer new.
	leftSpacer hResizing: #rigid; vResizing: #rigid.
	leftSpacer extent: 16@16.
	self makeTexture: leftSpacer color: CWidgetLibrary default baseColor.
	self add: leftSpacer.


	{rcvrLabel. scriptName. trigger} do:[:label|
		label allowEditing: false. "only after dbl click"
		label font: CTileLibrary default tileFont.
		label color: CWidgetLibrary default paneColor.
		label borderWidth: 1.
		label borderColor: CWidgetLibrary default frameColor.
		label borderStyle: #simple.
		label textInset: 5@3.
		label roundCorners: true.
		label hResizing: #shrinkWrap.
	].

	self add: rcvrLabel.
	scriptName allowEditing: true.
	scriptName autoRevert: true.
	self add: scriptName.

	rightSpacer := CPlayer new.
	rightSpacer hResizing: #spaceFill; vResizing: #rigid.
	rightSpacer extent: 16@16.
	rightSpacer graphic: leftSpacer graphic.
	rightSpacer asPrimCostume minWidth: 32.
	self add: rightSpacer.

	ticker extent: 20@20.
	self add: ticker.
	trigger value: 'normal'.
	self add: trigger.

	closeButton font: (TextStyle named: 'WindowFont') defaultFont.
	closeButton label: 'x'.
	self add: closeButton.! !

!CScriptorHeader methodsFor: 'accessing' stamp: 'ar 3/12/2004 16:24'!
ticker
	"Answer the ticker of the receiver"
	<bewareOf: #tickerChanged>
	^self propertyValueAt: #ticker! !

!CScriptorHeader methodsFor: 'accessing' stamp: 'ar 3/12/2004 16:24'!
ticker: aValue
	"Modify the receiver's ticker"
	^self propertyValueAt: #ticker put: aValue with: #tickerChanged! !

!CScriptorHeader methodsFor: 'accessing' stamp: 'ar 3/12/2004 16:24'!
trigger
	"Answer the trigger of the receiver"
	<bewareOf: #triggerChanged>
	^self propertyValueAt: #trigger! !

!CScriptorHeader methodsFor: 'accessing' stamp: 'ar 3/12/2004 16:24'!
trigger: aValue
	"Modify the receiver's trigger"
	^self propertyValueAt: #trigger put: aValue with: #triggerChanged! !

CPlayer subclass: #CSeparatorTile
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tweak-Tiles-Players'!

CSeparatorTile defineFields: '<?xml  version="1.0" ?>
<fields>
</fields>'!

!CSeparatorTile methodsFor: 'testing' stamp: 'ar 5/12/2005 17:25'!
isSeparator
	^true! !

!CSeparatorTile methodsFor: 'initialize' stamp: 'ar 5/12/2005 17:26'!
setupCostume
	super setupCostume.
	self extent: 2@5.
	self color: CWidgetLibrary default baseColor.
	self borderWidth := 2.
	self hResizing: #spaceFill; vResizing: #rigid.
! !

CPlayer subclass: #CTickIndicator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tweak-Tiles-Costumes'!

CTickIndicator defineFields: '<?xml  version="1.0" ?>
<fields>
	<field toGet="tickTime" toSet="tickTime:" changeEvent="tickTimeChanged">tickTime</field>
	<field toGet="ticking" toSet="ticking:" changeEvent="tickingChanged">ticking</field>
	<field toGet="lastTick" toSet="lastTick:" changeEvent="lastTickChanged">lastTick</field>
	<field toGet="tickIndex" toSet="tickIndex:" changeEvent="tickIndexChanged">tickIndex</field>
	<field toGet="corners" toSet="corners:" changeEvent="cornersChanged">corners</field>
</fields>'!

!CTickIndicator methodsFor: 'accessing' stamp: 'ar 4/19/2005 18:55'!
corners
	"Answer the corners of the receiver"
	<bewareOf: #cornersChanged>
	^self propertyValueAt: #corners! !

!CTickIndicator methodsFor: 'accessing' stamp: 'ar 4/19/2005 18:55'!
corners: aValue
	"Modify the receiver's corners"
	^self propertyValueAt: #corners put: aValue with: #cornersChanged! !

!CTickIndicator methodsFor: 'initialize' stamp: 'ar 4/19/2005 19:19'!
defaultColor
	^CWidgetLibrary default frameColor.! !

!CTickIndicator methodsFor: 'drawing' stamp: 'ar 4/19/2005 19:06'!
drawOn: aCanvas in: drawingBounds
	| r centerPt cc deg box |
	aCanvas fillOval: drawingBounds fill: self fill border: self border.
	box := 0@0 extent: self extent.
	corners ifNil:[
		r := (box topCenter - box center) r - 2.
		corners := Array new: 32.
		1 to: corners size do:[:i|
			deg := 360.0 / corners size * (i-1).
			corners at: i put: (Point r: r degrees: deg-90) asIntegerPoint]].
	tickIndex := tickIndex \\ corners size.
	cc := color darker.
	centerPt := box center.
	1 to: corners size by: 4 do:[:i|
		aCanvas fillRectangle: (centerPt + (corners at: i)-2  extent: 4@4) color: cc.
	].
	cc := cc darker.
	aCanvas line: centerPt to: centerPt + (corners at: tickIndex + 1) width: 2 color: cc.! !

!CTickIndicator methodsFor: 'initialize' stamp: 'ar 4/19/2005 19:02'!
initialize
	super initialize.
	ticking := false.
	tickTime := 125.
	lastTick := 0.
	tickIndex := 0.
	corners := nil.
! !

!CTickIndicator methodsFor: 'accessing' stamp: 'ar 4/19/2005 18:53'!
lastTick
	"Answer the lastTick of the receiver"
	<bewareOf: #lastTickChanged>
	^self propertyValueAt: #lastTick! !

!CTickIndicator methodsFor: 'accessing' stamp: 'ar 4/19/2005 18:53'!
lastTick: aValue
	"Modify the receiver's lastTick"
	^self propertyValueAt: #lastTick put: aValue with: #lastTickChanged! !

!CTickIndicator methodsFor: 'events' stamp: 'ar 4/19/2005 19:09'!
onExtentChanged
	<on: extentChanged>
	corners := nil.
! !

!CTickIndicator methodsFor: 'events' stamp: 'ar 4/19/2005 19:11'!
onFillChanged
	<on: fillChanged>
	self borderColor := color darker.! !

!CTickIndicator methodsFor: 'events' stamp: 'ar 4/19/2005 18:55'!
onTick
	"No trigger here - we set the ticking rate programmatically"
	| delta nowTick |
	nowTick := Time millisecondClockValue.
	(lastTick isNil or: [nowTick < lastTick]) ifTrue: [lastTick := nowTick].
	delta := (nowTick - lastTick) // tickTime.
	delta > 0 ifFalse:[^self].
	tickIndex := tickIndex + delta.
	lastTick := nowTick.
	self invalidate.! !

!CTickIndicator methodsFor: 'events' stamp: 'bf 5/4/2005 19:21'!
onTickingChanged
	<on: tickingChanged>
	self stopScript: #onTick.
	ticking ifTrue:[
		self startTicking: #onTick at: 50.
		self world ifNil:[self pauseScript: #onTick].
	].! !

!CTickIndicator methodsFor: 'initialize' stamp: 'ar 4/19/2005 19:19'!
setupCostume
	super setupCostume.
	self extent: 20@20.
	self userDraw: true.
	self color := self defaultColor.
	self borderStyle: #simple.
	self borderWidth: 2.
	self borderColor: color darker.! !

!CTickIndicator methodsFor: 'accessing' stamp: 'ar 4/19/2005 18:54'!
tickIndex
	"Answer the tickIndex of the receiver"
	<bewareOf: #tickIndexChanged>
	^self propertyValueAt: #tickIndex! !

!CTickIndicator methodsFor: 'accessing' stamp: 'ar 4/19/2005 18:54'!
tickIndex: aValue
	"Modify the receiver's tickIndex"
	^self propertyValueAt: #tickIndex put: aValue with: #tickIndexChanged! !

!CTickIndicator methodsFor: 'accessing' stamp: 'ar 4/19/2005 18:53'!
tickTime
	"Answer the tickTime of the receiver"
	<bewareOf: #tickTimeChanged>
	^self propertyValueAt: #tickTime! !

!CTickIndicator methodsFor: 'accessing' stamp: 'ar 4/19/2005 18:53'!
tickTime: aValue
	"Modify the receiver's tickTime"
	^self propertyValueAt: #tickTime put: aValue with: #tickTimeChanged! !

!CTickIndicator methodsFor: 'accessing' stamp: 'ar 4/19/2005 18:59'!
ticking
	"Answer the ticking of the receiver"
	<bewareOf: #tickingChanged>
	^self propertyValueAt: #ticking! !

!CTickIndicator methodsFor: 'accessing' stamp: 'ar 4/19/2005 18:59'!
ticking: aValue
	"Modify the receiver's ticking"
	^self propertyValueAt: #ticking put: aValue with: #tickingChanged! !

CPlayer subclass: #CTileDefinition
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tweak-Tiles-Definition'!

!CTileDefinition commentStamp: 'tk 1/20/2005 10:59' prior: 0!
Tiles are used as statements in scripts.

externalName    in natural language, as appears on the tile (in a foreign language)
type 		type of the result
helpText 	a string
selector		

Note that the receiver is not here.  It is supplied when the actual tiles are created (newFor:).  Info about which tiles are for which class of receiver is carried in the XML in the category header.

Note that I act like a CObject, and not like a CPlayer.  I am not put on the screen.  When the Player that owns me is duplicated, veryDeepCopy should duplicate me also.  It does, because my container is nil.!

CTileDefinition subclass: #CArgumentDefinition
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tweak-Tiles-Definition'!

CArgumentDefinition defineFields: '<?xml  version="1.0" ?>
<fields>
	<field toGet="defaultValue" toSet="defaultValue:" changeEvent="defaultValueChanged">defaultValue</field>
</fields>'!

!CArgumentDefinition methodsFor: 'accessing' stamp: 'ar 3/21/2004 16:21'!
defaultValue
	"Answer the defaultValue of the receiver"
	<bewareOf: #defaultValueChanged>
	^self propertyValueAt: #defaultValue! !

!CArgumentDefinition methodsFor: 'accessing' stamp: 'ar 3/21/2004 16:21'!
defaultValue: aValue
	"Modify the receiver's defaultValue"
	^self propertyValueAt: #defaultValue put: aValue with: #defaultValueChanged! !

!CArgumentDefinition methodsFor: 'creating' stamp: 'ar 5/12/2005 19:51'!
new
	| tile expr |
	expr := type valueTileMaker new.
	expr valueType: type.
	expr value: defaultValue.

	tile := CVariableTile new.
	tile valueType: type.
	tile name: (externalName ifNil:['']).
	tile expression: expr.
	tile balloonHelp: helpText.
	^tile! !

!CArgumentDefinition methodsFor: 'creating' stamp: 'ar 3/21/2004 17:43'!
newFor: anObject
	^self new! !

!CArgumentDefinition methodsFor: 'printing' stamp: 'tk 11/19/2004 14:34'!
printXmlOn: xmlWriter

	self printXmlOn: xmlWriter tag: 'argument'.
! !

!CArgumentDefinition methodsFor: 'printing' stamp: 'tk 11/19/2004 14:34'!
printXmlOn: xmlWriter tag: aTag

	xmlWriter startTag: aTag.
	externalName ifNotNil:[xmlWriter attribute: 'name' value: externalName].
	type ifNotNil:[xmlWriter attribute: 'type' value: type name].
	defaultValue ifNotNil:[xmlWriter attribute: 'defaultValue' value: (type print: defaultValue)].
	helpText ifNotNil:[xmlWriter attribute: 'help' value: helpText].
	xmlWriter endEmptyTag: aTag.! !

!CArgumentDefinition methodsFor: 'printing' stamp: 'tk 11/19/2004 14:53'!
readXmlFrom: xmlNode
	| attr |
	(xmlNode name == #argument or: [xmlNode name == #receiver]) 
		ifFalse:[^self error: 'Not an argument'].
	attr := xmlNode attributeAt: 'name'.
	attr ifNotNil:[externalName := attr].
	attr := xmlNode attributeAt: 'type'.
	attr ifNotNil:[type := self typeNamed: attr].
	attr := xmlNode attributeAt: 'default'.
	attr ifNotNil:[defaultValue := type read: attr].
	attr := xmlNode attributeAt: 'help'.
	attr ifNotNil:[helpText := attr].
	^self! !

CTileDefinition subclass: #CCategoryDefinition
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tweak-Tiles-Definition'!

CCategoryDefinition defineFields: '<?xml  version="1.0" ?>
<fields>
	<field toGet="items" toSet="items:" changeEvent="itemsChanged">items</field>
</fields>'!

!CCategoryDefinition methodsFor: 'initialize' stamp: 'ar 3/21/2004 17:05'!
initialize
	super initialize.
	self define: #items as: CCollection new.! !

!CCategoryDefinition methodsFor: 'accessing' stamp: 'ar 3/21/2004 17:01'!
items
	"Answer the items of the receiver"
	<bewareOf: #itemsChanged>
	^self propertyValueAt: #items! !

!CCategoryDefinition methodsFor: 'accessing' stamp: 'ar 3/21/2004 17:01'!
items: aValue
	"Modify the receiver's items"
	^self propertyValueAt: #items put: aValue with: #itemsChanged! !

!CCategoryDefinition methodsFor: 'creating' stamp: 'bf 3/7/2005 17:54'!
new
	| tile |
	tile := CTileViewerPage new.
	tile name: externalName.
	tile definition: self.
	^tile
! !

!CCategoryDefinition methodsFor: 'creating' stamp: 'bf 3/7/2005 17:57'!
newFor: anObject
	| tile |
	tile := CTileViewerPage new.
	tile name: externalName.
	tile target: anObject.
	tile definition: self.
	^tile! !

!CCategoryDefinition methodsFor: 'printing' stamp: 'ar 3/21/2004 17:25'!
printXmlOn: xmlWriter
	xmlWriter startTag: 'category'.
	selector ifNotNil:[xmlWriter attribute: 'selector' value: selector].
	externalName ifNotNil:[xmlWriter attribute: 'name' value: externalName].
	type ifNotNil:[xmlWriter attribute: 'type' value: type name].
	helpText ifNotNil:[xmlWriter attribute: 'help' value: helpText].
	xmlWriter endTag.
	items do:[:each| each printXmlOn: xmlWriter].
	xmlWriter endTag: 'category'.! !

!CCategoryDefinition methodsFor: 'printing' stamp: 'ar 3/21/2004 17:03'!
readXmlFrom: xmlNode
	| attr |
	xmlNode name == #category ifFalse:[^self error: 'Not a category'].
	attr := xmlNode attributeAt: 'selector'.
	attr ifNotNil:[selector := attr asSymbol].
	attr := xmlNode attributeAt: 'name'.
	attr ifNotNil:[externalName := attr].
	attr := xmlNode attributeAt: 'type'.
	attr ifNotNil:[type := self typeNamed: attr].
	attr := xmlNode attributeAt: 'help'.
	attr ifNotNil:[helpText := attr].
	xmlNode elements do:[:each|
		items add: (CTileDefinition xmlFrom: each).
	].
	^self! !

CTileDefinition subclass: #CMessageDefinition
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tweak-Tiles-Definition'!

CMessageDefinition defineFields: '<?xml  version="1.0" ?>
<fields>
	<field toGet="arguments" toSet="arguments:" changeEvent="argumentsChanged">arguments</field>
</fields>'!

!CMessageDefinition methodsFor: 'accessing' stamp: 'ar 3/21/2004 16:17'!
arguments
	"Answer the arguments of the receiver"
	<bewareOf: #argumentsChanged>
	^self propertyValueAt: #arguments! !

!CMessageDefinition methodsFor: 'accessing' stamp: 'ar 3/21/2004 16:17'!
arguments: aValue
	"Modify the receiver's arguments"
	^self propertyValueAt: #arguments put: aValue with: #argumentsChanged! !

!CMessageDefinition methodsFor: 'initialize' stamp: 'ar 3/21/2004 16:45'!
initialize
	super initialize.
	self define: #arguments as: CCollection new.! !

!CMessageDefinition methodsFor: 'creating' stamp: 'ar 5/12/2005 20:07'!
new
	"Create a new tile for the given object"
	| tile |
	tile := CMessageTile new.
	tile valueType: type.
	tile selector: selector.
	tile name: (externalName ifNil:[selector]).
	tile displayName: (externalName ifNil:[selector]).
	tile arguments: (arguments collect:[:each| each new]).
	tile definition: self.
	tile balloonHelp: helpText.
	^tile! !

!CMessageDefinition methodsFor: 'creating' stamp: 'tak 5/26/2005 17:45'!
newFor: anObject
	"Create a new tile for the given object"
	| tile |
	tile := CMessageTile new.
	tile receiver: anObject.
	tile valueType: type.
	tile selector: selector.
	tile name: (externalName ifNil:[selector]).
	tile displayName: (externalName ifNil:[selector]) translated.
	tile arguments: (arguments collect:[:each| each new]).
	tile definition: self.
	tile balloonHelp: (helpText ifNotNil: [helpText translated]).
	^tile! !

!CMessageDefinition methodsFor: 'printing' stamp: 'ar 3/21/2004 17:25'!
printXmlOn: xmlWriter
	xmlWriter startTag: 'message'.
	selector ifNotNil:[xmlWriter attribute: 'selector' value: selector].
	externalName ifNotNil:[xmlWriter attribute: 'name' value: externalName].
	type ifNotNil:[xmlWriter attribute: 'type' value: type name].
	helpText ifNotNil:[xmlWriter attribute: 'help' value: helpText].
	arguments size = 0 ifTrue:[^xmlWriter endEmptyTag: 'message'].
	xmlWriter endTag.
	arguments do:[:each| each printXmlOn: xmlWriter].
	xmlWriter endTag: 'message'.! !

!CMessageDefinition methodsFor: 'printing' stamp: 'tk 2/22/2005 15:46'!
readXmlFrom: xmlNode
	| attr |
	xmlNode name == #message ifFalse:[^self error: 'Not a message'].
	attr := xmlNode attributeAt: 'selector'.
	attr ifNotNil:[selector := attr asSymbol].
	attr := xmlNode attributeAt: 'name'.
	attr ifNotNil:[externalName := attr].
	attr := xmlNode attributeAt: 'type'.
	attr ifNotNil:[type := self typeNamed: attr].
	attr := xmlNode attributeAt: 'help'.
	attr ifNotNil:[helpText := attr].
	xmlNode elements do:[:each|
		each name == #argument ifTrue: [
			arguments add: (CArgumentDefinition new readXmlFrom: each)].
		"Should we allow a receiver, in order to hold a default value?"
		"each name == #receiver ifTrue: [
			receiver := (CArgumentDefinition new readXmlFrom: each)]."
		].
	^self! !

CTileDefinition subclass: #COperatorDefinition
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tweak-Tiles-Definition'!

!COperatorDefinition commentStamp: 'tk 11/19/2004 14:30' prior: 0!
The receiver inst var is only for operators that are sent to non-Players, like integers.  Since (1+1) does not have a Player as the receiver, we need to describe what to put in as the receiver.  It is an ArgumentTile, but used as the receiver of a MessageTile.!

COperatorDefinition defineFields: '<?xml  version="1.0" ?>
<fields>
	<field toGet="receiver" toSet="receiver:" changeEvent="receiverChanged">receiver</field>
	<field toGet="arguments" toSet="arguments:" changeEvent="argumentsChanged">arguments</field>
</fields>'!

!COperatorDefinition methodsFor: 'accessing' stamp: 'ar 3/22/2004 16:07'!
arguments
	"Answer the arguments of the receiver"
	<bewareOf: #argumentsChanged>
	^self propertyValueAt: #arguments! !

!COperatorDefinition methodsFor: 'accessing' stamp: 'ar 3/22/2004 16:07'!
arguments: aValue
	"Modify the receiver's arguments"
	^self propertyValueAt: #arguments put: aValue with: #argumentsChanged! !

!COperatorDefinition methodsFor: 'initialize' stamp: 'ar 3/22/2004 16:07'!
initialize
	super initialize.
	self define: #arguments as: CCollection new.! !

!COperatorDefinition methodsFor: 'creating' stamp: 'bf 7/22/2005 19:48'!
new
	"Create a new tile"
	| tile |
	tile := COperatorTile new.
	tile receiver: receiver new.
	tile valueType: type.
	tile selector: selector.
	tile name: (externalName ifNil:[selector]).
	tile displayName: (externalName ifNil:[selector]).
	tile arguments: (arguments collect:[:each| each new]).
	tile definition: self.
	tile balloonHelp: helpText.
	^tile! !

!COperatorDefinition methodsFor: 'creating' stamp: 'bf 7/22/2005 17:57'!
newFor: anObject
	^self new! !

!COperatorDefinition methodsFor: 'printing' stamp: 'tk 11/19/2004 14:36'!
printXmlOn: xmlWriter
	xmlWriter startTag: 'operator'.
	selector ifNotNil:[xmlWriter attribute: 'selector' value: selector].
	externalName ifNotNil:[xmlWriter attribute: 'name' value: externalName].
	type ifNotNil:[xmlWriter attribute: 'type' value: type name].
	helpText ifNotNil:[xmlWriter attribute: 'help' value: helpText].
	xmlWriter endTag.
	receiver ifNotNil: [receiver printXmlOn: xmlWriter tag: 'receiver'].
	arguments do:[:each| each printXmlOn: xmlWriter].
	xmlWriter endTag: 'operator'.! !

!COperatorDefinition methodsFor: 'printing' stamp: 'bf 2/21/2005 14:05'!
readXmlFrom: xmlNode
	| attr |
	xmlNode name == #operator ifFalse:[^self error: 'Not an operator'].
	attr := xmlNode attributeAt: 'selector'.
	attr ifNotNil:[selector := attr asSymbol].
	attr := xmlNode attributeAt: 'name'.
	attr ifNotNil:[externalName := attr].
	attr := xmlNode attributeAt: 'type'.
	attr ifNotNil:[type := self typeNamed: attr].
	attr := xmlNode attributeAt: 'help'.
	attr ifNotNil:[helpText := attr].
	xmlNode elements do:[:each|
		each name == #receiver ifTrue: [
			receiver := (CArgumentDefinition new readXmlFrom: each)].
		each name == #argument ifTrue: [
			arguments add: (CArgumentDefinition new readXmlFrom: each)].
	].
	^self! !

!COperatorDefinition methodsFor: 'accessing' stamp: 'tk 11/19/2004 14:16'!
receiver
	"Answer the receiver of the receiver"
	<bewareOf: #receiverChanged>
	^self propertyValueAt: #receiver! !

!COperatorDefinition methodsFor: 'accessing' stamp: 'tk 11/19/2004 14:16'!
receiver: aValue
	"Modify the receiver's receiver"
	^self propertyValueAt: #receiver put: aValue with: #receiverChanged! !

CTileDefinition subclass: #CSeparatorDefinition
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tweak-Tiles-Definition'!

CSeparatorDefinition defineFields: '<?xml  version="1.0" ?>
<fields>
</fields>'!

!CSeparatorDefinition methodsFor: 'creating' stamp: 'ar 6/21/2005 19:57'!
new
	^CSeparatorTile new! !

!CSeparatorDefinition methodsFor: 'creating' stamp: 'ar 5/12/2005 17:19'!
newFor: anObject
	^self new! !

!CSeparatorDefinition methodsFor: 'printing' stamp: 'ar 5/12/2005 17:18'!
printXmlOn: xmlWriter
	xmlWriter startTag: 'separator'.
	xmlWriter endEmptyTag: 'separator'.! !

!CSeparatorDefinition methodsFor: 'printing' stamp: 'ar 5/12/2005 17:18'!
readXmlFrom: xmlNode
	xmlNode name == #separator ifFalse:[^self error: 'Not a separator'].
	^self! !

CTileDefinition subclass: #CSlotDefinition
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tweak-Tiles-Definition'!

CSlotDefinition defineFields: '<?xml  version="1.0" ?>
<fields>
</fields>'!

!CSlotDefinition methodsFor: 'creating' stamp: 'ar 5/12/2005 20:08'!
new
	| tile |
	tile := CPropertyTile new.
	tile valueType: type.
	tile property: selector.
	tile name: (externalName ifNil:[selector]).
	tile displayName: (externalName ifNil:[selector]).
	tile definition: self.
	tile balloonHelp: helpText.
	^tile! !

!CSlotDefinition methodsFor: 'creating' stamp: 'ar 6/23/2005 14:25'!
newFor: anObject
	| tile |
	tile := CPropertyTile new.
	tile receiver: anObject.
	tile valueType: (type asConcreteTypeFor: anObject)..
	tile property: selector.
	tile name: externalName.
	tile displayName: (externalName ifNil:[selector]) translated.
	tile definition: self.
	tile balloonHelp: helpText.
	^tile! !

!CSlotDefinition methodsFor: 'printing' stamp: 'ar 3/21/2004 16:41'!
printXmlOn: xmlWriter
	xmlWriter startTag: 'slot'.
	selector ifNotNil:[xmlWriter attribute: 'selector' value: selector].
	externalName ifNotNil:[xmlWriter attribute: 'name' value: externalName].
	type ifNotNil:[xmlWriter attribute: 'type' value: type name].
	helpText ifNotNil:[xmlWriter attribute: 'help' value: helpText].
	xmlWriter endEmptyTag: 'slot'.! !

!CSlotDefinition methodsFor: 'printing' stamp: 'ar 3/21/2004 16:41'!
readXmlFrom: xmlNode
	| attr |
	xmlNode name == #slot ifFalse:[^self error: 'Not an argument'].
	attr := xmlNode attributeAt: 'selector'.
	attr ifNotNil:[selector := attr asSymbol].
	attr := xmlNode attributeAt: 'name'.
	attr ifNotNil:[externalName := attr].
	attr := xmlNode attributeAt: 'type'.
	attr ifNotNil:[type := self typeNamed: attr].
	attr := xmlNode attributeAt: 'help'.
	attr ifNotNil:[helpText := attr].
	^self! !

CTileDefinition defineFields: '<?xml  version="1.0" ?>
<fields>
	<field toGet="externalName" toSet="externalName:" changeEvent="externalNameChanged">externalName</field>
	<field toGet="type" toSet="type:" changeEvent="typeChanged">type</field>
	<field toGet="helpText" toSet="helpText:" changeEvent="helpTextChanged">helpText</field>
	<field toGet="selector" toSet="selector:" changeEvent="selectorChanged">selector</field>
</fields>'!

!CTileDefinition class methodsFor: 'reading' stamp: 'sw 2/14/2005 13:19'!
defaultViewerOn: aPlayer
	"Answer a default viewer for the given player, not yet opened"

	^ (self readXmlFrom: self defaultViewerSpec readStream) first newFor: aPlayer! !

!CTileDefinition class methodsFor: 'reading' stamp: 'bf 8/23/2005 12:33'!
defaultViewerSpec
	"CTileDefinition readXmlFrom: self defaultViewerSpec readStream"
^'
<viewer type="CPlayer">

	<category name="basic">
		<message selector="makeSound:" name="make sound:" help="Make the specified sound">
			<argument name="aSound" type="CSoundNameType" default="croak"/>
		</message>
		<message selector="forwardBy:" name="forward by:" help="Move the object forward into the direction it is heading">
			<argument name="delta" type="CNumberType" default="5"/>
		</message>
		<message selector="turnBy:" name="turn by:" help="Change the angle of the object by the specified amount">
			<argument name="delta" type="CNumberType" default="5"/>
		</message>
		<slot selector="cartesianX" name="x" type="CNumberType" help="The x coordinate"/>
		<slot selector="cartesianY" name="y" type="CNumberType" help="The y coordinate"/>
		<slot selector="heading" name="heading" type="CNumberType" help="The object''s heading in degrees"/>
	</category>

	<category name="basic" type="CButton">
		<slot selector="label" type="CStringType" help="The button label"/>
	</category>

	<category id="scripts" name="scripts">
		<message selector="emptyScript" name="emptyScript" help="The empty script"/>
	</category>

	<category name="variables">
	</category>

	<category name="color list" type="CColorListDemo">
		<slot selector="value" name="value" type="CValueType" help="The value of the object"/>
		<slot selector="cursor" name="cursor" type="CNumberType" help="The index of the currently selected color"/>
	</category>

	<category name="animation" type="CPlayer">
		<slot selector="rotationStyle" name="rotation style" type="CRotationStyleType" help="The rotation style of the object"/>
		<message selector="turnTo:" name="turn to:" help="Turn the object towards the argument">
			<argument name="target" type="CPlayerType" default="nil"/>
		</message>
		<message selector="turnBy:duration:" name="turn by:duration:" help="Turn the object by the specified amount">
			<argument name="amount" type="CNumberType" default="90"/>
			<argument name="duration" type="CNumberType" default="1"/>
		</message>
		<message selector="forwardBy:duration:" name="forward by:duration:" help="Move the object by the specified amount">
			<argument name="amount" type="CNumberType" default="100"/>
			<argument name="duration" type="CNumberType" default="1"/>
		</message>

	</category>

	<category name="animation" type="CSprite">
		<slot selector="cursor" name="cursor" type="CNumberType" help="The index of the currently selected costume"/>
		<widget class="CSpriteCostumeList" label="Frames:"/>
	</category>

	<category name="and or not">
		<operator selector="=" name="=" type="CBooleanType" 
				help="True if the values are equal">
			<receiver name="A" type="CBooleanType" default="true"/>
			<argument name="B" type="CBooleanType" default="true"/>
		</operator>
		<operator selector="&amp;" name="and" type="CBooleanType" 
				help="True if the values are both true">
			<receiver name="A" type="CBooleanType" default="true"/>
			<argument name="B" type="CBooleanType" default="true"/>
		</operator>
		<operator selector="|" name="or" type="CBooleanType" 
				help="True if at least one of the values is true">
			<receiver name="A" type="CBooleanType" default="true"/>
			<argument name="B" type="CBooleanType" default="false"/>
		</operator>
		<message selector="not" name="not" type="CBooleanType" 
				help="The opposite value">
			<receiver name="A" type="CBooleanType" default="true"/>
		</message>
	</category>

	<category name="button" type="CButton">
		<slot selector="label" type="CStringType" help="The button label"/>
		<slot selector="image" type="CImageResourceType" help="The button image"/>
		<slot selector="state" type="CButtonStateType" help="The button state"/>
		<slot selector="selected" type="CBooleanType" help="The selection state"/>
		<slot selector="value" type="CBooleanType" help="The button label"/>
		<message selector="select"/>
		<message selector="deselect"/>
		<message selector="fire" help="fire the button"/>
		<message selector="enable"/>
		<message selector="disable"/>
	</category>

	<category name="color and border">
		<message selector="color:sees:" name="color:sees:" type="CBooleanType">
			<argument name="myColor" type="CColorType" default="blue"/>
			<argument name="aColor" type="CColorType" default="red"/>
		</message>
		<slot selector="color" name="color" type="CColorType"/>
		<slot selector="fillStyle" name="fillStyle" type="CFillStyleType"/>

		<slot selector="borderWidth" name="borderWidth" type="CIntegerType"/>
		<slot selector="borderStyle" name="borderStyle" type="CBorderStyleType"/>
		<slot selector="borderColor" name="borderColor" type="CColorType"/>

		<slot selector="dropShadowEnabled" name="shadowEnabled" type="CBooleanType"/>
		<slot selector="dropShadowColor" name="shadowColor" type="CColorType"/>

		<slot selector="roundCorners" name="roundedCorners" type="CBooleanType"/>
		<operator selector="r:g:b:" name="red:green:blue:" type="CColorType" help="make a color">
			<receiver name="Color" type="Color class" default="Color" />
			<argument name="R" type="CNumberType" default="1"/>
			<argument name="G" type="CNumberType" default="1"/>
			<argument name="B" type="CNumberType" default="1"/>
		</operator>
	</category>
	

	<category name="test and repeat">
		<operator selector="ifTrue:ifFalse:" name="yes:no:" help="control structure">
			<receiver name="Condition" type="CBooleanType" default="false"/>
			<argument name="trueBlock" type="CBlockType"/>
			<argument name="falseBlock" type="CBlockType"/>
		</operator>
		<message selector="forEach:do:until:" name="for each:do:until:" help="control structure">
			<argument name="event" type="CEventType" default="mouseMove"/>
			<argument name="block" type="CBlockType"/>
			<argument name="stopEvent" type="CEventType" default="mouseUp"/>
		</message>
		<operator selector="timesRepeat:" name="repeat:" help="control structure">
			<receiver name="count" type="CNumberType" default="5"/>
			<argument name="block" type="CBlockType"/>
		</operator>
		<message selector="wait:" name="wait" help="wait for seconds">
			<argument name="duration" type="CNumberType" default="1.0"/>
		</message>
	</category>

	<category name="geometry">
		<slot selector="cartesianX" name="x" type="CNumberType" help="The x coordinate"/>
		<slot selector="cartesianY" name="y" type="CNumberType" help="The y coordinate"/>
		<separator/>
		<slot selector="width" name="width" type="CNumberType"/>
		<slot selector="height" name="height" type="CNumberType"/>
		<separator/>
		<slot selector="angle" name="angle" type="CNumberType" help="The angle (in degrees)"/>
		<slot selector="scale" name="scale" type="CNumberType"/>
		<separator/>
		<slot selector="cartesianLeft" name="left" type="CNumberType"/>
		<slot selector="cartesianRight" name="right" type="CNumberType"/>
		<slot selector="cartesianTop" name="top" type="CNumberType"/>
		<slot selector="cartesianBottom" name="bottom" type="CNumberType"/>
	</category>

	<category name="holder" type="CEToyHolder">
		<slot selector="cursor" type="CNumberType" help="The index of the selected element"/>
		<slot selector="graphicAtCursor" type="CImageResourceType" help="The graphical image of the selected element"/>
		<slot selector="playerAtCursor" type="CPlayerType" help="The object at my cursor position"/>
		<slot selector="numberAtCursor" type="CNumberType" help="The number represented by the object at my cursor position"/>
		<slot selector="firstElement" type="CPlayerType" help="The first of the objects within me"/>
		<slot selector="lastElement" type="CPlayerType" help="The last of the objects within me"/>
	</category>

	<category name="images" type="CImageButton">
		<slot selector="normalImage" type="CImageResourceType"/>
		<slot selector="overImage" type="CImageResourceType"/>
		<slot selector="pressedImage" type="CImageResourceType"/>
		<slot selector="disabledImage" type="CImageResourceType"/>
		<slot selector="selectedImage" type="CImageResourceType"/>
	</category>

	<category name="text editor" type="CTextEditor">
		<slot selector="contents" type="CStringType"/>
		<slot selector="numericValue" type="CNumberType"/>
		<slot selector="selectionStart" type="CIntegerType"/>
		<slot selector="selectionStop" type="CIntegerType"/>
		<slot selector="selection" type="CStringType"/>
		<slot selector="allowEditing" type="CBooleanType"/>
		<slot selector="hScrollable" type="CBooleanType"/>
		<slot selector="vScrollable" type="CBooleanType"/>
		<message selector="findText:" name="find text" help="find a portion of text">
			<argument name="aString" type="CStringType" default=""/>
		</message>
		<message selector="append:" name="append text" help="append the text">
			<argument name="aString" type="CStringType" default=""/>
		</message>
		<message selector="enter" name="enter" help="simulate the enter key"/>
		<message selector="undo" name="undo" help="undo last editing"/>
		<message selector="redo" name="redo" help="redo last editing"/>
	</category>

	<category name="label" type="CLabel">
		<slot selector="contents" name="contents" type="CStringType" help="The label text"/>
		<slot selector="numericValue" type="CNumberType" help="The label''s numeric value"/>
	</category>

	<category name="book" type="CBook">
		<slot selector="cursor" type="CNumberType"/>
		<message selector="next"/>
		<message selector="prev"/>
		<slot selector="view" type="CBookViewType"/>
	</category>

	<category name="list" type="CList">
		<slot selector="cursor" type="CNumberType"/>
		<message selector="next"/>
		<message selector="prev"/>
	</category>

	<category name="math">
		<operator selector="+" name="+" type="CNumberType" help="Add two values">
			<receiver name="A" type="CNumberType" default="1"/>
			<argument name="B" type="CNumberType" default="1"/>
		</operator>
		<operator selector="-" name="-" type="CNumberType" help="Subtract two values">
			<receiver name="A" type="CNumberType" default="1"/>
			<argument name="B" type="CNumberType" default="1"/>
		</operator>
		<operator selector="*" name="*" type="CNumberType" help="Multiply two values">
			<receiver name="A" type="CNumberType" default="1"/>
			<argument name="B" type="CNumberType" default="1"/>
		</operator>
		<operator selector="/" name="/" type="CNumberType" help="Divide two values">
			<receiver name="A" type="CNumberType" default="1"/>
			<argument name="B" type="CNumberType" default="1"/>
		</operator>

		<separator/>

		<operator selector="<" name="<" type="CBooleanType" 
				help="True if the first number is less">
			<receiver name="A" type="CNumberType" default="1"/>
			<argument name="B" type="CNumberType" default="2"/>
		</operator>
		<operator selector=">" name=">" type="CBooleanType" 
				help="True if the first number is greater">
			<receiver name="A" type="CNumberType" default="2"/>
			<argument name="B" type="CNumberType" default="1"/>
		</operator>
		<operator selector="=" name="=" type="CBooleanType" 
				help="True if the numbers are equal">
			<receiver name="A" type="CNumberType" default="1"/>
			<argument name="B" type="CNumberType" default="1"/>
		</operator>

		<separator/>

		<operator selector="abs" name="absolute value" type="CNumberType" 
				help="Make into a positive number">
			<receiver name="A" type="CNumberType" default="-5"/>
		</operator>
		<operator selector="truncated" name="truncated" type="CNumberType" 
				help="The whole number part">
			<receiver name="A" type="CNumberType" default="3.6"/>
		</operator>
		<operator selector="rounded" name="rounded" type="CNumberType" 
				help="The whole number part">
			<receiver name="A" type="CNumberType" default="3.6"/>
		</operator>
		<operator selector="atRandom" name="random" type="CNumberType" 
				help="The whole number part">
			<receiver name="A" type="CIntegerType" default="180"/>
		</operator>
		<operator selector="raisedTo:" name="to the power of:" type="CNumberType" help="Raise arg1 to the power of arg2">
			<receiver name="A" type="CNumberType" default="2"/>
			<argument name="B" type="CNumberType" default="2"/>
		</operator>
	</category>

	<category name="slider" type="CSlider">
		<slot selector="value" type="CNumberType" help="The current value"/>
		<slot selector="min" type="CIntegerType" help="The minimum value"/>
		<slot selector="max" type="CIntegerType" help="The maximum value"/>
		<slot selector="step" type="CIntegerType" help="The step value"/>
		<message selector="next"/>
		<message selector="prev"/>
	</category>

	<category name="scripting">
		<message selector="startScript:" name="start script" help="Start this script">
			<argument name="aScript" type="CScriptNameType" default="emptyScript"/>
		</message>
		<message selector="stopScript:" name="stop script" help="Stop this script">
			<argument name="aScript" type="CScriptNameType" default="emptyScript"/>
		</message>
		<message selector="pauseScript:" name="pause script" help="Pause this script">
			<argument name="aScript" type="CScriptNameType" default="emptyScript"/>
		</message>
		<message selector="resumeScript:" name="resume script" help="Resume this script">
			<argument name="aScript" type="CScriptNameType" default="emptyScript"/>
		</message>
		<message selector="startTicking:at:" name="start ticking:at:" help="Start to tick this script">
			<argument name="aScript" type="CScriptNameType" default="emptyScript"/>
			<argument name="aFrequency" type="CNumberType" default="8"/>
		</message>
	</category>

	<category name="pen use">
		<slot selector="penDown" name="pen down" type="CBooleanType" />
		<slot selector="penColor" name="pen color" type="CColorType" />
		<slot selector="penWidth" name="pen width" type="CIntegerType" />
		<slot selector="penTrailStyle" name="trail style" type="CPenTrailStyleType" />
		<message selector="clearPenTrails" />
	</category>

	<category id="vectors" name="vectors">
		<slot selector="position" name="position" type="CPointType" help="The position of the object"/>
		<slot selector="direction" name="direction" type="CPointType" help="The direction of the object"/>
		<slot selector="normal" name="normal" type="CPointType" help="The direction of the object"/>
		<message selector="distanceTo:" name="distance to:" type="CNumberType">
			<argument name="aPlayer" type="CPlayerType"/>
		</message>
	</category>

	<category name="magnifier" type="CMagnifier">
		<slot selector="scale" name="scale" type="CNumberType"/>
		<slot selector="trackPointer" type="CBooleanType" />
		<slot selector="showPointer" type="CBooleanType" />
	</category>

</viewer>
'! !

!CTileDefinition class methodsFor: 'reading' stamp: 'bf 5/18/2005 18:21'!
isStatement: aSelector
	"remove once we have proper statement tiles"
	^#(
		ifTrue:ifFalse: 
		forEach:do:until:
	) includes: aSelector! !

!CTileDefinition class methodsFor: 'reading' stamp: 'ar 3/21/2004 19:23'!
openDefaultViewer
	^(self readXmlFrom: self testViewerSpec readStream) first new open! !

!CTileDefinition class methodsFor: 'reading' stamp: 'ar 3/21/2004 23:43'!
openDefaultViewerOn: aPlayer
	^((self readXmlFrom: self defaultViewerSpec readStream) first newFor: aPlayer) open! !

!CTileDefinition class methodsFor: 'reading' stamp: 'ar 3/21/2004 16:29'!
readXmlFrom: aStream
	| doc |
	doc := XMLDOMParser parseDocumentFrom: aStream.
	^doc elements collect:[:each| self xmlFrom: each].! !

!CTileDefinition class methodsFor: 'reading' stamp: 'ar 3/21/2004 20:41'!
testViewerSpec
	"CTileDefinition readXmlFrom: self defaultViewerSpec readStream"
^'
<viewer type="CPlayer">
	<category name="basic">
		<message selector="makeSound:" name="make sound:" help="Make the specified sound">
			<argument name="aSound" type="CSoundNameType" default="croak"/>
		</message>
		<message selector="forwardBy:" name="forward by:" help="">
			<argument name="delta" type="CNumberType" default="5"/>
		</message>
		<slot selector="angle" name="heading" type="CNumberType" help="The heading"/>
		<slot selector="color" name="color" type="CColorType"/>
		<message selector="color:sees:" name="color:sees:" type="CBooleanType">
			<argument name="myColor" type="CColorType" default="blue"/>
			<argument name="otherColor" type="CColorType" default="red"/>
		</message>
		<message selector="borderStyle:" name="borderStyle:">
			<argument name="aStyle" type="CBorderStyleType" default="simple"/>
		</message>
		<message selector="fillStyle:" name="fillStyle:">
			<argument name="aStyle" type="CFillStyleType" default="simple"/>
		</message>
	</category>
</viewer>
'! !

!CTileDefinition class methodsFor: 'reading' stamp: 'ar 6/21/2005 19:46'!
xmlFrom: xmlNode
	xmlNode name == #message 
		ifTrue:[^CMessageDefinition new readXmlFrom: xmlNode].
	xmlNode name == #slot
		ifTrue:[^CSlotDefinition new readXmlFrom: xmlNode].
	xmlNode name == #widget
		ifTrue:[^CWidgetDefinition new readXmlFrom: xmlNode].
	xmlNode name == #operator 
		ifTrue:[^COperatorDefinition new readXmlFrom: xmlNode].
	xmlNode name == #category
		ifTrue:[^CCategoryDefinition new readXmlFrom: xmlNode].
	xmlNode name == #viewer
		ifTrue:[^CViewerDefinition new readXmlFrom: xmlNode].
	xmlNode name == #separator
		ifTrue:[^CSeparatorDefinition new readXmlFrom: xmlNode].
	self error: xmlNode name, ' - unknown tile definition'.
	^nil! !

!CTileDefinition methodsFor: 'accessing' stamp: 'ar 3/21/2004 16:37'!
externalName
	"Answer the externalName of the receiver"
	<bewareOf: #externalNameChanged>
	^self propertyValueAt: #externalName! !

!CTileDefinition methodsFor: 'accessing' stamp: 'ar 3/21/2004 16:37'!
externalName: aValue
	"Modify the receiver's externalName"
	^self propertyValueAt: #externalName put: aValue with: #externalNameChanged! !

!CTileDefinition methodsFor: 'accessing' stamp: 'ar 3/21/2004 16:37'!
helpText
	"Answer the helpText of the receiver"
	<bewareOf: #helpTextChanged>
	^self propertyValueAt: #helpText! !

!CTileDefinition methodsFor: 'accessing' stamp: 'ar 3/21/2004 16:37'!
helpText: aValue
	"Modify the receiver's helpText"
	^self propertyValueAt: #helpText put: aValue with: #helpTextChanged! !

!CTileDefinition methodsFor: 'creating' stamp: 'ar 3/21/2004 17:36'!
new
	"Create a new tile for this definition"
	^self subclassResponsibility! !

!CTileDefinition methodsFor: 'creating' stamp: 'ar 3/21/2004 17:34'!
newFor: anObject
	"Create a new tile for the given object"
	^self subclassResponsibility! !

!CTileDefinition methodsFor: 'accessing' stamp: 'ar 3/21/2004 16:38'!
selector
	"Answer the selector of the receiver"
	<bewareOf: #selectorChanged>
	^self propertyValueAt: #selector! !

!CTileDefinition methodsFor: 'accessing' stamp: 'ar 3/21/2004 16:38'!
selector: aValue
	"Modify the receiver's selector"
	^self propertyValueAt: #selector put: aValue with: #selectorChanged! !

!CTileDefinition methodsFor: 'accessing' stamp: 'ar 3/21/2004 16:37'!
type
	"Answer the type of the receiver"
	<bewareOf: #typeChanged>
	^self propertyValueAt: #type! !

!CTileDefinition methodsFor: 'accessing' stamp: 'ar 3/21/2004 16:37'!
type: aValue
	"Modify the receiver's type"
	^self propertyValueAt: #type put: aValue with: #typeChanged! !

!CTileDefinition methodsFor: 'private' stamp: 'bf 6/23/2005 18:59'!
typeNamed: aString
	^Smalltalk classNamed: aString! !

!CTileDefinition methodsFor: 'printing' stamp: 'ar 3/21/2004 16:25'!
xmlString
	| stream writer |
	stream := WriteStream on: String new.
	writer := XMLWriterIndenting on: stream.
	self printXmlOn: writer.
	^stream contents! !

CTileDefinition subclass: #CViewerDefinition
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tweak-Tiles-Definition'!

CViewerDefinition defineFields: '<?xml  version="1.0" ?>
<fields>
	<field toGet="categories" toSet="categories:" changeEvent="categoriesChanged">categories</field>
</fields>'!

!CViewerDefinition methodsFor: 'accessing' stamp: 'ar 3/21/2004 17:03'!
categories
	"Answer the categories of the receiver"
	<bewareOf: #categoriesChanged>
	^self propertyValueAt: #categories! !

!CViewerDefinition methodsFor: 'accessing' stamp: 'ar 3/21/2004 17:03'!
categories: aValue
	"Modify the receiver's categories"
	^self propertyValueAt: #categories put: aValue with: #categoriesChanged! !

!CViewerDefinition methodsFor: 'initialize' stamp: 'ar 3/21/2004 17:05'!
initialize
	super initialize.
	self define: #categories as: CCollection new.! !

!CViewerDefinition methodsFor: 'creating' stamp: 'ar 3/21/2004 23:38'!
new
	| tile |
	tile := CTileViewer new.
	tile name: externalName.
	categories do:[:each| tile addPage: each new].
	tile pages cursor: 1.
	^tile! !

!CViewerDefinition methodsFor: 'creating' stamp: 'bf 10/4/2004 15:08'!
newFor: anObject
	| tile |
	tile := CTileViewer new.
	tile name: externalName.
	tile target: anObject.
	(categories 
		select:[:page| page type == nil or:[anObject isKindOf: page type]])
			do:[:each| tile addPage: (each newFor: anObject)].
	tile pages cursor: 1.
	^tile! !

!CViewerDefinition methodsFor: 'printing' stamp: 'ar 3/21/2004 17:25'!
printXmlOn: xmlWriter
	xmlWriter startTag: 'viewer'.
	selector ifNotNil:[xmlWriter attribute: 'selector' value: selector].
	externalName ifNotNil:[xmlWriter attribute: 'name' value: externalName].
	type ifNotNil:[xmlWriter attribute: 'type' value: type name].
	helpText ifNotNil:[xmlWriter attribute: 'help' value: helpText].
	xmlWriter endTag.
	categories do:[:each| each printXmlOn: xmlWriter].
	xmlWriter endTag: 'viewer'.! !

!CViewerDefinition methodsFor: 'printing' stamp: 'ar 3/21/2004 17:04'!
readXmlFrom: xmlNode
	| attr |
	xmlNode name == #viewer ifFalse:[^self error: 'Not a viewer'].
	attr := xmlNode attributeAt: 'selector'.
	attr ifNotNil:[selector := attr asSymbol].
	attr := xmlNode attributeAt: 'name'.
	attr ifNotNil:[externalName := attr].
	attr := xmlNode attributeAt: 'type'.
	attr ifNotNil:[type := self typeNamed: attr].
	attr := xmlNode attributeAt: 'help'.
	attr ifNotNil:[helpText := attr].
	xmlNode elements do:[:each|
		categories add: (CCategoryDefinition new readXmlFrom: each).
	].
	^self! !

CTileDefinition subclass: #CWidgetDefinition
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tweak-Tiles-Definition'!

!CWidgetDefinition commentStamp: '<historical>' prior: 0!
CWidgetDefinition defines an arbitrary widget in a viewer.!

CWidgetDefinition defineFields: '<?xml  version="1.0" ?>
<fields>
	<field toGet="widgetClassName" toSet="widgetClassName:" changeEvent="widgetClassNameChanged">widgetClassName</field>
	<field toGet="widgetLabel" toSet="widgetLabel:" changeEvent="widgetLabelChanged">widgetLabel</field>
</fields>'!

!CWidgetDefinition methodsFor: 'creating' stamp: 'ar 6/22/2005 11:38'!
new
	| theClass widget |
	theClass := Smalltalk classNamed: widgetClassName.
	theClass ifNil:[^nil].
	widget := theClass new.
	externalName ifNotNil:[widget name: externalName].
	widgetLabel ifNotNil:[widget label: widgetLabel].
	^widget! !

!CWidgetDefinition methodsFor: 'creating' stamp: 'ar 6/21/2005 19:50'!
newFor: anObject
	| widget |
	widget := self new.
	widget target: anObject.
	^widget! !

!CWidgetDefinition methodsFor: 'printing' stamp: 'ar 6/22/2005 11:38'!
printXmlOn: xmlWriter
	xmlWriter startTag: 'widget'.
	widgetClassName ifNotNil:[xmlWriter attribute: 'class' value: widgetClassName].
	externalName ifNotNil:[xmlWriter attribute: 'name' value: externalName].
	widgetLabel ifNotNil:[xmlWriter attribute: 'label' value: widgetLabel].
	xmlWriter endEmptyTag: 'widget'.! !

!CWidgetDefinition methodsFor: 'printing' stamp: 'ar 6/22/2005 11:38'!
readXmlFrom: xmlNode
	| attr |
	xmlNode name == #widget ifFalse:[^self error: 'Not a widget'].
	attr := xmlNode attributeAt: 'class'.
	attr ifNotNil:[widgetClassName := attr asSymbol].
	attr := xmlNode attributeAt: 'name'.
	attr ifNotNil:[externalName := attr].
	attr := xmlNode attributeAt: 'label'.
	attr ifNotNil:[widgetLabel := attr].
	^self! !

!CWidgetDefinition methodsFor: 'accessing' stamp: 'ar 6/21/2005 19:45'!
widgetClassName
	"Answer the widgetClassName of the receiver"
	<bewareOf: #widgetClassNameChanged>
	^self propertyValueAt: #widgetClassName! !

!CWidgetDefinition methodsFor: 'accessing' stamp: 'ar 6/21/2005 19:45'!
widgetClassName: aValue
	"Modify the receiver's widgetClassName"
	^self propertyValueAt: #widgetClassName put: aValue with: #widgetClassNameChanged! !

!CWidgetDefinition methodsFor: 'accessing' stamp: 'ar 6/22/2005 11:37'!
widgetLabel
	"Answer the widgetLabel of the receiver"
	<bewareOf: #widgetLabelChanged>
	^self propertyValueAt: #widgetLabel! !

!CWidgetDefinition methodsFor: 'accessing' stamp: 'ar 6/22/2005 11:37'!
widgetLabel: aValue
	"Modify the receiver's widgetLabel"
	^self propertyValueAt: #widgetLabel put: aValue with: #widgetLabelChanged! !

CPlayer subclass: #CTileInspector
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tweak-Tiles-Inspector'!

CTileInspector defineFields: '<?xml  version="1.0" ?>
<fields>
	<field toGet="target" toSet="target:" changeEvent="targetChanged">target</field>
	<field toGet="fieldList" toSet="fieldList:" changeEvent="fieldListChanged">fieldList</field>
	<field toGet="category" toSet="category:" changeEvent="categoryChanged">category</field>
</fields>'!

!CTileInspector methodsFor: 'accessing' stamp: 'ar 3/19/2004 15:24'!
category
	"Answer the category of the receiver"
	<bewareOf: #categoryChanged>
	^self propertyValueAt: #category! !

!CTileInspector methodsFor: 'accessing' stamp: 'ar 3/19/2004 15:24'!
category: aValue
	"Modify the receiver's category"
	^self propertyValueAt: #category put: aValue with: #categoryChanged! !

!CTileInspector methodsFor: 'accessing' stamp: 'ar 3/19/2004 15:18'!
fieldList
	"Answer the fieldList of the receiver"
	<bewareOf: #fieldListChanged>
	^self propertyValueAt: #fieldList! !

!CTileInspector methodsFor: 'accessing' stamp: 'ar 3/19/2004 15:18'!
fieldList: aValue
	"Modify the receiver's fieldList"
	^self propertyValueAt: #fieldList put: aValue with: #fieldListChanged! !

!CTileInspector methodsFor: 'initialize' stamp: 'ar 4/3/2004 20:10'!
initialize
	super initialize.
	self define: #fieldList as: CList new.
	fieldList itemMaker: CTileReadout.
! !

!CTileInspector methodsFor: 'private' stamp: 'ar 3/19/2004 18:16'!
messageTileFor: aSelector in: aClass
	"Answer a message tile for the given selector in our target"
	| source argNames msgTile varTile |
	source := aClass sourceCodeAt: aSelector.
	argNames := aClass parserClass new parseArgs: source notifying: nil.
	msgTile := CMessageTile new.
	msgTile receiver: (CValueTile on: target).
	msgTile selector: aSelector.
	msgTile arguments: CCollection new.
	argNames do:[:each|
		varTile := CVariableTile new.
		varTile name: each.
		msgTile arguments add: varTile.
	].
	^msgTile! !

!CTileInspector methodsFor: 'events' stamp: 'ar 3/19/2004 19:53'!
onCategoryChanged
	<on: categoryChanged>
	self updateFieldList.! !

!CTileInspector methodsFor: 'events' stamp: 'ar 3/19/2004 19:51'!
onTargetChanged
	<on: targetChanged>
	self updateFieldList.! !

!CTileInspector methodsFor: 'initialize' stamp: 'ar 4/30/2004 15:55'!
setupCostume
	super setupCostume.
	self layout: CTableLayout new.
	self add: fieldList.
! !

!CTileInspector methodsFor: 'private' stamp: 'ar 3/21/2004 02:16'!
shouldShowField: aField
	"Answer whether we want to show the given field"
	aField isPrivateField ifTrue:[^false].
	category ifNil:[^true].
	^(aField category ifNil:[#'user fields']) = category! !

!CTileInspector methodsFor: 'accessing' stamp: 'ar 3/19/2004 15:17'!
target
	"Answer the target of the receiver"
	<bewareOf: #targetChanged>
	^self propertyValueAt: #target! !

!CTileInspector methodsFor: 'accessing' stamp: 'ar 3/19/2004 15:17'!
target: aValue
	"Modify the receiver's target"
	^self propertyValueAt: #target put: aValue with: #targetChanged! !

!CTileInspector methodsFor: 'updating' stamp: 'ar 3/19/2004 19:55'!
updateFieldList
	"Update my field list"
	| newFields |
	target ifNil:[^fieldList removeAll].
	newFields := target definedFields 
		select:[:aField| self shouldShowField: aField].
	fieldList elements contents: (newFields collect:[:aField| target fieldNamed: aField name]).! !

CPlayer subclass: #CTileLibrary
	instanceVariableNames: ''
	classVariableNames: 'Default'
	poolDictionaries: ''
	category: 'Tweak-Tiles-Definition'!

CTileLibrary defineFields: '<?xml  version="1.0" ?>
<fields>
</fields>'!

!CTileLibrary class methodsFor: 'accessing' stamp: 'ar 3/7/2004 21:36'!
default
	^Default ifNil:[Default := self new].! !

!CTileLibrary class methodsFor: 'accessing' stamp: 'ar 3/7/2004 21:36'!
default: aLibrary
	Default := aLibrary.! !

!CTileLibrary methodsFor: 'defaults' stamp: 'ar 3/7/2004 23:26'!
argumentTileCostume
	^CVariableTileCostume! !

!CTileLibrary methodsFor: 'defaults' stamp: 'bf 2/15/2005 15:30'!
assignmentTileCostume
	^CAssignmentTileCostume! !

!CTileLibrary methodsFor: 'defaults' stamp: 'ar 3/22/2004 16:21'!
messageTileCostume
	^CMessageTileCostume! !

!CTileLibrary methodsFor: 'defaults' stamp: 'bf 4/23/2005 15:22'!
operatorTileCostume
	^COperatorTileCostume
! !

!CTileLibrary methodsFor: 'defaults' stamp: 'ar 3/8/2004 21:47'!
scriptorTileCostume
	^CScriptorTileCostume! !

!CTileLibrary methodsFor: 'defaults' stamp: 'ar 3/8/2004 21:07'!
sequenceTileCostume
	^CSequenceTileCostume! !

!CTileLibrary methodsFor: 'defaults' stamp: 'ar 3/22/2004 23:43'!
tileFont
	^TextStyle defaultFont
	"^(TextStyle named: 'ComicBold') fontOfSize: 16."! !

!CTileLibrary methodsFor: 'value tiles' stamp: 'ar 5/12/2005 13:24'!
valueTileForType: aType
	(aType includesBehavior: CStringType) ifTrue:[^CStringTileCostume].
	(aType includesBehavior: CNumberType) ifTrue:[^CNumberTileCostume].
	(aType includesBehavior: CEnumType) ifTrue:[^CEnumListCostume].
	(aType includesBehavior: CColorType) ifTrue:[^CColorTileCostume].
	(aType includesBehavior: CImageResourceType) ifTrue:[^CImageResourceTileCostume].
	(aType includesBehavior: CScriptNameType) ifTrue:[^CScriptNameCostume].
	(aType includesBehavior: CVectorType) ifTrue:[^CVectorTileCostume].
	^CValueTileCostume! !

CPlayer subclass: #CTilePlayer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tweak-Tiles-Players'!

!CTilePlayer commentStamp: 'tk 3/7/2005 10:13' prior: 0!
A kind of tile.  It is used in a script to make an object do something.

### Allow Users to see the above explanation ###

CTilePlayer is the abstract superclass for all elements in the tile scripting world. It is essentially a parse node representation, but it also knows how to represent itself graphically via the appropriate costumes.

Each tile element has a "valueType" which represents the type associated with it. Note that the meaning of "valueType" is polymorphic, meaning that depending on the concrete element its interpretation differs. For example:
* CMessageTile: "valueType" describes the return type of the message (if any). Messages without a return type are assumed to be "commands" not returning any useful value.
* CValueTile: "valueType" describes the type of the value represented by the tile. If the type has not been set explicitly it will be derived from the concrete object (in which case it may too specific).
* CArgumentTile: "valueType" describes the *expected* type of the argument.
* CPropertyTile: "valueType" describes the type of the (virtual or real) property.
!

CTilePlayer subclass: #CAssignmentTile
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tweak-Tiles-Players'!

!CAssignmentTile commentStamp: '<historical>' prior: 0!
A tile for setting a property of some object. If operator is not nil, it compiles an in-place modifier.!

CAssignmentTile defineFields: '<?xml  version="1.0" ?>
<fields>
	<field toGet="property" toSet="property:" changeEvent="propertyChanged">property</field>
	<field toGet="expression" toSet="expression:" changeEvent="expressionChanged">expression</field>
	<field toGet="operator" toSet="operator:" changeEvent="operatorChanged">operator</field>
</fields>'!

!CAssignmentTile methodsFor: 'compiling' stamp: 'ar 12/11/2009 22:16'!
asParseNode: encoder

	^ MessageNode new
		receiver: (encoder encodeScriptObject: self receiver)
		selector: self selector
		arguments: {operator
			ifNil: [self expressionAsParseNode: encoder]
			ifNotNil: [self operatorAsParseNode: encoder]}
		precedence: 1
		from: encoder! !

!CAssignmentTile methodsFor: 'accessing' stamp: 'bf 7/21/2005 22:36'!
childTiles
	^{property. expression}! !

!CAssignmentTile methodsFor: 'initialize' stamp: 'bf 2/15/2005 15:28'!
defaultCostume
	^CTileLibrary default assignmentTileCostume! !

!CAssignmentTile methodsFor: 'evaluating' stamp: 'bf 5/9/2005 17:25'!
eval
	"If operator is, e.g., '+=' we do 
		receiver property: receiver property += expression
	instead of the regular
		receiver property: expression"

	| expr |
	expr := expression eval.
	operator ifNotNil: [expr := property eval perform: operator with: expr].
	^self receiver perform: self selector with: expr! !

!CAssignmentTile methodsFor: 'accessing' stamp: 'bf 2/15/2005 15:01'!
expression
	"Answer the expression of the receiver"
	<bewareOf: #expressionChanged>
	^self propertyValueAt: #expression! !

!CAssignmentTile methodsFor: 'accessing' stamp: 'bf 2/15/2005 15:01'!
expression: aValue
	"Modify the receiver's expression"
	^self propertyValueAt: #expression put: aValue with: #expressionChanged! !

!CAssignmentTile methodsFor: 'compiling' stamp: 'bf 5/9/2005 17:16'!
expressionAsParseNode: encoder

	^ expression asParseNode: encoder
! !

!CAssignmentTile methodsFor: 'testing' stamp: 'bf 2/15/2005 16:24'!
isStatementLike
	^true! !

!CAssignmentTile methodsFor: 'events' stamp: 'bf 2/16/2005 15:47'!
onExpressionChanged
	<on: expressionChanged>

	expression parent: self.
	self compileNeeded.! !

!CAssignmentTile methodsFor: 'events' stamp: 'bf 5/9/2005 16:58'!
onOperatorChanged
	<on: operatorChanged>

	self compileNeeded.! !

!CAssignmentTile methodsFor: 'events' stamp: 'bf 2/16/2005 15:47'!
onPropertyChanged
	<on: propertyChanged>

	property parent: self.
	self compileNeeded.! !

!CAssignmentTile methodsFor: 'accessing' stamp: 'bf 5/9/2005 16:52'!
operator
	"Answer the operator of the receiver"
	<bewareOf: #operatorChanged>
	^self propertyValueAt: #operator! !

!CAssignmentTile methodsFor: 'accessing' stamp: 'bf 5/9/2005 16:52'!
operator: aValue
	"Modify the receiver's operator"
	^self propertyValueAt: #operator put: aValue with: #operatorChanged! !

!CAssignmentTile methodsFor: 'compiling' stamp: 'ar 12/11/2009 22:17'!
operatorAsParseNode: encoder

	^ MessageNode new
		receiver: (property asParseNode: encoder)
		selector: operator
		arguments: {self expressionAsParseNode: encoder}
		precedence: operator precedence
		from: encoder ! !

!CAssignmentTile methodsFor: 'accessing' stamp: 'bf 2/15/2005 15:24'!
property
	"Answer the property of the receiver"
	<bewareOf: #propertyChanged>
	^self propertyValueAt: #property! !

!CAssignmentTile methodsFor: 'accessing' stamp: 'bf 2/15/2005 15:24'!
property: aValue
	"Modify the receiver's property"
	^self propertyValueAt: #property put: aValue with: #propertyChanged! !

!CAssignmentTile methodsFor: 'accessing' stamp: 'bf 4/22/2005 19:49'!
receiver
	^property receiver! !

!CAssignmentTile methodsFor: 'accessing' stamp: 'tak 1/7/2006 12:55'!
receiver: aValue 
	^ property receiver: aValue! !

!CAssignmentTile methodsFor: 'accessing' stamp: 'bf 4/22/2005 20:05'!
selector
	^(property property, ':') asSymbol! !

CTilePlayer subclass: #CMessageTile
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tweak-Tiles-Players'!

!CMessageTile commentStamp: 'tk 3/10/2005 11:02' prior: 0!
A Message Tile holds one message to send to an object.  The message will make the object do something.  The little tile inside it on the left is an object.  Next is the message name, also called the selector.  On the right, there may be 'arguments' to hold more information.  
	Drop the tile into a scriptor to make it be part of a script.  Get new scriptor from the Tile Viewer of an object (from the aqua halo handle).  
	Start a new script by going to the 'scripts' category of the Tile Viewer, and drag out the 'emptyScript' tile.  Change categories by clicking on the title of the Tile Viewer.[general]

### Allow Users to see the above explanation ###!

CMessageTile defineFields: '<?xml  version="1.0" ?>
<fields>
	<field toGet="arguments" toSet="arguments:" changeEvent="argumentsChanged">arguments</field>
	<field toGet="receiver" toSet="receiver:" changeEvent="receiverChanged">receiver</field>
	<field toGet="selector" toSet="selector:" changeEvent="selectorChanged">selector</field>
	<field toGet="displayName" toSet="displayName:" changeEvent="displayNameChanged">displayName</field>
</fields>'!

!CMessageTile methodsFor: 'accessing' stamp: 'ar 3/17/2004 15:11'!
arguments
	"This method was automatically generated"
	<bewareOf: #argumentsChanged>
	^self propertyValueAt: #arguments! !

!CMessageTile methodsFor: 'accessing' stamp: 'ar 3/11/2004 14:23'!
arguments: aValue
	"This method was automatically generated"
	^self propertyValueAt: #arguments put: aValue with: #argumentsChanged! !

!CMessageTile methodsFor: 'compiling' stamp: 'ar 12/11/2009 22:17'!
asParseNode: encoder

	| recNode |
	recNode := encoder encodeScriptObject: receiver.
	selector ifNil: [^ recNode].
	^ MessageNode new
		receiver: recNode
		selector: selector
		arguments: (arguments collect: [:each | each asParseNode: encoder])
		precedence: selector precedence
		from: encoder ! !

!CMessageTile methodsFor: 'accessing' stamp: 'bf 7/21/2005 22:36'!
childTiles
	^arguments! !

!CMessageTile methodsFor: 'initialize' stamp: 'ar 3/7/2004 21:39'!
defaultCostume
	^CTileLibrary default messageTileCostume! !

!CMessageTile methodsFor: 'accessing' stamp: 'ar 5/12/2005 20:05'!
displayName
	"Answer the displayName of the receiver"
	<bewareOf: #displayNameChanged>
	^self propertyValueAt: #displayName! !

!CMessageTile methodsFor: 'accessing' stamp: 'ar 5/12/2005 20:05'!
displayName: aValue
	"Modify the receiver's displayName"
	^self propertyValueAt: #displayName put: aValue with: #displayNameChanged! !

!CMessageTile methodsFor: 'evaluating' stamp: 'bf 4/26/2005 16:20'!
eval
	^self receiver
		perform: self selector
		withArguments: (self arguments asArray collect:[:each| each eval])! !

!CMessageTile methodsFor: 'testing' stamp: 'ar 3/22/2004 18:15'!
isComplex
	"Am I a complex expression which needs parenthesis?"
	^arguments size > 0! !

!CMessageTile methodsFor: 'testing' stamp: 'ar 3/7/2004 18:53'!
isMessageTile
	^true! !

!CMessageTile methodsFor: 'testing' stamp: 'bf 3/4/2005 16:44'!
isStatementLike
	^self valueType isNil! !

!CMessageTile methodsFor: 'events' stamp: 'bf 9/27/2004 17:36'!
onArgumentsChanged
	<on: argumentsChanged>

	arguments do: [:each | each parent: self].
	self compileNeeded.! !

!CMessageTile methodsFor: 'events' stamp: 'bf 4/22/2005 19:58'!
onReceiverChanged
	<on: receiverChanged>

	self compileNeeded.! !

!CMessageTile methodsFor: 'events' stamp: 'bf 2/21/2005 14:00'!
onSelectorChanged
	<on: selectorChanged>

	self compileNeeded.! !

!CMessageTile methodsFor: 'accessing' stamp: 'ar 3/17/2004 15:11'!
receiver
	"This method was automatically generated"
	<bewareOf: #receiverChanged>
	^self propertyValueAt: #receiver! !

!CMessageTile methodsFor: 'accessing' stamp: 'ar 3/11/2004 14:23'!
receiver: aValue
	"This method was automatically generated"
	^self propertyValueAt: #receiver put: aValue with: #receiverChanged! !

!CMessageTile methodsFor: 'queries' stamp: 'bf 7/31/2005 19:18'!
referencesDo: aBlock
	aBlock value: receiver.
	^super referencesDo: aBlock
! !

!CMessageTile methodsFor: 'accessing' stamp: 'ar 3/17/2004 15:11'!
selector
	"This method was automatically generated"
	<bewareOf: #selectorChanged>
	^self propertyValueAt: #selector! !

!CMessageTile methodsFor: 'accessing' stamp: 'ar 3/11/2004 14:23'!
selector: aValue
	"This method was automatically generated"
	^self propertyValueAt: #selector put: aValue with: #selectorChanged! !

CTilePlayer subclass: #COperatorTile
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tweak-Tiles-Players'!

!COperatorTile commentStamp: 'bf 7/31/2005 19:33' prior: 0!
In contrast to MessageTiles, OperatorTiles do not belong to their receiver but the receiver is a variable that other objects can be dropped on.!

COperatorTile defineFields: '<?xml  version="1.0" ?>
<fields>
	<field toGet="arguments" toSet="arguments:" changeEvent="argumentsChanged">arguments</field>
	<field toGet="receiver" toSet="receiver:" changeEvent="receiverChanged">receiver</field>
	<field toGet="selector" toSet="selector:" changeEvent="selectorChanged">selector</field>
	<field toGet="displayName" toSet="displayName:" changeEvent="displayNameChanged">displayName</field>
</fields>'!

!COperatorTile methodsFor: 'accessing' stamp: 'bf 7/31/2005 19:27'!
arguments
	"Answer the arguments of the receiver"
	<bewareOf: #argumentsChanged>
	^self propertyValueAt: #arguments! !

!COperatorTile methodsFor: 'accessing' stamp: 'bf 7/31/2005 19:27'!
arguments: aValue
	"Modify the receiver's arguments"
	^self propertyValueAt: #arguments put: aValue with: #argumentsChanged! !

!COperatorTile methodsFor: 'compiling' stamp: 'ar 12/11/2009 22:17'!
asParseNode: encoder

	| recNode |
	recNode := receiver asParseNode: encoder.
	selector ifNil: [^ recNode].
	^ MessageNode new
		receiver: recNode
		selector: selector
		arguments: (arguments collect: [:each | each asParseNode: encoder])
		precedence: selector precedence
		from: encoder ! !

!COperatorTile methodsFor: 'accessing' stamp: 'bf 7/21/2005 22:36'!
childTiles
	^{receiver}, arguments! !

!COperatorTile methodsFor: 'initialize' stamp: 'ar 3/22/2004 16:04'!
defaultCostume
	^CTileLibrary default operatorTileCostume! !

!COperatorTile methodsFor: 'accessing' stamp: 'bf 7/31/2005 19:27'!
displayName
	"Answer the displayName of the receiver"
	<bewareOf: #displayNameChanged>
	^self propertyValueAt: #displayName! !

!COperatorTile methodsFor: 'accessing' stamp: 'bf 7/31/2005 19:27'!
displayName: aValue
	"Modify the receiver's displayName"
	^self propertyValueAt: #displayName put: aValue with: #displayNameChanged! !

!COperatorTile methodsFor: 'evaluating' stamp: 'bf 4/26/2005 16:21'!
eval
	^self receiver eval
		perform: self selector
		withArguments: (self arguments asArray collect: [:each| each eval])! !

!COperatorTile methodsFor: 'testing' stamp: 'ar 3/22/2004 18:12'!
isComplex
	"Am I a complex expression which needs parenthesis?"
	^true! !

!COperatorTile methodsFor: 'testing' stamp: 'ar 8/5/2005 21:38'!
isStatementLike
	^self valueType isNil! !

!COperatorTile methodsFor: 'events' stamp: 'bf 7/31/2005 19:27'!
onArgumentsChanged
	<on: argumentsChanged>

	arguments do: [:each | each parent: self].
	self compileNeeded.! !

!COperatorTile methodsFor: 'events' stamp: 'bf 7/31/2005 19:28'!
onReceiverChanged
	<on: receiverChanged>

	receiver parent: self.
	self compileNeeded.
! !

!COperatorTile methodsFor: 'events' stamp: 'bf 7/31/2005 19:27'!
onSelectorChanged
	<on: selectorChanged>

	self compileNeeded.! !

!COperatorTile methodsFor: 'accessing' stamp: 'bf 7/31/2005 19:27'!
receiver
	"Answer the receiver of the receiver"
	<bewareOf: #receiverChanged>
	^self propertyValueAt: #receiver! !

!COperatorTile methodsFor: 'accessing' stamp: 'bf 7/31/2005 19:27'!
receiver: aValue
	"Modify the receiver's receiver"
	^self propertyValueAt: #receiver put: aValue with: #receiverChanged! !

!COperatorTile methodsFor: 'converting' stamp: 'bf 7/31/2005 17:23'!
replacing: previousExpression
	"I am dropped into a variable tile, use previousExpression as our receiver if appropriate (see willReplaceReceiverWith:)"
	(self willReplaceReceiverWith: previousExpression) ifTrue: [
		previousExpression parent: nil.
		receiver expression: previousExpression].
	^self! !

!COperatorTile methodsFor: 'accessing' stamp: 'bf 7/31/2005 19:27'!
selector
	"Answer the selector of the receiver"
	<bewareOf: #selectorChanged>
	^self propertyValueAt: #selector! !

!COperatorTile methodsFor: 'accessing' stamp: 'bf 7/31/2005 19:27'!
selector: aValue
	"Modify the receiver's selector"
	^self propertyValueAt: #selector put: aValue with: #selectorChanged! !

!COperatorTile methodsFor: 'converting' stamp: 'bf 2/8/2006 18:47'!
willReplaceReceiverWith: otherExpression
	"if I am dropped into a variable tile, answer if we are willing to use otherExpression as our receiver"
	^selector precedence < 3
		and: [otherExpression isValueTile not]! !

CTilePlayer subclass: #CPropertyTile
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tweak-Tiles-Players'!

!CPropertyTile commentStamp: 'tk 3/10/2005 10:43' prior: 0!
A PropertyTile reads a single field or property of an object.  Running this tile will return a value.  This tile is not used to make the object do anything.  The little tile inside it on the left is an object.  Next is the name of the field, also called a property name.  
	Drop the tile into a scriptor to make it be part of a script.  Drop it on an existing tile, on an object or a value.  

### Allow Users to see the above explanation ###

A tile representing a (virtual or real) property of some object.!

CPropertyTile defineFields: '<?xml  version="1.0" ?>
<fields>
	<field toGet="property" toSet="property:" changeEvent="propertyChanged">property</field>
	<field toGet="receiver" toSet="receiver:" changeEvent="receiverChanged">receiver</field>
	<field toGet="displayName" toSet="displayName:" changeEvent="displayNameChanged">displayName</field>
</fields>'!

!CPropertyTile methodsFor: 'accessing' stamp: 'tk 11/22/2004 15:25'!
arguments
	"just answer something"
	^ #()! !

!CPropertyTile methodsFor: 'converting' stamp: 'bf 2/18/2005 10:08'!
asAssignmentTile
	^ CAssignmentTile new
		property: self;
		expression: (CVariableTile new
			expression: self asValueTile;
			valueType: self valueType;
			yourself);
		yourself! !

!CPropertyTile methodsFor: 'compiling' stamp: 'ar 12/11/2009 22:17'!
asParseNode: encoder

	| recNode |
	recNode := encoder encodeScriptObject: receiver.
	property == nil ifTrue: [^ recNode].
	^ MessageNode new
		receiver: recNode
		selector: (encoder encodeSelector: property)
		arguments: #()
		precedence: 1
! !

!CPropertyTile methodsFor: 'converting' stamp: 'bf 2/18/2005 10:20'!
asValueTile
	^ (CValueTile on: self eval)
		valueType: self valueType;
		yourself! !

!CPropertyTile methodsFor: 'accessing' stamp: 'bf 7/21/2005 22:39'!
childTiles
	^#()! !

!CPropertyTile methodsFor: 'initialize' stamp: 'ar 3/21/2004 18:17'!
defaultCostume
	^CPropertyTileCostume! !

!CPropertyTile methodsFor: 'accessing' stamp: 'ar 5/12/2005 20:05'!
displayName
	"Answer the displayName of the receiver"
	<bewareOf: #displayNameChanged>
	^self propertyValueAt: #displayName! !

!CPropertyTile methodsFor: 'accessing' stamp: 'ar 5/12/2005 20:05'!
displayName: aValue
	"Modify the receiver's displayName"
	^self propertyValueAt: #displayName put: aValue with: #displayNameChanged! !

!CPropertyTile methodsFor: 'evaluating' stamp: 'ar 3/22/2004 21:18'!
eval
	^self receiver get: property! !

!CPropertyTile methodsFor: 'accessing' stamp: 'ar 3/28/2004 16:04'!
fieldName
	"If the receiver represents an updating field, answer its name"
	^property! !

!CPropertyTile methodsFor: 'testing' stamp: 'ar 3/7/2004 18:53'!
isPropertyTile
	^true! !

!CPropertyTile methodsFor: 'events' stamp: 'bf 9/27/2004 17:27'!
onPropertyChanged
	<on: propertyChanged>

	self compileNeeded.! !

!CPropertyTile methodsFor: 'events' stamp: 'bf 9/27/2004 17:26'!
onReceiverChanged
	<on: receiverChanged>

	self compileNeeded.! !

!CPropertyTile methodsFor: 'converting' stamp: 'ar 7/14/2005 14:43'!
openAsWatcher
	"Given that the receiver has a value, open a watcher that will display the value.  The watcher replaces the receiver."

	| aWatcher aPosition aSelector |
	aWatcher := CWatcher new.
	aWatcher target: self receiver.
	aSelector := self property.
	aWatcher selector: aSelector.
	aWatcher displayName: displayName.
	aWatcher originalTiles: self.
	aWatcher valueType: self valueType.
	aPosition := self globalPosition.
	self destroy.
	aWatcher openAt: aPosition! !

!CPropertyTile methodsFor: 'accessing' stamp: 'ar 3/17/2004 15:13'!
property
	"This method was automatically generated"
	<bewareOf: #propertyChanged>
	^self propertyValueAt: #property! !

!CPropertyTile methodsFor: 'accessing' stamp: 'ar 3/11/2004 14:24'!
property: aValue
	"This method was automatically generated"
	^self propertyValueAt: #property put: aValue with: #propertyChanged! !

!CPropertyTile methodsFor: 'accessing' stamp: 'ar 3/17/2004 15:13'!
receiver
	"This method was automatically generated"
	<bewareOf: #receiverChanged>
	^self propertyValueAt: #receiver! !

!CPropertyTile methodsFor: 'accessing' stamp: 'ar 3/11/2004 14:24'!
receiver: aValue
	"This method was automatically generated"
	^self propertyValueAt: #receiver put: aValue with: #receiverChanged! !

!CPropertyTile methodsFor: 'queries' stamp: 'bf 7/31/2005 19:17'!
referencesDo: aBlock
	^aBlock value: receiver! !

CTilePlayer subclass: #CSequenceTile
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tweak-Tiles-Players'!

CSequenceTile subclass: #CScriptorTile
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tweak-Tiles-Players'!

!CScriptorTile commentStamp: 'tk 3/10/2005 11:01' prior: 0!
A ScriptTileEditor is a place to create and edit scripts that are made out of tiles.  A script is a small program that tells an object what to do.  To try it, click on the (!!) button.  The name of the script is at the bottom middle.
	To add tiles, get a Tile Viewer on the object (aqua handle on its halo).  Drag a tile to the ScriptTileEditor and drop it.  To remove a tile, just drag it out of the scriptor.
	Start a new script by going to the 'scripts' category of the Tile Viewer, and drag out the 'emptyScript' tile.  Change categories by clicking on the title of the Tile Viewer.[general]

### Allow Users to see the above explanation ###!

CScriptorTile defineFields: '<?xml  version="1.0" ?>
<fields>
	<field toGet="receiver" toSet="receiver:" changeEvent="receiverChanged">receiver</field>
	<field toGet="selector" toSet="selector:" changeEvent="selectorChanged">selector</field>
	<field toGet="properties" toSet="properties:" changeEvent="propertiesChanged">properties</field>
	<field virtual="true" serializable="false" toGet="trigger" toSet="trigger:" changeEvent="triggerChanged">trigger</field>
	<field virtual="true" serializable="false" toGet="frequency" toSet="frequency:" changeEvent="triggerChanged">frequency</field>
</fields>'!

!CScriptorTile class methodsFor: 'as yet unclassified' stamp: 'tk 12/13/2004 10:35'!
initialize

	Preferences addPreference: #tileMessagePanel categories: #('Tweak') default: false balloonHelp: 'When true, show a panel of messages to send to the tile outlined in red.  Select any dropped tile.'.! !

!CScriptorTile methodsFor: 'compiling' stamp: 'ar 12/11/2009 22:20'!
asParseNode
	| enc |
	enc := Encoder new.
	enc initScriptee: receiver context: nil notifying: self.
	^ self asParseNode: enc! !

!CScriptorTile methodsFor: 'compiling' stamp: 'ar 12/11/2009 22:17'!
asParseNode: encoder
	^MethodNode new
		selector: selector
		arguments: #()
		precedence: selector precedence
		temporaries: #()
		block: (super asParseNode: encoder) returnNilIfNoOther
		encoder: encoder
		primitive: 0
		properties: properties copy "(!!)"
! !

!CScriptorTile methodsFor: 'initialize' stamp: 'ar 12/19/2004 17:30'!
chooseNewName
	"Choose initial name for new script"
	| index sel |
	index := 0.
	[index := index + 1.
	sel := ('script', index) asSymbol.
	receiver respondsTo: sel] whileTrue.
	selector := sel.! !

!CScriptorTile methodsFor: 'compiling' stamp: 'bf 10/11/2004 17:03'!
compile
	receiver compileScriptNode: self asParseNode.
	receiver addScriptor: self.
! !

!CScriptorTile methodsFor: 'initialize' stamp: 'ar 3/8/2004 21:46'!
defaultCostume
	^CTileLibrary default scriptorTileCostume! !

!CScriptorTile methodsFor: 'private' stamp: 'ar 8/5/2005 21:02'!
dragOffset
	"Do not offset scriptors"
	^0! !

!CScriptorTile methodsFor: 'evaluating' stamp: 'bf 5/3/2005 18:57'!
eval
	"compiled now"
	^receiver perform: selector! !

!CScriptorTile methodsFor: 'accessing' stamp: 'bf 5/4/2005 19:01'!
frequency
	"Answer the frequency of the receiver"
	<bewareOf: #triggerChanged>	

	^(trigger isNil or: [trigger key ~~ #ticking])
		ifTrue: [8]
		ifFalse: [trigger value]! !

!CScriptorTile methodsFor: 'accessing' stamp: 'bf 5/17/2005 17:41'!
frequency: aValue
	"Modify the receiver's frequency - sets the script to ticking"
	"Do not just modify trigger's value - must do a recompile!!"
	trigger := #ticking -> aValue.
	^aValue! !

!CScriptorTile methodsFor: 'testing' stamp: 'bf 7/31/2005 19:04'!
isScriptor
	^ true! !

!CScriptorTile methodsFor: 'events' stamp: 'bf 7/14/2005 18:43'!
onClose
	<on: close>
	self abandonCostume.! !

!CScriptorTile methodsFor: 'events' stamp: 'bf 9/27/2004 17:33'!
onCompileNeeded
	<on: compileNeeded>

	self compile! !

!CScriptorTile methodsFor: 'events' stamp: 'bf 5/3/2005 16:43'!
onPropertiesChanged
	<on: propertiesChanged>

	self compileNeeded.! !

!CScriptorTile methodsFor: 'events' stamp: 'bf 9/27/2004 17:23'!
onReceiverChanged
	<on: receiverChanged>

	self compileNeeded.! !

!CScriptorTile methodsFor: 'events' stamp: 'bf 5/17/2005 18:25'!
onScriptStatusChangedInReceiver: aSelector
	<on: scriptStatusChanged in: receiver>
	
	"enable this when bug #1219 is fixed"
	"aSelector == selector ifTrue: ["
		self signal: #tickingChanged! !

!CScriptorTile methodsFor: 'events' stamp: 'bf 10/11/2004 17:48'!
onSelectorChangedTo: newSelector from: oldSelector
	<on: selectorChanged>

	self compile.
	oldSelector
		ifNotNil: [receiver removeScript: oldSelector].
! !

!CScriptorTile methodsFor: 'events' stamp: 'bf 5/3/2005 18:35'!
onTriggerChanged
	<on: triggerChanged>

	self compileNeeded.! !

!CScriptorTile methodsFor: 'evaluating' stamp: 'bf 5/17/2005 18:02'!
pause
	trigger := #ticking->frequency.
	self syncScript.
	receiver pauseScript: selector.
! !

!CScriptorTile methodsFor: 'testing' stamp: 'bf 5/3/2005 19:06'!
paused
	(receiver findScript: selector)
		ifNotNilDo: [:script | ^script isPaused].
	^false! !

!CScriptorTile methodsFor: 'printing' stamp: 'bf 10/11/2004 16:54'!
printOn: aStream
	super printOn: aStream.
	aStream nextPut: $(;
		print: selector;
		nextPut: $)! !

!CScriptorTile methodsFor: 'accessing' stamp: 'bf 5/3/2005 16:42'!
properties
	"Answer the properties of the receiver"
	<bewareOf: #propertiesChanged>
	^self propertyValueAt: #properties! !

!CScriptorTile methodsFor: 'accessing' stamp: 'bf 5/3/2005 16:42'!
properties: aValue
	"Modify the receiver's properties"
	^self propertyValueAt: #properties put: aValue with: #propertiesChanged! !

!CScriptorTile methodsFor: 'accessing' stamp: 'ar 3/17/2004 15:13'!
receiver
	"This method was automatically generated"
	<bewareOf: #receiverChanged>
	^self propertyValueAt: #receiver! !

!CScriptorTile methodsFor: 'accessing' stamp: 'ar 3/11/2004 14:25'!
receiver: aValue
	"This method was automatically generated"
	^self propertyValueAt: #receiver put: aValue with: #receiverChanged! !

!CScriptorTile methodsFor: 'compiling' stamp: 'bf 11/30/2004 17:04'!
remove
	receiver removeScript: self selector.
! !

!CScriptorTile methodsFor: 'accessing' stamp: 'ar 3/17/2004 15:13'!
selector
	"This method was automatically generated"
	<bewareOf: #selectorChanged>
	^self propertyValueAt: #selector! !

!CScriptorTile methodsFor: 'accessing' stamp: 'ar 3/11/2004 14:25'!
selector: aValue
	"This method was automatically generated"
	^self propertyValueAt: #selector put: aValue with: #selectorChanged! !

!CScriptorTile methodsFor: 'evaluating' stamp: 'bf 5/17/2005 18:02'!
start
	trigger := #ticking->frequency.
	receiver resumeScript: selector.
! !

!CScriptorTile methodsFor: 'evaluating' stamp: 'bf 5/3/2005 18:39'!
stop
	trigger := nil.! !

!CScriptorTile methodsFor: 'accessing' stamp: 'bf 5/4/2005 19:33'!
ticking
	"answer false if paused, true if ticking, nil otherwise"
	^ trigger ifNotNil: [
		trigger key == #ticking
			ifTrue: [self paused not]]
! !

!CScriptorTile methodsFor: 'accessing' stamp: 'bf 5/3/2005 17:23'!
trigger
	"trigger may be #on->#eventName or #ticking->frequency"
	<bewareOf: #triggerChanged>

	^properties ifNotNil: [
		properties associationAt: #on
			ifAbsent: [properties associationAt: #ticking
				ifAbsent: [nil]]]! !

!CScriptorTile methodsFor: 'accessing' stamp: 'bf 5/4/2005 19:59'!
trigger: newTrigger
	"trigger may be #on->#eventName or #ticking->frequency"

	| oldTrigger wasTicking isTicking |

	"Might already exist"
	(properties notNil
		and: [newTrigger notNil
			and: [properties includesAssociation: newTrigger]])
				ifTrue: [^newTrigger].

	wasTicking := isTicking := false.

	"remove old trigger"
	oldTrigger := trigger.
	oldTrigger ifNotNil: [
		properties removeKey: oldTrigger key.
		wasTicking := oldTrigger key == #ticking].

	"add new one"
	newTrigger ifNotNil: [
		self assert: [#(on ticking) includes: newTrigger key].
		properties ifNil: [properties := IdentityDictionary new].
		properties add: newTrigger.
		isTicking := newTrigger key == #ticking].

	properties isEmptyOrNil ifTrue: [properties := nil].
	
	oldTrigger = newTrigger
		ifFalse: [
			wasTicking & isTicking not ifTrue: [self signal: #tickingChanged].
			self signalChanged: #triggerChanged from: oldTrigger to: newTrigger].

	^newTrigger! !

CSequenceTile defineFields: '<?xml  version="1.0" ?>
<fields>
	<field toGet="statements" toSet="statements:" changeEvent="statementsChanged">statements</field>
</fields>'!

!CSequenceTile methodsFor: 'compiling' stamp: 'ar 12/11/2009 22:15'!
asParseNode: encoder

	^BlockNode new
		arguments: #()
		statements: (statements collect: [:each | each asParseNode: encoder])
		returns: false
		from: encoder! !

!CSequenceTile methodsFor: 'accessing' stamp: 'bf 7/21/2005 22:37'!
childTiles
	^statements! !

!CSequenceTile methodsFor: 'initialize' stamp: 'ar 3/8/2004 21:07'!
defaultCostume
	^CTileLibrary default sequenceTileCostume! !

!CSequenceTile methodsFor: 'evaluating' stamp: 'bf 5/18/2005 19:17'!
eval
	"Evaluate the receiver"
	^[
		| result |
		result := nil.
		self statements do: [:each | result := each eval].
		result
	]! !

!CSequenceTile methodsFor: 'initialize' stamp: 'ar 3/8/2004 20:51'!
initialize
	super initialize.
	self statements: CCollection new.
! !

!CSequenceTile methodsFor: 'events' stamp: 'bf 2/17/2005 17:57'!
onStatementsChanged
	<on: changed in: statements>

	"need a signal because our costume depends on it"
	self signal: #statementsModified! !

!CSequenceTile methodsFor: 'events' stamp: 'bf 2/17/2005 17:09'!
onStatementsModified
	<on: statementsModified>

	statements do: [:each | each parent: self].
	self compileNeeded.! !

!CSequenceTile methodsFor: 'accessing' stamp: 'ar 3/17/2004 15:14'!
statements
	"This method was automatically generated"
	<bewareOf: #statementsChanged>
	^self propertyValueAt: #statements! !

!CSequenceTile methodsFor: 'accessing' stamp: 'ar 3/11/2004 14:25'!
statements: aValue
	"This method was automatically generated"
	^self propertyValueAt: #statements put: aValue with: #statementsChanged! !

CTilePlayer defineFields: '<?xml  version="1.0" ?>
<fields>
	<field toGet="valueType" toSet="valueType:" changeEvent="valueTypeChanged">valueType</field>
	<field toGet="definition" toSet="definition:" changeEvent="definitionChanged">definition</field>
	<field toGet="parent" toSet="parent:" changeEvent="parentChanged">parent</field>
</fields>'!

!CTilePlayer methodsFor: 'private' stamp: 'bf 7/21/2005 22:38'!
abandonCostume
	| c |
	(c := myCostume) ifNotNil: [
		self costume: nil.
		c destroy].

	self childTiles do: [:each |
		each isTilePlayer ifFalse: [self error: 'Children must be tiles!!'].
		each abandonCostume]! !

!CTilePlayer methodsFor: 'compiling' stamp: 'bf 9/24/2004 13:53'!
asParseNode: encoder
	^self subclassResponsibility! !

!CTilePlayer methodsFor: 'accessing' stamp: 'bf 7/21/2005 22:37'!
childTiles
	^self subclassResponsibility ! !

!CTilePlayer methodsFor: 'compiling' stamp: 'bf 7/26/2005 11:46'!
compileNeeded

	| tile seen |
	tile := self.
	seen := IdentitySet new.
	[	(seen includes: tile) ifTrue: [^self error: 'circular tile hierarchy'].
		(tile handlesEvent: #compileNeeded)
			ifTrue: [^tile signal: #compileNeeded].
		seen add: tile.
		tile := tile parent.
		tile isNil
	] whileFalse.
! !

!CTilePlayer methodsFor: 'accessing' stamp: 'ar 3/22/2004 15:25'!
definition
	"Answer the definition of the receiver"
	<bewareOf: #definitionChanged>
	^self propertyValueAt: #definition! !

!CTilePlayer methodsFor: 'accessing' stamp: 'ar 3/22/2004 15:25'!
definition: aValue
	"Modify the receiver's definition"
	^self propertyValueAt: #definition put: aValue with: #definitionChanged! !

!CTilePlayer methodsFor: 'private' stamp: 'ar 8/5/2005 21:01'!
dragOffset
	"This is so that we can prevent dragging here"
	| pos |
	pos := container cursorPoint.
	^top - (pos y + 8)! !

!CTilePlayer methodsFor: 'evaluating' stamp: 'ar 3/7/2004 19:53'!
eval
	"Evaluate the receiver"
	^self value! !

!CTilePlayer methodsFor: 'accessing' stamp: 'ar 3/28/2004 16:04'!
fieldName
	"If the receiver represents an updating field, answer its name"
	^nil! !

!CTilePlayer methodsFor: 'testing' stamp: 'ar 3/7/2004 21:05'!
isArgumentTile
	^false! !

!CTilePlayer methodsFor: 'testing' stamp: 'ar 3/22/2004 18:11'!
isComplex
	"Am I a complex expression which needs parenthesis?"
	^false! !

!CTilePlayer methodsFor: 'testing' stamp: 'ar 3/7/2004 18:52'!
isMessageTile
	^false! !

!CTilePlayer methodsFor: 'testing' stamp: 'ar 3/7/2004 18:52'!
isPropertyTile
	^false! !

!CTilePlayer methodsFor: 'testing' stamp: 'tk 10/30/2004 13:35'!
isScriptor
	^ false! !

!CTilePlayer methodsFor: 'testing' stamp: 'ar 5/12/2005 17:25'!
isSeparator
	^false! !

!CTilePlayer methodsFor: 'testing' stamp: 'ar 3/22/2004 17:48'!
isStatementLike
	^false! !

!CTilePlayer methodsFor: 'testing' stamp: 'tk 11/20/2004 20:04'!
isTileLike
	"Answer whether the receiver represents a tile-scripting element (player or costume)"
	^true! !

!CTilePlayer methodsFor: 'testing' stamp: 'ar 3/8/2004 00:02'!
isTilePlayer
	^true! !

!CTilePlayer methodsFor: 'testing' stamp: 'ar 3/7/2004 18:52'!
isValueTile
	^false! !

!CTilePlayer methodsFor: 'private' stamp: 'ar 8/5/2005 21:02'!
onDrag
	"this really should be in the costume, but it is the player that is sent #onDrag"
	| pos offs |
	pos := container cursorPoint.
	offs := self dragOffset.
	self moveBy: 0@offs negated.
	super onDrag.
	(container notNil and: [pos = container cursorPoint])
		ifTrue: [self moveBy: 0@offs].
! !

!CTilePlayer methodsFor: 'converting' stamp: 'sw 5/18/2005 04:22'!
openAsWatcher
	"If possible, replace the receiver with a watcher.  If not, just answer self"

	^ self! !

!CTilePlayer methodsFor: 'accessing' stamp: 'bf 9/27/2004 16:37'!
parent
	"Answer the parent of the receiver"
	<bewareOf: #parentChanged>
	^self propertyValueAt: #parent! !

!CTilePlayer methodsFor: 'accessing' stamp: 'bf 9/27/2004 16:37'!
parent: aValue
	"Modify the receiver's parent"
	^self propertyValueAt: #parent put: aValue with: #parentChanged! !

!CTilePlayer methodsFor: 'queries' stamp: 'bf 7/31/2005 19:19'!
referencesDo: aBlock
	^self childTiles do: [:ea | ea referencesDo: aBlock]! !

!CTilePlayer methodsFor: 'converting' stamp: 'bf 7/22/2005 16:09'!
replacing: anotherTile
	"This tile is about to replace anotherTile."
	^self! !

!CTilePlayer methodsFor: 'accessing' stamp: 'sw 5/25/2005 03:43'!
tileExpression
	"Answer the tile to be added when I am dropped onto a value-pad in drag-and-drop scripting; normally this would be just myself; see reimplementors for usage."

	^ self! !

!CTilePlayer methodsFor: 'accessing' stamp: 'ar 3/17/2004 15:15'!
valueType
	"This method was automatically generated"
	<bewareOf: #valueTypeChanged>
	^self propertyValueAt: #valueType! !

!CTilePlayer methodsFor: 'accessing' stamp: 'ar 3/11/2004 14:27'!
valueType: aValue
	"This method was automatically generated"
	^self propertyValueAt: #valueType put: aValue with: #valueTypeChanged! !

CTilePlayer subclass: #CValueTile
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tweak-Tiles-Players'!

!CValueTile commentStamp: 'ar 3/7/2004 21:31' prior: 0!
A tile representing a literal value.!

CValueTile defineFields: '<?xml  version="1.0" ?>
<fields>
</fields>'!

!CValueTile methodsFor: 'compiling' stamp: 'bf 9/24/2004 18:37'!
asParseNode: encoder
	^encoder encodeScriptObject: value! !

!CValueTile methodsFor: 'accessing' stamp: 'bf 7/21/2005 22:49'!
childTiles
	^#()! !

!CValueTile methodsFor: 'initialize' stamp: 'ar 6/21/2005 14:09'!
defaultCostume
	^CTileLibrary default valueTileForType: self valueType! !

!CValueTile methodsFor: 'testing' stamp: 'ar 3/7/2004 18:52'!
isValueTile
	^true! !

!CValueTile methodsFor: 'events' stamp: 'bf 9/27/2004 17:17'!
onValueChanged
	<on: valueChanged>

	self compileNeeded.
! !

!CValueTile methodsFor: 'queries' stamp: 'bf 7/31/2005 19:16'!
referencesDo: aBlock
	^aBlock value: value! !

!CValueTile methodsFor: 'initialize' stamp: 'ar 6/21/2005 15:03'!
setupCostume
	super setupCostume.
	self borderWidth := 1.
	self borderStyle := #simple.
	self borderColor: CWidgetLibrary default frameColor.! !

!CValueTile methodsFor: 'accessing' stamp: 'ar 3/7/2004 21:21'!
valueType
	^super valueType ifNil:["use the concrete type" self value class].! !

!CValueTile methodsFor: 'accessing' stamp: 'ar 6/21/2005 14:07'!
valueType: v

	^super valueType: v! !

CTilePlayer subclass: #CVariableTile
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tweak-Tiles-Players'!

!CVariableTile commentStamp: 'ar 3/7/2004 21:32' prior: 0!
A tile representing an "argument slot" in some message. It can contain an arbitrary expression which is evaluated in order to retrieve the actual argument value.!

CVariableTile defineFields: '<?xml  version="1.0" ?>
<fields>
	<field toGet="expression" toSet="expression:" changeEvent="expressionChanged">expression</field>
</fields>'!

!CVariableTile methodsFor: 'compiling' stamp: 'bf 9/24/2004 13:53'!
asParseNode: encoder
	^ expression asParseNode: encoder! !

!CVariableTile methodsFor: 'accessing' stamp: 'bf 7/21/2005 22:38'!
childTiles
	^{expression}! !

!CVariableTile methodsFor: 'initialize' stamp: 'ar 3/7/2004 21:38'!
defaultCostume
	^CTileLibrary default argumentTileCostume
! !

!CVariableTile methodsFor: 'evaluating' stamp: 'ar 3/7/2004 21:07'!
eval
	^self expression eval! !

!CVariableTile methodsFor: 'accessing' stamp: 'ar 3/17/2004 15:02'!
expression
	"This method was automatically generated"
	<bewareOf: #expressionChanged>
	^self propertyValueAt: #expression! !

!CVariableTile methodsFor: 'accessing' stamp: 'ar 3/11/2004 14:18'!
expression: aValue
	"This method was automatically generated"
	^self propertyValueAt: #expression put: aValue with: #expressionChanged! !

!CVariableTile methodsFor: 'initialize' stamp: 'ar 3/19/2004 18:30'!
initialize
	super initialize.
! !

!CVariableTile methodsFor: 'testing' stamp: 'ar 3/7/2004 21:05'!
isArgumentTile
	^true! !

!CVariableTile methodsFor: 'events' stamp: 'bf 9/27/2004 16:41'!
onExpressionChanged
	<on: expressionChanged>

	expression parent: self.
	self compileNeeded.! !

CPlayer subclass: #CTileReadout
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tweak-Tiles-Inspector'!

CTileReadout defineFields: '<?xml  version="1.0" ?>
<fields>
	<field toGet="label" toSet="label:" changeEvent="labelChanged">label</field>
	<field toGet="editor" toSet="editor:" changeEvent="editorChanged">editor</field>
	<field toGet="field" toSet="field:" changeEvent="fieldChanged">field</field>
	<field toGet="fieldType" toSet="fieldType:" changeEvent="fieldTypeChanged">fieldType</field>
</fields>'!

!CTileReadout methodsFor: 'private' stamp: 'ar 3/23/2004 12:54'!
defaultEditor
	| newEditor |
	newEditor := CInputField new.
	newEditor converter: self.
	newEditor textInset: 2.
	^newEditor! !

!CTileReadout methodsFor: 'accessing' stamp: 'ar 3/18/2004 13:47'!
editor
	"Answer the editor of the receiver"
	<bewareOf: #editorChanged>
	^self propertyValueAt: #editor! !

!CTileReadout methodsFor: 'accessing' stamp: 'ar 3/18/2004 13:47'!
editor: aValue
	"Modify the receiver's editor"
	^self propertyValueAt: #editor put: aValue with: #editorChanged! !

!CTileReadout methodsFor: 'private' stamp: 'ar 3/25/2004 21:24'!
editorForType: aType
	| newEditor |
	aType ifNil:[^self defaultEditor].
	(aType includesBehavior: Boolean) ifTrue:[
		newEditor := CDropDownList new.
		newEditor add: true.
		newEditor add: false.
		^newEditor
	].
	(aType includesBehavior: Integer) ifTrue:[
		newEditor := CSpinner new.
		newEditor min: nil; max: nil.
		newEditor label: nil.
		^newEditor
	].
	(aType includesBehavior: String) ifTrue:[
		newEditor := self defaultEditor.
		newEditor converter: CStringConverter new.
		^newEditor
	].
	(aType includesBehavior: Text) ifTrue:[
		newEditor := self defaultEditor.
		newEditor converter: nil.
		^newEditor
	].
	(aType includesBehavior: Date) ifTrue:[
		newEditor := self defaultEditor.
		newEditor converter: CDateConverter new.
		^newEditor
	].
	(aType includesBehavior: Number) ifTrue:[
		newEditor := self defaultEditor.
		newEditor converter: CNumberConverter new.
		^newEditor
	].
	^self defaultEditor! !

!CTileReadout methodsFor: 'accessing' stamp: 'ar 3/18/2004 14:11'!
field
	"Answer the field of the receiver"
	<bewareOf: #fieldChanged>
	^self propertyValueAt: #field! !

!CTileReadout methodsFor: 'accessing' stamp: 'ar 3/18/2004 14:11'!
field: aValue
	"Modify the receiver's field"
	^self propertyValueAt: #field put: aValue with: #fieldChanged! !

!CTileReadout methodsFor: 'accessing' stamp: 'ar 3/19/2004 15:59'!
fieldType
	"Answer the fieldType of the receiver"
	<bewareOf: #fieldTypeChanged>
	^self propertyValueAt: #fieldType! !

!CTileReadout methodsFor: 'accessing' stamp: 'ar 3/19/2004 15:59'!
fieldType: aValue
	"Modify the receiver's fieldType"
	^self propertyValueAt: #fieldType put: aValue with: #fieldTypeChanged! !

!CTileReadout methodsFor: 'initialize' stamp: 'ar 3/22/2004 14:41'!
initialize
	super initialize.
	self define: #label as: CLabel new.
	self define: #editor as: nil.
! !

!CTileReadout methodsFor: 'private' stamp: 'ar 3/27/2004 21:15'!
installFieldType: newType
	| oldEditor newEditor |
	self fieldType: newType.
	oldEditor := editor.
	newEditor := self editorForType: newType.
	newEditor hResizing: #rigid; vResizing: #shrinkWrap.
	newEditor width: 80.
	oldEditor 
		ifNil:[elements addLast: newEditor]
		ifNotNil:[elements replace: oldEditor with: newEditor].
	editor := newEditor.
	oldEditor ifNotNil:[oldEditor destroy].! !

!CTileReadout methodsFor: 'accessing' stamp: 'ar 3/18/2004 13:46'!
label
	"Answer the label of the receiver"
	<bewareOf: #labelChanged>
	^self propertyValueAt: #label! !

!CTileReadout methodsFor: 'accessing' stamp: 'ar 3/18/2004 13:46'!
label: aValue
	"Modify the receiver's label"
	^self propertyValueAt: #label put: aValue with: #labelChanged! !

!CTileReadout methodsFor: 'events' stamp: 'ar 3/19/2004 16:02'!
onValueChanged	
	| newType |
	<on: valueChanged>
	value ifNotNil:[label value: value fieldName].
	newType := value valueType.
	newType == fieldType ifFalse:[self installFieldType: newType].
	editor player: value.! !

!CTileReadout methodsFor: 'events' stamp: 'ar 3/18/2004 15:03'!
printValue: aValue
	^aValue displayText! !

!CTileReadout methodsFor: 'events' stamp: 'ar 3/26/2004 15:51'!
readValue: aString
	^CCompiler evaluate: aString asString! !

!CTileReadout methodsFor: 'initialize' stamp: 'ar 4/30/2004 15:55'!
setupCostume
	super setupCostume.

	self color: CWidgetLibrary default paneColor.
	self borderStyle: #raised.
	self borderWidth: 1.

	self layout: CTableLayout new.
	self layout listDirection: #leftToRight.
	self hResizing: #spaceFill.
	self vResizing: #rigid.
	self height: 22.

	label hResizing: #spaceFill.
	label vResizing: #spaceFill.
	label clipping: true.
	label borderStyle: #simple.
	label borderWidth: 1.
	label borderColor: CWidgetLibrary default frameColor.
	label textInset: 2; textEmphasis: 1.
	self add: label.
! !

CPlayer subclass: #CTileType
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tweak-Tiles-Types'!

CTileType subclass: #CBlockType
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tweak-Tiles-Types'!

CBlockType defineFields: '<?xml  version="1.0" ?>
<fields>
</fields>'!

!CBlockType class methodsFor: 'converting' stamp: 'bf 2/18/2005 12:29'!
print: aValue
	self shouldNotImplement! !

!CBlockType class methodsFor: 'converting' stamp: 'bf 2/18/2005 12:29'!
read: aString
	self shouldNotImplement.! !

!CBlockType class methodsFor: '*Tweak-Tiles-Definitions' stamp: 'bf 2/18/2005 13:30'!
valueTileMaker
	"the tile class used for representing objects of our type"
	^CSequenceTile! !

CTileType subclass: #CColorType
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tweak-Tiles-Types'!

CColorType defineFields: '<?xml  version="1.0" ?>
<fields>
</fields>'!

!CColorType class methodsFor: 'accessing' stamp: 'ar 3/22/2004 18:34'!
defaultValue
	^Color white! !

!CColorType class methodsFor: 'as yet unclassified' stamp: 'ar 3/21/2004 19:25'!
print: aColor
	^aColor name! !

!CColorType class methodsFor: 'as yet unclassified' stamp: 'ar 3/21/2004 19:25'!
read: aString
	^Color perform: aString asSymbol! !

CTileType subclass: #CEnumType
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tweak-Tiles-Types'!

CEnumType subclass: #CBooleanType
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tweak-Tiles-Types'!

CBooleanType defineFields: '<?xml  version="1.0" ?>
<fields>
</fields>'!

!CBooleanType class methodsFor: 'accessing' stamp: 'ar 3/21/2004 20:27'!
enumValues
	^{true. false}! !

!CBooleanType class methodsFor: 'converting' stamp: 'ar 3/21/2004 16:57'!
read: aString
	"Read a textual representation of some value"
	^aString = 'true'! !

CEnumType defineFields: '<?xml  version="1.0" ?>
<fields>
</fields>'!

!CEnumType class methodsFor: 'accessing' stamp: 'ar 3/22/2004 18:34'!
defaultValue
	^self enumValues first! !

!CEnumType class methodsFor: 'accessing' stamp: 'ar 3/21/2004 18:51'!
enumValues
	^#()! !

CEnumType subclass: #CSoundNameType
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tweak-Tiles-Types'!

CSoundNameType defineFields: '<?xml  version="1.0" ?>
<fields>
</fields>'!

!CSoundNameType class methodsFor: 'accessing' stamp: 'ar 3/21/2004 18:51'!
enumValues
	^SampledSound soundNames! !

!CSoundNameType class methodsFor: 'converting' stamp: 'ar 3/21/2004 17:27'!
print: aValue
	^aValue! !

!CSoundNameType class methodsFor: 'converting' stamp: 'ar 3/21/2004 16:58'!
read: aString
	"Read a textual representation of some value"
	^aString asString! !

CEnumType subclass: #CSymbolType
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tweak-Tiles-Types'!

CSymbolType subclass: #CBorderStyleType
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tweak-Tiles-Types'!

CBorderStyleType defineFields: '<?xml  version="1.0" ?>
<fields>
</fields>'!

!CBorderStyleType class methodsFor: 'accessing' stamp: 'ar 12/19/2004 13:08'!
enumValues
	^#(none simple inset framed inner outer raised 
	complexAltFramed complexAltInset complexAltRaised 
	complexFramed complexInset complexRaised)! !

CSymbolType subclass: #CButtonStateType
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tweak-Tiles-Types'!

CButtonStateType defineFields: '<?xml  version="1.0" ?>
<fields>
</fields>'!

!CButtonStateType class methodsFor: 'as yet unclassified' stamp: 'ar 3/28/2004 22:40'!
enumValues
	^#(normal over pressed disabled)! !

CSymbolType subclass: #CEventType
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tweak-Tiles-Types'!

CEventType defineFields: '<?xml  version="1.0" ?>
<fields>
</fields>'!

!CEventType class methodsFor: 'accessing' stamp: 'bf 3/9/2005 15:58'!
enumValues
	^CPrimitiveCostume eventsGenerated! !

CSymbolType subclass: #CFillStyleType
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tweak-Tiles-Types'!

CFillStyleType defineFields: '<?xml  version="1.0" ?>
<fields>
</fields>'!

!CFillStyleType class methodsFor: 'accessing' stamp: 'ar 3/21/2004 20:30'!
enumValues
	^#(none solid horizontalGradient verticalGradient diagonalGradient linearGradient radialGradient)! !

CSymbolType subclass: #CPenTrailStyleType
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tweak-Tiles-Types'!

CPenTrailStyleType defineFields: '<?xml  version="1.0" ?>
<fields>
</fields>'!

!CPenTrailStyleType class methodsFor: 'accessing' stamp: 'ar 6/22/2005 10:26'!
enumValues
	^#(lines arrows arrowheads dots)! !

CSymbolType subclass: #CResizingType
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tweak-Tiles-Types'!

CResizingType defineFields: '<?xml  version="1.0" ?>
<fields>
</fields>'!

!CResizingType class methodsFor: 'as yet unclassified' stamp: 'pe 2/17/2006 12:16'!
enumValues
	^#(rigid spaceFill shrinkWrap).! !

CSymbolType subclass: #CRotationStyleType
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tweak-Tiles-Types'!

CRotationStyleType defineFields: '<?xml  version="1.0" ?>
<fields>
</fields>'!

!CRotationStyleType class methodsFor: 'accessing' stamp: 'ar 6/26/2005 13:16'!
enumValues
	^#(smooth #'left-right' #'up-down' #none)! !

CSymbolType defineFields: '<?xml  version="1.0" ?>
<fields>
</fields>'!

!CSymbolType class methodsFor: 'converting' stamp: 'ar 3/21/2004 17:26'!
print: aValue
	^aValue! !

!CSymbolType class methodsFor: 'converting' stamp: 'ar 3/27/2004 19:42'!
read: aString
	^self enumValues detect:[:any| any = aString] ifNone:[self defaultValue]. "@@@ should this raise an error???"! !

CTileType subclass: #CNumberType
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tweak-Tiles-Types'!

CNumberType subclass: #CIntegerType
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tweak-Tiles-Types'!

CIntegerType defineFields: '<?xml  version="1.0" ?>
<fields>
</fields>'!

!CIntegerType class methodsFor: 'operations' stamp: 'bf 5/10/2005 18:43'!
assignmentOperators

	^#(+ - *)! !

!CIntegerType class methodsFor: 'converting' stamp: 'ar 4/20/2005 13:21'!
read: aString
	"Read a textual representation of some value"
	^Integer readFrom: aString readStream! !

CNumberType defineFields: '<?xml  version="1.0" ?>
<fields>
</fields>'!

!CNumberType class methodsFor: 'operations' stamp: 'bf 5/10/2005 18:38'!
assignmentOperators

	^#(+ - * roundTo:)! !

!CNumberType class methodsFor: 'accessing' stamp: 'ar 3/22/2004 18:34'!
defaultValue
	^1! !

!CNumberType class methodsFor: 'converting' stamp: 'ar 3/21/2004 16:58'!
read: aString
	"Read a textual representation of some value"
	^Number readFrom: aString! !

CTileType subclass: #CPlayerType
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tweak-Tiles-Types'!

CPlayerType defineFields: '<?xml  version="1.0" ?>
<fields>
</fields>'!

!CPlayerType class methodsFor: 'defaults' stamp: 'bf 12/9/2005 10:20'!
defaultValue
	^CPlayer new! !

!CPlayerType class methodsFor: 'reading' stamp: 'ar 6/27/2005 13:50'!
read: aString
	^nil! !

CPlayerType subclass: #CSpriteCostumeType
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tweak-Tiles-Types'!

!CSpriteCostumeType commentStamp: '<historical>' prior: 0!
The sprite's costume type.!

CSpriteCostumeType defineFields: '<?xml  version="1.0" ?>
<fields>
</fields>'!

CTileType subclass: #CResourceType
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tweak-Tiles-Types'!

CResourceType subclass: #CImageResourceType
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tweak-Tiles-Types'!

CImageResourceType defineFields: '<?xml  version="1.0" ?>
<fields>
</fields>'!

!CImageResourceType class methodsFor: 'accessing' stamp: 'ar 4/3/2004 14:07'!
read: aString
	| aStream |
	aString isEmpty ifTrue:[^nil].
	aStream := aString asUrl contentStream.
	^[CImageResource readFrom: aStream] ensure:[aStream close].! !

CResourceType defineFields: '<?xml  version="1.0" ?>
<fields>
</fields>'!

!CResourceType class methodsFor: 'accessing' stamp: 'ar 4/3/2004 14:07'!
print: aValue
	"Print a textual representation of the given value"
	aValue ifNil:[^''].
	^aValue resourceUrl! !

CTileType subclass: #CScriptNameType
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tweak-Tiles-Types'!

CScriptNameType defineFields: '<?xml  version="1.0" ?>
<fields>
</fields>'!

!CScriptNameType class methodsFor: 'accessing' stamp: 'ar 4/20/2005 13:31'!
defaultValue
	^#emptyScript! !

!CScriptNameType class methodsFor: 'accessing' stamp: 'ar 4/20/2005 13:32'!
print: aValue
	"Print a textual representation of the given value"
	^aValue! !

!CScriptNameType class methodsFor: 'accessing' stamp: 'ar 4/20/2005 13:32'!
read: aString
	"Read a textual representation of some value"
	^aString asSymbol! !

CTileType subclass: #CStringType
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tweak-Tiles-Types'!

CStringType defineFields: '<?xml  version="1.0" ?>
<fields>
</fields>'!

!CStringType class methodsFor: 'operations' stamp: 'bf 5/10/2005 16:16'!
assignmentOperators

	^#(#,)! !

!CStringType class methodsFor: 'accessing' stamp: 'ar 3/22/2004 18:35'!
defaultValue
	^''! !

!CStringType class methodsFor: 'converting' stamp: 'ar 3/21/2004 17:26'!
print: aValue
	^aValue! !

!CStringType class methodsFor: 'converting' stamp: 'ar 3/21/2004 16:58'!
read: aString
	"Read a textual representation of some value"
	^aString asString! !

CTileType defineFields: '<?xml  version="1.0" ?>
<fields>
</fields>'!

!CTileType class methodsFor: 'converting' stamp: 'ar 6/23/2005 14:15'!
asConcreteTypeFor: anObject
	"If this type is an abstract type which needs to be reified in the context of a concrete receiver, answer the concrete type to be used for the object. By default, just answer self. Examples for abstract types include the CScriptNameType (which needs to query the object for which scripts are accessbile) and the CValueType (which is polymorphic and needs to query the object as well)."
	^self! !

!CTileType class methodsFor: 'defaults' stamp: 'ar 6/23/2005 14:23'!
asTileType: aClass
	"Answer the type from a class"

	(aClass includesBehavior: BlockContext) ifTrue:[^CBlockType].
	(aClass includesBehavior: Color) ifTrue:[^CColorType].
	(aClass includesBehavior: Boolean) ifTrue:[^CBooleanType].
	(aClass includesBehavior: Integer) ifTrue:[^CIntegerType].
	(aClass includesBehavior: Number) ifTrue:[^CNumberType].
	(aClass includesBehavior: CPlayer) ifTrue:[^CPlayerType].
	(aClass includesBehavior: String) ifTrue:[^CStringType].
	(aClass includesBehavior: Point) ifTrue:[^CVectorType].
	(aClass includesBehavior: Form) ifTrue: [^CImageResourceType].
	^aClass! !

!CTileType class methodsFor: 'operations' stamp: 'bf 5/10/2005 16:12'!
assignmentOperators
	"answer a list of modifying assignment operators appropriate for this type."

	^#()! !

!CTileType class methodsFor: 'type tests' stamp: 'ar 3/25/2004 22:39'!
canCoerceType: aType
	^aType includesBehavior: self! !

!CTileType class methodsFor: 'defaults' stamp: 'sw 5/17/2005 18:21'!
defaultTileTypeFor: anObject
	"Answer the class representing the type of the given object."

	anObject isBlock ifTrue:[^CBlockType].
	anObject isColor ifTrue:[^CColorType].
	(anObject isKindOf: Boolean) ifTrue:[^CBooleanType].
	anObject isInteger ifTrue:[^CIntegerType].
	(anObject isKindOf: Number) ifTrue:[^CNumberType].
	(anObject isPlayer) ifTrue:[^CPlayerType].
	anObject isString ifTrue:[^CStringType].
	anObject isPoint ifTrue:[^CVectorType].
	anObject isForm ifTrue: [^CImageResourceType].
	^anObject class! !

!CTileType class methodsFor: 'defaults' stamp: 'bf 12/9/2005 10:19'!
defaultValue
	^self subclassResponsibility! !

!CTileType class methodsFor: 'converting' stamp: 'ar 3/21/2004 16:59'!
print: aValue
	"Print a textual representation of the given value"
	^aValue printString! !

!CTileType class methodsFor: 'converting' stamp: 'ar 3/21/2004 16:59'!
read: aString
	"Read a textual representation of some value"
	^self subclassResponsibility! !

!CTileType class methodsFor: '*Tweak-Tiles-Definitions' stamp: 'bf 2/18/2005 13:30'!
valueTileMaker
	"the tile class used for representing objects of our type"
	^CValueTile! !

CTileType subclass: #CValueType
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tweak-Tiles-Types'!

!CValueType commentStamp: '<historical>' prior: 0!
I represent the (polymorphic) value type of some object.!

CValueType defineFields: '<?xml  version="1.0" ?>
<fields>
	<field toGet="receiver" toSet="receiver:" changeEvent="receiverChanged">receiver</field>
</fields>'!

!CValueType class methodsFor: 'converting' stamp: 'ar 6/23/2005 14:23'!
asConcreteTypeFor: anObject
	"Make it simple and just answer the type directly"
	^self asTileType: anObject valueType! !

!CValueType methodsFor: 'accessing' stamp: 'ar 6/23/2005 14:15'!
receiver
	"Answer the receiver of the receiver"
	<bewareOf: #receiverChanged>
	^self propertyValueAt: #receiver! !

!CValueType methodsFor: 'accessing' stamp: 'ar 6/23/2005 14:15'!
receiver: aValue
	"Modify the receiver's receiver"
	^self propertyValueAt: #receiver put: aValue with: #receiverChanged! !

CTileType subclass: #CVectorType
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tweak-Tiles-Types'!

!CVectorType commentStamp: '<historical>' prior: 0!
I represent a generic vector type.!

CVectorType subclass: #CPointType
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tweak-Tiles-Types'!

CPointType defineFields: '<?xml  version="1.0" ?>
<fields>
</fields>'!

!CPointType class methodsFor: 'accessing' stamp: 'ar 5/12/2005 15:03'!
defaultValue
	"Answer the default value for this type"
	^0@0! !

!CPointType class methodsFor: 'accessing' stamp: 'ar 5/12/2005 16:13'!
elementNames
	"CPointType does NOT answer its element names since it does not implement x: and y: messages."
	^nil! !

!CPointType class methodsFor: 'accessing' stamp: 'ar 5/12/2005 14:53'!
elementType
	"Answer the type for the elements of the vector. 
	If the elements cannot be typed, answer nil."
	^CNumberType! !

!CPointType class methodsFor: 'accessing' stamp: 'ar 5/12/2005 16:14'!
read: aString
	"Read a textual representation of some value"
	^Point readFrom: aString! !

CVectorType defineFields: '<?xml  version="1.0" ?>
<fields>
</fields>'!

!CVectorType class methodsFor: 'accessing' stamp: 'ar 5/12/2005 14:03'!
assignmentOperators
	^(self elementType ifNil:[^#()]) assignmentOperators! !

!CVectorType class methodsFor: 'accessing' stamp: 'ar 5/12/2005 14:47'!
defaultValue
	^#()! !

!CVectorType class methodsFor: 'accessing' stamp: 'ar 5/12/2005 13:15'!
elementNames
	"Answer the names for the elements of the vector. 
	If the elements cannot be named, answer nil."
	^nil! !

!CVectorType class methodsFor: 'accessing' stamp: 'ar 5/12/2005 13:15'!
elementType
	"Answer the type for the elements of the vector. 
	If the elements cannot be typed, answer nil."
	^nil! !

!CVectorType class methodsFor: 'accessing' stamp: 'ar 5/12/2005 14:35'!
vectorFieldsOf: aVector
	"Answer the fields for accessing this vector"
	| names def |
	(names := self elementNames) ifNotNil:[
		^names collect:[:str|
			def := CFieldDefinition new.
			def name: str asSymbol.
			def toGet: str asSymbol.
			def toSet: (str copyWith: $:) asSymbol.
			def]].
	aVector class isVariable ifTrue:[
		^(1 to: aVector size) collect:[:i|
			def := CFieldDefinition new.
			def name: i printString.
			def toGet: #at:.
			def toSet: #at:put:.
			def accessKey: i.
			def].
	] ifFalse:[
		| allNames |
		allNames := aVector class allInstVarNames.
		^(1 to: aVector class instSize) collect:[:i|
			def := CFieldDefinition new.
			def name: (allNames at: i).
			def toGet: #instVarAt:.
			def toSet: #instVarAt:put:.
			def accessKey: i.
			def].
	].! !

CPlayer subclass: #CTileViewer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tweak-Tiles-Viewer'!

!CTileViewer commentStamp: 'tk 3/7/2005 09:24' prior: 0!
The main use of the Tile Viewer is to display script commands that the user can drag out and put into a script.  However, the tiles are 'alive' when they are in the Viewer.  Click on the (!!) button to cause a tile to do its action.  
	Change the value on the right hand side of a tile, and that part of the object will change.  To change a number, click the up/down spinner arrows. Or, select the number, press delete, type a new number, and press return.
	Get a halo on an object by right-click, or Command-click.  Click the aqua-blue handle on the left to get a Tile Viewer for that object.

### Allow Users to see the above explanation ###!

CTileViewer defineFields: '<?xml  version="1.0" ?>
<fields>
	<field toGet="target" toSet="target:" changeEvent="targetChanged">target</field>
	<field toGet="pages" toSet="pages:" changeEvent="pagesChanged">pages</field>
	<field toGet="header" toSet="header:" changeEvent="headerChanged">header</field>
	<field toGet="holder" toSet="holder:" changeEvent="holderChanged">holder</field>
</fields>'!

!CTileViewer methodsFor: 'adding' stamp: 'ar 6/26/2005 13:38'!
addPage: aPage
	| existing |
	existing := pages detect:[:any| any name = aPage name] ifNone:[nil].
	existing ifNil:[pages add: aPage] ifNotNil:[existing addAll: aPage elements]! !

!CTileViewer methodsFor: 'accessing' stamp: 'ar 3/21/2004 23:27'!
header
	"Answer the header of the receiver"
	<bewareOf: #headerChanged>
	^self propertyValueAt: #header! !

!CTileViewer methodsFor: 'accessing' stamp: 'ar 3/21/2004 23:27'!
header: aValue
	"Modify the receiver's header"
	^self propertyValueAt: #header put: aValue with: #headerChanged! !

!CTileViewer methodsFor: 'accessing' stamp: 'ar 3/21/2004 23:28'!
holder
	"Answer the holder of the receiver"
	<bewareOf: #holderChanged>
	^self propertyValueAt: #holder! !

!CTileViewer methodsFor: 'accessing' stamp: 'ar 3/21/2004 23:28'!
holder: aValue
	"Modify the receiver's holder"
	^self propertyValueAt: #holder put: aValue with: #holderChanged! !

!CTileViewer methodsFor: 'initialize' stamp: 'ar 3/22/2004 14:42'!
initialize
	super initialize.
	self define: #header as: CViewerCategoryHeader new.
	self define: #pages as: CCollection new.
	pages wrapCursor: true.
	self define: #holder as: CHolder new.
! !

!CTileViewer methodsFor: 'private' stamp: 'bf 2/18/2005 13:23'!
messageDefinitionFor: aSelector in: aClass
	"Answer a message tile for the given selector in aClass"
	| source argNames msgDef argDef |
	source := aClass sourceCodeAt: aSelector.
	argNames := aClass parserClass new parseArgs: source notifying: nil.
	msgDef := CMessageDefinition new.
	msgDef selector: aSelector.
	argNames do:[:each|
		argDef := CArgumentDefinition new.
		argDef name: each.
		msgDef arguments add: argDef.
	].
	^msgDef! !

!CTileViewer methodsFor: 'events' stamp: 'ar 6/18/2005 18:44'!
onCloseRequest
	<on: closeRequest in: header>
	self close.! !

!CTileViewer methodsFor: 'events' stamp: 'bf 12/7/2005 11:39'!
onClosed
	<on: closed>
	target := nil.
! !

!CTileViewer methodsFor: 'events' stamp: 'tak 5/26/2005 17:08'!
onCursorChanged
	| selected |
	<on: cursorChanged in: pages>
	holder removeAll.
	selected := pages selection.
	selected ifNil:[^self].
	holder add: pages selection.
	header label: selected name translated.! !

!CTileViewer methodsFor: 'events' stamp: 'ar 3/21/2004 23:34'!
onGotoNext
	<on: gotoNext in: header>
	pages next.! !

!CTileViewer methodsFor: 'events' stamp: 'ar 3/21/2004 23:34'!
onGotoPrev
	<on: gotoPrev in: header>
	pages prev.! !

!CTileViewer methodsFor: 'events' stamp: 'ar 8/19/2005 10:09'!
onLabelHit
	| result menu |
	<on: mouseDown in: header>
	menu := CMenu new.
	menu chooseFrom: (pages collect:[:each| {each name. each}]) in: menu.
	menu menuItems first separator: true.
	menu menuItems third separator: true.
	menu hResizing: #spaceFill.
	menu width: header width.
	menu openAt: (header localToGlobal: 0@header height).
	result := menu run.
	result ifNotNil:[pages selection: result].
! !

!CTileViewer methodsFor: 'events' stamp: 'bf 12/7/2005 14:46'!
onScriptModified: selector 
	| scriptsPage scriptTiles current |

	scriptsPage := pages detect: [:each | each name = 'scripts'] ifNone: [^self].

	"try partial rebuild"
	current := scriptsPage elements
		detect: [:each | each messageTile selector = selector] ifNone: [nil].
	current ifNotNil: [
		(target class includesSelector: selector) 
			ifFalse: [^scriptsPage elements remove: current].
		"update current here if necessary"
		^self].

	"full rebuild"
	scriptTiles := Array streamContents: [:strm |
		strm nextPut: ((self messageDefinitionFor: #emptyScript in: CObject)
			newFor: target).
		target scriptsWithClassDo: [:sel :cls |
			strm nextPut: ((self messageDefinitionFor: sel in: cls)
					newFor: target)]].
	scriptsPage contents: scriptTiles.! !

!CTileViewer methodsFor: 'events' stamp: 'bf 12/7/2005 14:45'!
onScriptModified: selector event: anEvent
	<on: scriptModified in: target>

	"need to catch all events"
	anEvent withDroppedEventsDo: [:evt |
		self onScriptModified: evt arguments first].! !

!CTileViewer methodsFor: 'events' stamp: 'bf 12/7/2005 14:46'!
onTargetChanged
	<on: targetChanged>

	target ifNotNil: [
		self onScriptModified: nil.
		self onVariableModified: nil].! !

!CTileViewer methodsFor: 'events' stamp: 'bf 12/7/2005 11:23'!
onUpdateVariablesPage
	<on: updateVariablesPage>

	self rebuildVariablesPage! !

!CTileViewer methodsFor: 'events' stamp: 'bf 12/7/2005 14:50'!
onVariableModified: variableName
	"A variable of the given name was added to or removed from the target.  Rather than attempt to be too clever for the moment anyway, simply rebuild my variables page"

	self signal: #updateVariablesPage! !

!CTileViewer methodsFor: 'events' stamp: 'bf 12/7/2005 14:49'!
onVariableModified: variableName event: anEvent
	"Triggered when a variable has been added, removed, or renamed in the target"
	<on: variableModified in: target>

	"need to catch all events"
	anEvent withDroppedEventsDo: [:evt |
		self onVariableModified: evt arguments first]! !

!CTileViewer methodsFor: 'accessing' stamp: 'ar 3/21/2004 23:26'!
pages
	"Answer the pages of the receiver"
	<bewareOf: #pagesChanged>
	^self propertyValueAt: #pages! !

!CTileViewer methodsFor: 'accessing' stamp: 'ar 3/21/2004 23:26'!
pages: aValue
	"Modify the receiver's pages"
	^self propertyValueAt: #pages put: aValue with: #pagesChanged! !

!CTileViewer methodsFor: 'initialize' stamp: 'sw 3/11/2005 02:47'!
pointToCategoryNamed: aName
	"Point the receiver at the category of the given name"

	| aPage |
	aPage := pages detect: [:p | p name = aName] ifNone: [self error: 'no such category in this viewer'].
	pages cursor: (pages indexOf: aPage)
! !

!CTileViewer methodsFor: 'private' stamp: 'sw 3/13/2005 18:47'!
rebuildVariablesPage
	"Rebuild my variables page"

	| variablesPage variablesTiles |
	target isUniClass ifFalse: [^ self].
	variablesPage := pages detect: [:each | each name = 'variables'].
	variablesTiles := Array streamContents: [:strm |
		target class classFields do:
			[:aField |
				(aField isPrivateField not and: [aField type notNil]) ifTrue:
					[strm nextPut: ((self slotDefinitionFor: aField)
						newFor: target)]]].
	variablesPage contents: variablesTiles! !

!CTileViewer methodsFor: 'initialize' stamp: 'bf 3/14/2005 20:10'!
setupCostume
	super setupCostume.
	self asPrimCostume localCache: CFormCache new.

	color := CWidgetLibrary default paneColor.
	self borderStyle: #simple.
	self borderColor: CWidgetLibrary default frameColor.
	self borderWidth: 1.
	self roundCorners: true.

	self layout: CTableLayout new.
	self hResizing: #spaceFill; vResizing: #spaceFill.
	self layout cellPositioning: #topLeft.
	self layout layoutInset: 4; cellInset: 2.

	self add: header.

	holder layout: CTableLayout new.
	holder layout ifNotNil: [holder layout layoutInset: 4].	"if CTextEditor, it has a nil layout"
	holder hResizing: #spaceFill; vResizing: #spaceFill.
	holder roundCorners: true.
	holder borderWidth: 1; borderStyle: #simple.
	holder borderColor: CWidgetLibrary default frameColor.
	holder color: Color white.

	self add: holder
! !

!CTileViewer methodsFor: 'private' stamp: 'sw 3/11/2005 13:51'!
slotDefinitionFor: aFieldDefinition
	"Answer a CSlotDefinition corresponding to the given CFieldDefinition"

	| slotDef |
	slotDef := CSlotDefinition new.
	slotDef name: aFieldDefinition name.
	slotDef selector: aFieldDefinition toGet.
	slotDef type: aFieldDefinition type.
	^ slotDef! !

!CTileViewer methodsFor: 'accessing' stamp: 'bf 10/4/2004 15:06'!
target
	"Answer the target of the receiver"
	<bewareOf: #targetChanged>
	^self propertyValueAt: #target! !

!CTileViewer methodsFor: 'accessing' stamp: 'bf 10/4/2004 15:06'!
target: aValue
	"Modify the receiver's target"
	^self propertyValueAt: #target put: aValue with: #targetChanged! !

CPlayer subclass: #CViewerCategoryHeader
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tweak-Tiles-Viewer'!

CViewerCategoryHeader defineFields: '<?xml  version="1.0" ?>
<fields>
	<field toGet="closeButton" toSet="closeButton:" changeEvent="closeButtonChanged">closeButton</field>
	<field toGet="prevArrow" toSet="prevArrow:" changeEvent="prevArrowChanged">prevArrow</field>
	<field toGet="nextArrow" toSet="nextArrow:" changeEvent="nextArrowChanged">nextArrow</field>
	<field toGet="categoryLabel" toSet="categoryLabel:" changeEvent="categoryLabelChanged">categoryLabel</field>
</fields>'!

!CViewerCategoryHeader methodsFor: 'accessing' stamp: 'ar 3/21/2004 23:09'!
categoryLabel
	"Answer the categoryLabel of the receiver"
	<bewareOf: #categoryLabelChanged>
	^self propertyValueAt: #categoryLabel! !

!CViewerCategoryHeader methodsFor: 'accessing' stamp: 'ar 3/21/2004 23:09'!
categoryLabel: aValue
	"Modify the receiver's categoryLabel"
	^self propertyValueAt: #categoryLabel put: aValue with: #categoryLabelChanged! !

!CViewerCategoryHeader methodsFor: 'accessing' stamp: 'tk 8/12/2004 14:23'!
closeButton
	"Answer the closeButton of the receiver"
	<bewareOf: #closeButtonChanged>
	^self propertyValueAt: #closeButton! !

!CViewerCategoryHeader methodsFor: 'accessing' stamp: 'tk 8/12/2004 14:23'!
closeButton: aValue
	"Modify the receiver's closeButton"
	^self propertyValueAt: #closeButton put: aValue with: #closeButtonChanged! !

!CViewerCategoryHeader methodsFor: 'intialize' stamp: 'ar 12/19/2004 00:55'!
initialize
	super initialize.
	self define: #closeButton as: CWindowButton new.
	self define: #prevArrow as: CPushButton new.
	self define: #nextArrow as: CPushButton new.
	self define: #categoryLabel as: CLabel new.
! !

!CViewerCategoryHeader methodsFor: 'accessing' stamp: 'ar 3/21/2004 23:38'!
label
	^categoryLabel value! !

!CViewerCategoryHeader methodsFor: 'accessing' stamp: 'ar 3/21/2004 23:39'!
label: aString
	categoryLabel value: aString! !

!CViewerCategoryHeader methodsFor: 'accessing' stamp: 'ar 3/21/2004 23:08'!
nextArrow
	"Answer the nextArrow of the receiver"
	<bewareOf: #nextArrowChanged>
	^self propertyValueAt: #nextArrow! !

!CViewerCategoryHeader methodsFor: 'accessing' stamp: 'ar 3/21/2004 23:08'!
nextArrow: aValue
	"Modify the receiver's nextArrow"
	^self propertyValueAt: #nextArrow put: aValue with: #nextArrowChanged! !

!CViewerCategoryHeader methodsFor: 'events' stamp: 'ar 6/18/2005 18:44'!
onCloseButtonFire
	<on: fire in: closeButton>
	self signal: #closeRequest.! !

!CViewerCategoryHeader methodsFor: 'events' stamp: 'ar 3/21/2004 23:55'!
onNextArrow
	<on: fire in: nextArrow>
	self signal: #gotoNext.! !

!CViewerCategoryHeader methodsFor: 'events' stamp: 'ar 3/21/2004 23:55'!
onPrevArrow
	<on: fire in: prevArrow>
	self signal: #gotoPrev.! !

!CViewerCategoryHeader methodsFor: 'accessing' stamp: 'ar 3/21/2004 23:08'!
prevArrow
	"Answer the prevArrow of the receiver"
	<bewareOf: #prevArrowChanged>
	^self propertyValueAt: #prevArrow! !

!CViewerCategoryHeader methodsFor: 'accessing' stamp: 'ar 3/21/2004 23:08'!
prevArrow: aValue
	"Modify the receiver's prevArrow"
	^self propertyValueAt: #prevArrow put: aValue with: #prevArrowChanged! !

!CViewerCategoryHeader methodsFor: 'intialize' stamp: 'ar 5/29/2005 17:13'!
setupCostume
	super setupCostume.
	self layout: CTableLayout new.
	self layout listDirection: #leftToRight.
	self hResizing: #spaceFill; vResizing: #shrinkWrap.
	self color: Color white.
	self borderStyle: #simple.
	self borderColor: CWidgetLibrary default frameColor.
	self borderWidth: 1.
	self extent: 1@1.

	categoryLabel font: CTileLibrary default tileFont.
	categoryLabel textAnchorPoint: #center.

	{prevArrow. nextArrow} do:[:button|
		button height: 5.
		button font: (TextStyle named:'WindowFont') defaultFont.
		button hResizing: #rigid; vResizing: #spaceFill; width: 16.
	].
	prevArrow label: 'l'.
	nextArrow label: 'r'.

	closeButton roundCorners: false.
	closeButton normalBorderColor: self borderColor.
	closeButton label: 'x'.
	closeButton normalTextColor: Color gray.

	self add: closeButton.
	self add: prevArrow.
	self add: categoryLabel.
	self add: nextArrow.
! !

CPlayer subclass: #CViewerFlap
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tweak-Tiles-Viewer'!

!CViewerFlap commentStamp: 'sw 3/13/2005 18:57' prior: 0!
A flap in which, potentially, multiple CTileViewers (each showing a category) can be viewed.  A header at the top of the flap holds controls for dismissing the viewer, adding a new category pane, adding a new variable, and viewing/editing the object's name.

Field						Description
-----						--------------------------------------------------------------------------------------------
target						The player which this viewer views.
header						The pane at the top of the flap, with controls and the name pane.
holder						The container that holds the individual category viewers.
flapTab						The tab of the viewer flap.
closeButton					The button in the header that serves to remove the viewer.
addCategoryViewerButton	The button in the header that, when clicks, adds a new category viewer.
menuButton					The button in the header that serves to offer a general menu for the viewer.
addVariableButton			The button in the header that serves to add a new variable to the target
namePane					The type-in pane in the header that shows the target's name.


Class Variable				Description
----- ----------				--------------------------------------------------------------------------------------------
TabWidth 					The canonical width of viewer flap tabs
OpenThreshold				How many pixels need to be visible for a flap to be deemed "open"
AnimationDuration			How long the open/close animations should take, in milliseconds.!

CViewerFlap defineFields: '<?xml  version="1.0" ?>
<fields>
	<field toGet="target" toSet="target:" changeEvent="targetChanged">target</field>
	<field toGet="header" toSet="header:" changeEvent="headerChanged">header</field>
	<field toGet="holder" toSet="holder:" changeEvent="holderChanged">holder</field>
	<field toGet="flapTab" toSet="flapTab:" changeEvent="flapTabChanged">flapTab</field>
	<field toGet="closeButton" toSet="closeButton:" changeEvent="closeButtonChanged">closeButton</field>
	<field toGet="addCategoryViewerButton" toSet="addCategoryViewerButton:" changeEvent="addCategoryViewerButtonChanged">addCategoryViewerButton</field>
	<field toGet="menuButton" toSet="menuButton:" changeEvent="menuButtonChanged">menuButton</field>
	<field toGet="addVariableButton" toSet="addVariableButton:" changeEvent="addVariableButtonChanged">addVariableButton</field>
	<field toGet="namePane" toSet="namePane:" changeEvent="namePaneChanged">namePane</field>
	<field toGet="topOffset" toSet="topOffset:" changeEvent="topOffsetChanged">topOffset</field>
</fields>'!

!CViewerFlap class methodsFor: 'instance creation' stamp: 'ar 6/19/2005 12:44'!
openForTarget: aTarget inWorld: aWorld
	"Open a viewer flap for the target, in the world specified (a ProjectBuilder)
	Reuse any existing Viewer, else make a new one.  Close other open viewer flaps."

	| evt viewer |
	aWorld signal: #openViewer with: aTarget.
	evt := aTarget catchAnyOf: {aTarget. #viewerOpened} during: [self waitTick].
	evt ifNil:["No existing Viewer caught this, so create a new one."
		viewer := self new.
		viewer openForTarget: aTarget in: aWorld.
	].
	aWorld signal: #closeViewerUnless with: aTarget.
	^viewer! !

!CViewerFlap methodsFor: 'accessing' stamp: 'sw 2/15/2005 01:31'!
addCategoryViewerButton
	"Answer the addCategoryViewerButton of the receiver"
	<bewareOf: #addCategoryViewerButtonChanged>
	^self propertyValueAt: #addCategoryViewerButton! !

!CViewerFlap methodsFor: 'accessing' stamp: 'sw 2/15/2005 01:31'!
addCategoryViewerButton: aValue
	"Modify the receiver's addCategoryViewerButton"
	^self propertyValueAt: #addCategoryViewerButton put: aValue with: #addCategoryViewerButtonChanged! !

!CViewerFlap methodsFor: 'accessing' stamp: 'sw 2/14/2005 13:39'!
addVariableButton
	"Answer the addVariableButton of the receiver"
	<bewareOf: #addVariableButtonChanged>
	^self propertyValueAt: #addVariableButton! !

!CViewerFlap methodsFor: 'accessing' stamp: 'sw 2/14/2005 13:39'!
addVariableButton: aValue
	"Modify the receiver's addVariableButton"
	^self propertyValueAt: #addVariableButton put: aValue with: #addVariableButtonChanged! !

!CViewerFlap methodsFor: 'private' stamp: 'ar 5/29/2005 17:46'!
alignWithTab
	"Make the receiver's left edge correspond to the right edge of its tab,"
	<on: geometryChanged in: flapTab>

	flapTab container ifNil: [^ self].
	topLeft := flapTab right @ topOffset.! !

!CViewerFlap methodsFor: 'queries' stamp: 'sw 3/24/2005 05:00'!
animationDuration
	"Answer the standard duration of a flap open/close animation, in ms"

	^ 300! !

!CViewerFlap methodsFor: 'private' stamp: 'ar 6/19/2005 12:44'!
assureClosedWithAnimation
	"If the receiver is not closed, close it at this time, using animation"
	self slideFlapTo: flapTab container width.! !

!CViewerFlap methodsFor: 'private' stamp: 'bf 12/6/2005 17:07'!
assureThatCategoryShows: aCategoryName
	"Assure that the category of the given name is showing"

	| aViewer |
	(holder elements asArray detect: [:aCategoryViewer | aCategoryViewer header label = aCategoryName] ifNone: [nil]) ifNil:
		[aViewer := CTileDefinition defaultViewerOn: target.
		aViewer pointToCategoryNamed: aCategoryName.
		holder addFirst: aViewer]! !

!CViewerFlap methodsFor: 'accessing' stamp: 'sw 2/14/2005 13:44'!
closeButton
	"Answer the closeButton of the receiver"
	<bewareOf: #closeButtonChanged>
	^self propertyValueAt: #closeButton! !

!CViewerFlap methodsFor: 'accessing' stamp: 'sw 2/14/2005 13:44'!
closeButton: aValue
	"Modify the receiver's closeButton"
	^self propertyValueAt: #closeButton put: aValue with: #closeButtonChanged! !

!CViewerFlap methodsFor: 'accessing' stamp: 'sw 2/14/2005 13:39'!
flapTab
	"Answer the flapTab of the receiver"
	<bewareOf: #flapTabChanged>
	^self propertyValueAt: #flapTab! !

!CViewerFlap methodsFor: 'accessing' stamp: 'sw 2/14/2005 13:39'!
flapTab: aValue
	"Modify the receiver's flapTab"
	^self propertyValueAt: #flapTab put: aValue with: #flapTabChanged! !

!CViewerFlap methodsFor: 'accessing' stamp: 'sw 2/14/2005 13:39'!
header
	"Answer the header of the receiver"
	<bewareOf: #headerChanged>
	^self propertyValueAt: #header! !

!CViewerFlap methodsFor: 'accessing' stamp: 'sw 2/14/2005 13:39'!
header: aValue
	"Modify the receiver's header"
	^self propertyValueAt: #header put: aValue with: #headerChanged! !

!CViewerFlap methodsFor: 'accessing' stamp: 'sw 2/14/2005 13:39'!
holder
	"Answer the holder of the receiver"
	<bewareOf: #holderChanged>
	^self propertyValueAt: #holder! !

!CViewerFlap methodsFor: 'accessing' stamp: 'sw 2/14/2005 13:39'!
holder: aValue
	"Modify the receiver's holder"
	^self propertyValueAt: #holder put: aValue with: #holderChanged! !

!CViewerFlap methodsFor: 'private' stamp: 'ar 7/14/2005 14:47'!
iconForm: aForm
	"Make the given form be the graphic for my tab."

	flapTab graphic: aForm.
	flapTab extent: aForm extent + (flapTab borderWidth * 2).! !

!CViewerFlap methodsFor: 'initialization' stamp: 'ar 5/29/2005 17:44'!
initialize
	"Initialize the flap... issue #define: declarations"

	super initialize.
	self define: #header as: CPlayer new.
	self define: #holder as: CHolder new.
	self define: #flapTab as: CPlayer new.

	self define: #closeButton as: CWindowButton new.
	self define: #addCategoryViewerButton as: CWindowButton new.
	self define: #menuButton as: CWindowButton new.
	self define: #addVariableButton as: CWindowButton new.
	self define: #namePane as: CInputField new.

	topOffset := 0.! !

!CViewerFlap methodsFor: 'queries' stamp: 'sw 2/28/2005 02:47'!
isFlapClosed
	"Answer whether the flap is closed."

	^ flapTab right >= flapTab container width! !

!CViewerFlap methodsFor: 'accessing' stamp: 'sw 2/14/2005 13:39'!
menuButton
	"Answer the menuButton of the receiver"
	<bewareOf: #menuButtonChanged>
	^self propertyValueAt: #menuButton! !

!CViewerFlap methodsFor: 'accessing' stamp: 'sw 2/14/2005 13:39'!
menuButton: aValue
	"Modify the receiver's menuButton"
	^self propertyValueAt: #menuButton put: aValue with: #menuButtonChanged! !

!CViewerFlap methodsFor: 'accessing' stamp: 'sw 2/14/2005 13:39'!
namePane
	"Answer the namePane of the receiver"
	<bewareOf: #namePaneChanged>
	^self propertyValueAt: #namePane! !

!CViewerFlap methodsFor: 'accessing' stamp: 'sw 2/14/2005 13:39'!
namePane: aValue
	"Modify the receiver's namePane"
	^self propertyValueAt: #namePane put: aValue with: #namePaneChanged! !

!CViewerFlap methodsFor: 'events' stamp: 'sw 3/4/2005 14:50'!
onAcceptNameChange
	"React to the user's submitting a name change in the name pane"

	<on: accept in: namePane>
	target name: namePane value! !

!CViewerFlap methodsFor: 'events' stamp: 'sw 2/15/2005 23:30'!
onAddCategoryViewer
	"The user submitted contents to the searchField by hitting the Search button; trigger a search"

	<on: addCategoryViewer in: header>
	holder add: (CTileDefinition defaultViewerOn: target)
! !

!CViewerFlap methodsFor: 'events' stamp: 'ar 5/22/2005 12:55'!
onClose
	"When closing, close the tab as well."
	<on: close>
	target := nil.
	flapTab destroy.! !

!CViewerFlap methodsFor: 'events' stamp: 'ar 5/29/2005 17:09'!
onCloseButtonFire
	<on: fire in: closeButton>
	self close.
! !

!CViewerFlap methodsFor: 'events' stamp: 'sw 3/9/2005 04:34'!
onCloseViewerUnlessTargetIs: aTarget
	"Close the receiver unless it views the given target"

	<on: closeViewerUnless in: world>

	aTarget == self target
		ifFalse:
			[self assureClosedWithAnimation]! !

!CViewerFlap methodsFor: 'events' stamp: 'bf 4/26/2005 18:29'!
onDrop: aPlayer
	<on: dragDrop>
	aPlayer signal: #dropReject.
! !

!CViewerFlap methodsFor: 'events' stamp: 'bf 12/6/2005 17:08'!
onFireInAddCategoryViewerButton
	"The button for adding a category viewer was pressed -- respond accordingly"

	<on: fire in: addCategoryViewerButton>
	holder add: (CTileDefinition defaultViewerOn: target)! !

!CViewerFlap methodsFor: 'events' stamp: 'bf 12/6/2005 18:10'!
onFireInAddVariableButton
	"The button for adding a variable was pressed -- respond accordingly."

	<on: fire in: addVariableButton>
	
	self assureThatCategoryShows: 'variables'.
	target addVariable.! !

!CViewerFlap methodsFor: 'events' stamp: 'sw 3/24/2005 05:05'!
onGeometryChangedInFlapTab
	"When I've had a geomery change in my tab, force it to an appropriate position (given the logic of flap look & feel) and bring me along"

	| outerBound aWidth |
	<on: geometryChanged in: flapTab>

	aWidth := self tabWidth.
	outerBound := ((flapTab container bounds right + ((aWidth + flapTab width + self width) negated)) @ 26) 		
		corner: (flapTab container bounds right @ (aWidth + 1 + (self height max: flapTab height))).
	flapTab bounds: (flapTab bounds translatedToBeWithin: outerBound).
	topLeft := flapTab right @ (flapTab container top + 26)! !

!CViewerFlap methodsFor: 'events' stamp: 'sw 2/17/2005 23:56'!
onMenuButtonPressed
	"The user submitted contents to the searchField by hitting the Search button; trigger a search"

	<on: menuButtonPressed in: header>

	self notYetImplemented! !

!CViewerFlap methodsFor: 'events' stamp: 'sw 3/4/2005 14:50'!
onNameChanged
	"React to a change in target's name"

	<on: nameChanged in: target>
	namePane value: target name! !

!CViewerFlap methodsFor: 'events' stamp: 'sw 3/9/2005 04:29'!
onOpenViewerFor: aTarget
	"If my target is the given one, open it fully, and signal that I handled it."

	<on: openViewer in: world>
	aTarget == self target
		ifTrue:
			[aTarget signal: #viewerOpened.
			self openFully]! !

!CViewerFlap methodsFor: 'events' stamp: 'ar 6/19/2005 12:39'!
onTabClicked
	"The user clicked on the tab.  Fully open or fully close the flap, depending..."

	| end |
	<on: mouseClick in: flapTab>
	self world signal: #closeViewerUnless with: self target.
	self waitTick.
	end := flapTab container width.
 	flapTab right >= (end - self openThreshold) ifTrue: [
		end := container width - (self width + self tabWidth)].
	self slideFlapTo: end.! !

!CViewerFlap methodsFor: 'events' stamp: 'sw 3/11/2005 01:35'!
onTabDragged
	"If the user drags the tab, move the tab, and thus also the flap, accordingly"

	| offset |
	<on: mouseDrag in: flapTab>

	offset := hand position - flapTab position.
	flapTab forEach: #mouseMove
		do: [flapTab position: hand position - offset]
		until: #mouseUp.
	self isFlapClosed ifFalse:
		[self world signal: #closeViewerUnless with: self target]! !

!CViewerFlap methodsFor: 'events' stamp: 'ar 6/21/2005 18:37'!
onTargetChanged
	"React to a change in target"
	<on: targetChanged>
	target ifNil:[^self].
	namePane value: target name.
	self refreshIcon.! !

!CViewerFlap methodsFor: 'events' stamp: 'sw 3/4/2005 14:50'!
onTargetClosed
	"React to the closing of my target player"

	<on: close in: target>
	self close! !

!CViewerFlap methodsFor: 'events' stamp: 'ar 6/21/2005 18:40'!
onTargetInvalidated
	"When fill-style of target changes, regenerate the icon"
	<on: invalidate in: target>
	self wait: 0.1. "avoid recreating icon too often"
	self refreshIcon.! !

!CViewerFlap methodsFor: 'initialization' stamp: 'ar 6/19/2005 12:46'!
openForTarget: aTarget in: aContainer
	"Open the receiver to be a viewer for the given target.  Open the tab fully"

	self target: aTarget.
	self open.
	self waitTick.
	"Run it asynchronously so sender can invoke close animation if necessary"
	self startScript: #openFully.! !

!CViewerFlap methodsFor: 'initialization' stamp: 'ar 6/21/2005 18:36'!
openFully
	"Make the flap fully open and fully visible, and with an up-to-date icon"
	self visible: true.
	self slideFlapTo: flapTab container right - (self width + self tabWidth).! !

!CViewerFlap methodsFor: 'queries' stamp: 'sw 3/24/2005 05:03'!
openThreshold
	"Answer the number of pixels by which a flap needs to be open to be considered as officially open, rather than as sloppily closed"

	^ 50! !

!CViewerFlap methodsFor: 'private' stamp: 'sw 3/13/2005 18:47'!
refreshIcon
	"Regenerate my icon"

	self iconForm: target makeIcon! !

!CViewerFlap methodsFor: 'initialization' stamp: 'ar 5/29/2005 17:09'!
setupCostume
	"Initialization: set up the costume of the receiver"

	| aCategoryViewer bottommost otherTabs worldHeight |
	super setupCostume.

	self hResizing: #shrinkWrap;
		 vResizing: #spaceFill.
	self layout: CTableLayout new.
	self layout listDirection: #topToBottom.
	self color: (Color white alpha: 0.3).
	sticky := true.

	header layout: CTableLayout new.
	header layout listDirection: #leftToRight.
	header hResizing: #spaceFill; vResizing: #shrinkWrap.
	header color: Color white.
	header borderStyle: #simple.
	header borderColor: CWidgetLibrary default frameColor.
	header borderWidth: 1.
	header extent: 1@1.

	closeButton roundCorners: true.
	closeButton normalBorderColor: self borderColor.
	closeButton label: 'x'.
	closeButton normalTextColor: Color gray.
	closeButton balloonHelp: 'remove'.
	header add: closeButton.

	header add: (CPlayer new extent: 8@0; yourself).  "spacer"
	addCategoryViewerButton roundCorners: true.
	addCategoryViewerButton normalBorderColor: self borderColor.
	addCategoryViewerButton label: '+'.
	addCategoryViewerButton normalTextColor: Color gray.
	addCategoryViewerButton balloonHelp: 'add a category-viewer'.
	header add: addCategoryViewerButton.

	"header add: (CPlayer new extent: 6@0; yourself). 
	menuButton  roundCorners: true.
	menuButton normalBorderColor: header borderColor.
	menuButton label: 'o'.
	menuButton normalTextColor: Color gray.
	menuButton balloonHelp: 'menu'.
	header add: menuButton."

	header add: (CPlayer new extent: 8@0; yourself). 
	addVariableButton  roundCorners: false.
	addVariableButton normalBorderColor: header borderColor.
	addVariableButton label: 'v'.
	addVariableButton normalTextColor: Color gray.
	addVariableButton balloonHelp: 'add a variable.'.
	header add: addVariableButton.

	header add: (CPlayer new extent: 4@0; yourself).
	namePane borderWidth: 0.
	header add: namePane.
	self add: header..

	self holder layout: CTableLayout new.
	holder layout listDirection: #topToBottom.
	holder hResizing: #shrinkWrap;
		 vResizing: #shrinkWrap.

	self add: holder.

	aCategoryViewer := CTileDefinition defaultViewerOn: self target.
	aCategoryViewer pointToCategoryNamed: 'basic'.
	holder add: aCategoryViewer.

	aCategoryViewer := CTileDefinition defaultViewerOn: self target.
	aCategoryViewer pointToCategoryNamed: 'color and border'.
	holder add: aCategoryViewer.

	flapTab borderStyle: #simple; borderWidth: 1;
		borderColor: CWidgetLibrary default frameColor; fillStyle: #solid;
		extent: (self tabWidth @ self tabWidth).
	flapTab propertyValueAt: #isViewerFlapTab put: true.
	flapTab visible: false.
	flapTab open.  "So we can converse with the container"
	self waitTick.

	otherTabs := flapTab container elements asArray select: [:e | ((e propertyValueAt: #isViewerFlapTab) == true) and: [e ~~ flapTab]].
	bottommost := otherTabs size > 0
		ifTrue:
			[(otherTabs collect: [:t | t bottom]) max]
		ifFalse:
			[26].  "CMenuBar's height"
	worldHeight := flapTab container height.
	flapTab top := bottommost min: (worldHeight - self tabWidth).
	flapTab right := flapTab container right - self width.
	flapTab visible: true.
	self height: worldHeight - self tabWidth! !

!CViewerFlap methodsFor: 'initialization' stamp: 'ar 6/21/2005 18:36'!
slideFlapTo: end
	"Slide the flap to align its right edge with the end point"
	| start delta elapsedTime startTime aDuration |
	startTime := Time millisecondClockValue.
	start := flapTab right.
	delta := end - start.
	delta = 0 ifTrue: [^ self].  "already there"

	[elapsedTime := Time millisecondsSince: startTime.
	aDuration := self animationDuration.
	flapTab right: start + (delta * elapsedTime // aDuration).
	elapsedTime >= aDuration] whileFalse:[self waitTick].	

	flapTab right: end.! !

!CViewerFlap methodsFor: 'queries' stamp: 'sw 3/24/2005 05:04'!
tabWidth
	"Answer the standard tab width"

	^ 32! !

!CViewerFlap methodsFor: 'accessing' stamp: 'sw 2/14/2005 13:57'!
target
	"Answer the target of the receiver"
	<bewareOf: #targetChanged>
	^self propertyValueAt: #target! !

!CViewerFlap methodsFor: 'accessing' stamp: 'ar 5/22/2005 12:56'!
target: aValue
	"Modify the receiver's target"

	| result |
	result := self propertyValueAt: #target put: aValue with: #targetChanged.
	aValue ifNotNil:[self iconForm: aValue makeIcon].
	^ result! !

!CViewerFlap methodsFor: 'accessing' stamp: 'ar 5/29/2005 18:02'!
topOffset
	"Answer the topOffset of the receiver"
	<bewareOf: #topOffsetChanged>
	^self propertyValueAt: #topOffset! !

!CViewerFlap methodsFor: 'accessing' stamp: 'ar 5/29/2005 18:03'!
topOffset: aValue
	"Modify the receiver's topOffset"
	^self propertyValueAt: #topOffset put: aValue with: #topOffsetChanged! !

CPlayer subclass: #CViewerRow
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tweak-Tiles-Viewer'!

CViewerRow defineFields: '<?xml  version="1.0" ?>
<fields>
	<field toGet="target" toSet="target:" changeEvent="targetChanged">target</field>
	<field toGet="fireButton" toSet="fireButton:" changeEvent="fireButtonChanged">fireButton</field>
	<field toGet="messageTile" toSet="messageTile:" changeEvent="messageTileChanged">messageTile</field>
	<field toGet="spacer" toSet="spacer:" changeEvent="spacerChanged">spacer</field>
	<field toGet="setter" toSet="setter:" changeEvent="setterChanged">setter</field>
	<field toGet="readout" toSet="readout:" changeEvent="readoutChanged">readout</field>
</fields>'!

!CViewerRow methodsFor: 'scriptors' stamp: 'bf 7/27/2005 17:46'!
dragTile: aTile
	"Drag the given tile, which may be a command or a slot retrieval"

	| evt |
	aTile balloonHelp: nil.
	hand attach: aTile.

	evt := self waitUntilAnyOf: {aTile. #dropAccept. aTile. #dropReject. aTile}.
	evt selector == #dropReject
		ifTrue: [^aTile flash;destroy].

	evt arguments first isTilePlayer ifTrue: [^ self].  "Accepted in a structure - pace"

	"drop accepted but not not in another tile, so sprout a scriptor or watcher"
	aTile isStatementLike
		ifTrue:
			[ | scriptor |
			"might be existing script"
			target scriptors do: [:oldScriptor | oldScriptor selector == aTile selector
				ifTrue: [oldScriptor openCenteredAt: hand position.
					^aTile destroy]].
			"make new script"
			scriptor := CScriptorTile new.
			scriptor receiver: target.
			scriptor chooseNewName.
			aTile selector == #emptyScript
				ifTrue: [aTile destroy]
				ifFalse: [scriptor statements add: aTile].
			scriptor openCenteredAt: hand position]
		ifFalse:
			[aTile openAsWatcher]! !

!CViewerRow methodsFor: 'accessing' stamp: 'ar 3/22/2004 20:46'!
fireButton
	"Answer the fireButton of the receiver"
	<bewareOf: #fireButtonChanged>
	^self propertyValueAt: #fireButton! !

!CViewerRow methodsFor: 'accessing' stamp: 'ar 3/22/2004 20:46'!
fireButton: aValue
	"Modify the receiver's fireButton"
	^self propertyValueAt: #fireButton put: aValue with: #fireButtonChanged! !

!CViewerRow methodsFor: 'firing' stamp: 'ar 3/22/2004 21:17'!
fireMessage
	| result tile |
	result := messageTile eval.
	messageTile valueType ifNil:[^nil].
	tile := CValueTile on: result.
	tile valueType: messageTile valueType.
	tile openInHand! !

!CViewerRow methodsFor: 'initialize' stamp: 'bf 3/11/2005 19:00'!
initialize
	super initialize.
	self define: #fireButton as: CLabel new.
	self define: #spacer as: CPlayer new.
	self define: #setter as: CLabel new.
! !

!CViewerRow methodsFor: 'accessing' stamp: 'ar 3/22/2004 15:21'!
messageTile
	"Answer the messageTile of the receiver"
	<bewareOf: #messageTileChanged>
	^self propertyValueAt: #messageTile! !

!CViewerRow methodsFor: 'accessing' stamp: 'ar 3/22/2004 15:21'!
messageTile: aValue
	"Modify the receiver's messageTile"
	^self propertyValueAt: #messageTile put: aValue with: #messageTileChanged! !

!CViewerRow methodsFor: 'events' stamp: 'bf 3/11/2005 19:16'!
onClick
	<on: mouseClick in: messageTile>
	self onMouseDrag.! !

!CViewerRow methodsFor: 'events' stamp: 'bf 3/11/2005 19:16'!
onClickSetter
	<on: mouseClick in: setter>
	self onMouseDragSetter.! !

!CViewerRow methodsFor: 'firing' stamp: 'ar 3/23/2004 13:15'!
onFireButtonDown
	<on: mouseDown in: fireButton>
	"fireButton borderStyle: #inset."
	fireButton fillAngle: -90.! !

!CViewerRow methodsFor: 'firing' stamp: 'ar 3/23/2004 13:15'!
onFireButtonEnter
	<on: mouseEnter in: fireButton>
	"fireButton borderStyle: #raised."
	fireButton fillStyle: #verticalGradient.
	fireButton fillColors: {Color white. CWidgetLibrary default baseColor}.! !

!CViewerRow methodsFor: 'firing' stamp: 'ar 3/23/2004 13:14'!
onFireButtonLeave
	<on: mouseLeave in: fireButton>
	"fireButton borderStyle: #simple."
	fireButton fillStyle: #solid.! !

!CViewerRow methodsFor: 'firing' stamp: 'ar 3/23/2004 13:15'!
onFireButtonUp
	<on: mouseUp in: fireButton>
	"fireButton borderStyle: #raised."
	fireButton fillAngle: 90.
	(fireButton containsPoint: fireButton cursorPoint)
		ifTrue:[self fireMessage].! !

!CViewerRow methodsFor: 'events' stamp: 'bf 7/27/2005 17:21'!
onMessageTileChanged
	| fieldRef fieldName |
	<on: messageTileChanged>
	self removeAll.
	messageTile valueType 
		ifNil:[fireButton label: '!!']
		ifNotNil:[fireButton label: '?'].
	self add: fireButton.
	self add: messageTile.
	messageTile borderStyle: #simple.
	messageTile isMessageTile ifTrue: [
		messageTile costume disableDrop].
	messageTile isPropertyTile ifTrue: [
		self add: spacer.
		readout := CValueTile on: messageTile eval.
		fieldName := messageTile fieldName.
		fieldName ifNotNil:[
			fieldRef := messageTile receiver fieldNamed: fieldName.
			readout player: fieldRef.
			fieldRef isReadOnly
				ifFalse: [self add: setter before: messageTile].
		].
		readout valueType: messageTile valueType.
		readout color: CWidgetLibrary default paneColor.
		self add: readout.
	]! !

!CViewerRow methodsFor: 'events' stamp: 'ar 3/22/2004 15:23'!
onMouseDown
	<on: mouseDown in: messageTile>
	messageTile borderStyle: #inset.! !

!CViewerRow methodsFor: 'events' stamp: 'bf 3/11/2005 19:15'!
onMouseDownSetter
	<on: mouseDown in: setter>
	setter borderStyle: #inset.! !

!CViewerRow methodsFor: 'events' stamp: 'bf 4/26/2005 18:13'!
onMouseDrag
	<on: mouseDrag in: messageTile>
	self onMouseLeave.
	self dragTile: (messageTile definition newFor: messageTile receiver).

! !

!CViewerRow methodsFor: 'events' stamp: 'bf 4/26/2005 18:12'!
onMouseDragSetter
	<on: mouseDrag in: setter>
	self onMouseLeaveSetter.
	self dragTile: (messageTile definition newFor: messageTile receiver) asAssignmentTile.
! !

!CViewerRow methodsFor: 'events' stamp: 'ar 3/23/2004 13:13'!
onMouseEnter
	<on: mouseEnter in: messageTile>
	messageTile borderStyle: #raised.! !

!CViewerRow methodsFor: 'events' stamp: 'bf 3/11/2005 19:15'!
onMouseEnterSetter
	<on: mouseEnter in: setter>
	setter borderStyle: #raised.! !

!CViewerRow methodsFor: 'events' stamp: 'ar 3/22/2004 15:22'!
onMouseLeave
	<on: mouseLeave in: messageTile>
	messageTile borderStyle: #simple.! !

!CViewerRow methodsFor: 'events' stamp: 'bf 3/11/2005 19:14'!
onMouseLeaveSetter
	<on: mouseLeave in: setter>
	setter borderStyle: #simple.! !

!CViewerRow methodsFor: 'events' stamp: 'ar 3/22/2004 15:23'!
onMouseUp
	<on: mouseUp in: messageTile>
	messageTile borderStyle: #raised.! !

!CViewerRow methodsFor: 'events' stamp: 'bf 3/11/2005 19:15'!
onMouseUpSetter
	<on: mouseUp in: setter>
	setter borderStyle: #raised.! !

!CViewerRow methodsFor: 'accessing' stamp: 'ar 3/22/2004 21:20'!
readout
	"Answer the readout of the receiver"
	<bewareOf: #readoutChanged>
	^self propertyValueAt: #readout! !

!CViewerRow methodsFor: 'accessing' stamp: 'ar 3/22/2004 21:20'!
readout: aValue
	"Modify the receiver's readout"
	^self propertyValueAt: #readout put: aValue with: #readoutChanged! !

!CViewerRow methodsFor: 'accessing' stamp: 'bf 3/11/2005 18:59'!
setter
	"Answer the setter of the receiver"
	<bewareOf: #setterChanged>
	^self propertyValueAt: #setter! !

!CViewerRow methodsFor: 'accessing' stamp: 'bf 3/11/2005 18:59'!
setter: aValue
	"Modify the receiver's setter"
	^self propertyValueAt: #setter put: aValue with: #setterChanged! !

!CViewerRow methodsFor: 'initialize' stamp: 'ar 5/14/2005 17:50'!
setupCostume
	super setupCostume.
	self layout: CTableLayout new.
	self layout listDirection: #leftToRight.
	self hResizing: #spaceFill; vResizing: #shrinkWrap.
	self layout cellInset: 2@0.

	fireButton width: 16.
	fireButton hResizing: #rigid; vResizing: #spaceFill.
	fireButton font: CTileLibrary default tileFont.
	fireButton textAnchorPoint: #center.
	fireButton textInset: 0@0.
	fireButton fillColors: {Color white. CWidgetLibrary default baseColor}.
	fireButton fillAlpha: 1.0.
	fireButton fillStyle: #solid.
	fireButton 
		borderWidth: 1; 
		borderStyle: #simple; 
		borderColor: CWidgetLibrary default frameColor.
	fireButton roundCorners: true.

	spacer hResizing: #spaceFill; vResizing: #rigid.
	spacer height: 10.

	setter hResizing: #shrinkWrap; vResizing: #shrinkWrap.
	setter font: CTileLibrary default tileFont.
	setter value: 'set' asText allBold.
	setter 
		borderWidth: 1; 
		borderStyle: #simple; 
		borderColor: CWidgetLibrary default frameColor.
! !

!CViewerRow methodsFor: 'accessing' stamp: 'ar 3/22/2004 21:20'!
spacer
	"Answer the spacer of the receiver"
	<bewareOf: #spacerChanged>
	^self propertyValueAt: #spacer! !

!CViewerRow methodsFor: 'accessing' stamp: 'ar 3/22/2004 21:20'!
spacer: aValue
	"Modify the receiver's spacer"
	^self propertyValueAt: #spacer put: aValue with: #spacerChanged! !

!CViewerRow methodsFor: 'accessing' stamp: 'bf 4/26/2005 17:31'!
target
	"Answer the target of the receiver"
	<bewareOf: #targetChanged>
	^self propertyValueAt: #target! !

!CViewerRow methodsFor: 'accessing' stamp: 'bf 4/26/2005 17:31'!
target: aValue
	"Modify the receiver's target"
	^self propertyValueAt: #target put: aValue with: #targetChanged! !

CPlayer subclass: #CWatcher
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tweak-Tiles-Viewer'!

!CWatcher commentStamp: 'sw 5/18/2005 02:15' prior: 0!
A player that watches the value of a slot of another player -- emulating traditional etoys-1 watchers.!

CWatcher defineFields: '<?xml  version="1.0" ?>
<fields>
	<field toGet="target" toSet="target:" changeEvent="targetChanged">target</field>
	<field toGet="selector" toSet="selector:" changeEvent="selectorChanged">selector</field>
	<field toGet="displayName" toSet="displayName:" changeEvent="displayNameChanged">displayName</field>
	<field toGet="label" toSet="label:" changeEvent="labelChanged">label</field>
	<field toGet="readout" toSet="readout:" changeEvent="readoutChanged">readout</field>
	<field toGet="originalTiles" toSet="originalTiles:" changeEvent="originalTilesChanged">originalTiles</field>
	<field virtual="true" serializable="false" toGet="valueType" toSet="valueType:">valueType</field>
</fields>'!

!CWatcher methodsFor: 'accessing' stamp: 'ar 7/14/2005 14:42'!
displayName
	"Answer the displayName of the receiver"
	<bewareOf: #displayNameChanged>
	^self propertyValueAt: #displayName! !

!CWatcher methodsFor: 'accessing' stamp: 'ar 7/14/2005 14:42'!
displayName: aValue
	"Modify the receiver's displayName"
	^self propertyValueAt: #displayName put: aValue with: #displayNameChanged! !

!CWatcher methodsFor: 'initialization' stamp: 'sw 6/2/2005 03:14'!
initialize
	"Initialize the receiver"

	super initialize.

	self define: #label as: CLabel new.
	self define: #readout as: CValueTile new.! !

!CWatcher methodsFor: 'testing' stamp: 'sw 5/25/2005 03:28'!
isStatementLike
	"I bear a value, rather than assert a comment, so answer false."

	^ false! !

!CWatcher methodsFor: 'testing' stamp: 'sw 5/25/2005 03:25'!
isTilePlayer
	"Since I can be dropped as if I were a value tile, answer true to this question."

	^ true! !

!CWatcher methodsFor: 'accessing' stamp: 'sw 4/26/2005 18:39'!
label
	"Answer the label of the receiver"
	<bewareOf: #labelChanged>
	^self propertyValueAt: #label! !

!CWatcher methodsFor: 'accessing' stamp: 'sw 4/26/2005 18:39'!
label: aValue
	"Modify the receiver's label"
	^self propertyValueAt: #label put: aValue with: #labelChanged! !

!CWatcher methodsFor: 'events' stamp: 'bf 7/27/2005 18:26'!
onDrag
	"offset me a bit"
	| pos offs |
	pos := container cursorPoint.
	offs := top - (pos y + 8).
	self moveBy: 0@offs negated.
	super onDrag.
	(container notNil and: [pos = container cursorPoint])
		ifTrue: [self moveBy: 0@offs].
! !

!CWatcher methodsFor: 'events' stamp: 'ar 6/18/2005 18:48'!
onTargetChanged
	"Handle a change in my target"
	<on: targetChanged>
	self onTargetNameChanged.
	readout value: (target perform: selector).! !

!CWatcher methodsFor: 'events' stamp: 'ar 7/14/2005 14:43'!
onTargetNameChanged
	"Handle a change in my target"
	<on: nameChanged in: target>
	name := target name, '''s ', (displayName ifNil:[selector]).
	label value: (name,':') asText allBold.! !

!CWatcher methodsFor: 'accessing' stamp: 'sw 6/2/2005 16:25'!
originalTiles
	"Answer the originalTiles of the receiver"
	<bewareOf: #originalTilesChanged>
	^self propertyValueAt: #originalTiles! !

!CWatcher methodsFor: 'accessing' stamp: 'sw 6/2/2005 16:25'!
originalTiles: aValue
	"Modify the receiver's originalTiles"
	^self propertyValueAt: #originalTiles put: aValue with: #originalTilesChanged! !

!CWatcher methodsFor: 'accessing' stamp: 'sw 4/26/2005 18:39'!
readout
	"Answer the readout of the receiver"
	<bewareOf: #readoutChanged>
	^self propertyValueAt: #readout! !

!CWatcher methodsFor: 'accessing' stamp: 'sw 4/26/2005 18:39'!
readout: aValue
	"Modify the receiver's readout"
	^self propertyValueAt: #readout put: aValue with: #readoutChanged! !

!CWatcher methodsFor: 'accessing' stamp: 'sw 4/26/2005 18:39'!
selector
	"Answer the selector of the receiver"
	<bewareOf: #selectorChanged>
	^self propertyValueAt: #selector! !

!CWatcher methodsFor: 'accessing' stamp: 'sw 4/26/2005 18:39'!
selector: aValue
	"Modify the receiver's selector"
	^self propertyValueAt: #selector put: aValue with: #selectorChanged! !

!CWatcher methodsFor: 'initialization' stamp: 'ar 6/21/2005 15:06'!
setupCostume
	"Assuming that my target and selector are set, build the widget"

	| fieldRef |
	super setupCostume.
	layout := CTableLayout new.
	layout listDirection: #leftToRight.
	layout hResizing: #shrinkWrap; vResizing: #shrinkWrap;  layoutInset: 4.

	self color := CWidgetLibrary default paneColor.
	self borderWidth := 1.
	self borderStyle := #simple.
	self borderColor := CWidgetLibrary default frameColor.
	self roundCorners := true.

	self add: label.
	label value: target name, '''s ', selector asString.
	label hResizing: #shrinkWrap; vResizing: #shrinkWrap.

	fieldRef := target fieldNamed: selector.
	readout player: fieldRef.
	readout vResizing: #shrinkWrap.
	readout borderStyle: #inset.
	readout color: Color white.
	self add: readout! !

!CWatcher methodsFor: 'accessing' stamp: 'sw 4/26/2005 18:39'!
target
	"Answer the target of the receiver"
	<bewareOf: #targetChanged>
	^self propertyValueAt: #target! !

!CWatcher methodsFor: 'accessing' stamp: 'sw 4/26/2005 18:39'!
target: aValue
	"Modify the receiver's target"
	^self propertyValueAt: #target put: aValue with: #targetChanged! !

!CWatcher methodsFor: 'testing' stamp: 'sw 5/25/2005 23:34'!
tileExpression
	"Answer the player to be dropped as an expression in tile-scripting d&d"

	^ originalTiles! !

!CWatcher methodsFor: 'testing' stamp: 'ar 6/21/2005 13:59'!
valueType
	"Answer the valueType of the receiver"
	^readout valueType! !

!CWatcher methodsFor: 'testing' stamp: 'ar 6/21/2005 14:00'!
valueType: aValue
	"Modify the receiver's valueType"
	^readout valueType: aValue! !

CButtonCostume subclass: #CTileButtonCostume
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tweak-Tiles-Costumes'!

CTileButtonCostume defineFields: '<?xml  version="1.0" ?>
<fields>
</fields>'!

!CTileButtonCostume methodsFor: 'events' stamp: 'ar 7/7/2005 10:08'!
onButtonStateChanged
	| css |
	(css := self styleSheet) ifNotNil:[^css applyTo: self].
	player state == #disabled
		ifTrue:[buttonLabel textColor: Color gray]
		ifFalse:[buttonLabel textColor: Color black].
	player state == #pressed ifTrue:[
		borderStyle := #inset.
		buttonLabel textOffset: 1.
		buttonImage graphicOffset: 1.
	] ifFalse:[
		borderStyle := #simple.
		buttonLabel textOffset: 0.
		buttonImage graphicOffset: 0.
	].
! !

!CTileButtonCostume methodsFor: 'initialize' stamp: 'bf 5/28/2004 16:36'!
setupCostume
	super setupCostume.
	self borderColor: CWidgetLibrary default frameColor.
	self borderStyle: #simple.
	self borderWidth: 0.
	self fillStyle: #verticalGradient.
	self fillColors: CWidgetLibrary default gradientColors.
	self textEmphasis: 1.
	self extent: 80@20.

! !

CCostume subclass: #CTileCostume
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tweak-Tiles-Costumes'!

!CTileCostume commentStamp: 'ar 3/30/2004 16:59' prior: 0!
CTileCostume is the basis for most of the costumes in Tweak's tile system. It provides the basics for the notification mechanism which is used for tiles in DnD interactions. ALL of the events involved are signaled in the players and not in the costumes in order to allow costumes to be completely exchangeable. In other words, a properly implemented tile costume has absolutely no need to know anything about other tile costumes except the event protocol involved. The events used are:

#tilePadDragged - This event notifies a tile that it was just dragged out of its (tile) container. For example, in an expression like [[Foo's x] + [Foo's y]] grabbing [Foo's x] would signal #tilePadDrag in [Foo's x]. The event should be handled by tiles which have a different representation depending on whether they are inside another tile or not.

#tilePadDropped - This event notifies a tile that it was just dropped into a (tile) container, For example, dropping [Foo's x] into an expression like [[ X ] + [Foo's y]] would signal #tilePadDrop in [Foo's x]. The event should be handled by tiles which have a different representation depending on whether they are inside another tile or not.

#tilePadHighlight - This event notifies a tile that its container (probably a variable) would allow dropping a tile in the place it currently is. For example, dragging [Foo's x] over the [ X ] tile in an expression like [ [ X ] + [Foo's y] ] would signal #tilePadHighlight in the [ X ] tile when the user drags [Foo's x] over it. This event should be handled by costumes which have a non-transparent representation to either become transparent (let the parent's highlight show through) or provide their own highlighting for giving user-feedback about the possibility of the drop operation.

#tilePadUnhighlight - This event notifies a tile to reverse the effect of a #tilePadHighlight.

#tilePadEnter - This event notifies a tile that it entered a 'hot area', e.g., another tile which would accept it if the user dropped it there.

#tilePadLeave - This event notifies a tile that it left a 'hot area'.
!

CTileCostume subclass: #CAssignmentTileCostume
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tweak-Tiles-Costumes'!

CAssignmentTileCostume defineFields: '<?xml  version="1.0" ?>
<fields>
	<field toGet="propertyTile" toSet="propertyTile:" changeEvent="propertyTileChanged">propertyTile</field>
	<field toGet="expressionTile" toSet="expressionTile:" changeEvent="expressionTileChanged">expressionTile</field>
	<field toGet="expandArrow" toSet="expandArrow:" changeEvent="expandArrowChanged">expandArrow</field>
</fields>'!

!CAssignmentTileCostume methodsFor: 'initialize' stamp: 'bf 2/15/2005 15:33'!
defaultPlayer
	^CAssignmentTile! !

!CAssignmentTileCostume methodsFor: 'accessing' stamp: 'bf 5/10/2005 17:18'!
expandArrow
	"Answer the expandArrow of the receiver"
	<bewareOf: #expandArrowChanged>
	^self propertyValueAt: #expandArrow! !

!CAssignmentTileCostume methodsFor: 'accessing' stamp: 'bf 5/10/2005 17:18'!
expandArrow: aValue
	"Modify the receiver's expandArrow"
	^self propertyValueAt: #expandArrow put: aValue with: #expandArrowChanged! !

!CAssignmentTileCostume methodsFor: 'events' stamp: 'bf 5/10/2005 18:28'!
expandArrowMouseDown
	| menu item |
	<on: mouseDown in: expandArrow>

	menu := CMenu new.
	
	{nil},self operatorList do: [:op |
		item := menu add: (self menuLabelForOperator: op)
			target: player
			action: #operator:
			argument: op.
		item checked: player operator == op].

	menu run! !

!CAssignmentTileCostume methodsFor: 'events' stamp: 'bf 5/10/2005 18:27'!
expandArrowMouseEnter
	| c |
	<on: mouseEnter in: expandArrow>
	c := expandArrow textColor.
	expandArrow textColor: (c alpha: 1.0).
	expandArrow waitUntil: #mouseLeave.
	expandArrow textColor: c.! !

!CAssignmentTileCostume methodsFor: 'accessing' stamp: 'bf 2/15/2005 15:32'!
expressionTile
	"Answer the expressionTile of the receiver"
	<bewareOf: #expressionTileChanged>
	^self propertyValueAt: #expressionTile! !

!CAssignmentTileCostume methodsFor: 'accessing' stamp: 'bf 2/15/2005 15:32'!
expressionTile: aValue
	"Modify the receiver's expressionTile"
	^self propertyValueAt: #expressionTile put: aValue with: #expressionTileChanged! !

!CAssignmentTileCostume methodsFor: 'initialize' stamp: 'bf 5/10/2005 17:53'!
initialize
	super initialize.

	self define: #expandArrow as: CPlayer new.
! !

!CAssignmentTileCostume methodsFor: 'private' stamp: 'bf 5/10/2005 18:37'!
labelsForOperator: operator
	operator ifNil: [^#('set' 'to')].

	#(	+ ('increase' 'by')
		- ('decrease' 'by')
		* ('multiply' 'by')
		/ ('divide' 'by')
		roundTo: ('round' 'to')
		, ('extend' 'by')
	) pairsDo: [:op :labels |
		operator = op ifTrue: [^labels]].

	"Generic: change <prop> by <op> <expr>"
	^ {'change'. 'by ', operator}! !

!CAssignmentTileCostume methodsFor: 'private' stamp: 'bf 2/17/2005 18:37'!
makeLabel: aString
	^CPlayer new
		text: aString asText allBold;
		hResizing: #shrinkWrap;
		vResizing: #shrinkWrap;
		yourself
! !

!CAssignmentTileCostume methodsFor: 'private' stamp: 'bf 5/10/2005 17:43'!
menuLabelForOperator: operator

	^String streamContents: [:stream |
		(self labelsForOperator: operator)
			do: [:ea | stream nextPutAll: ea]
			separatedBy: [stream space; nextPutAll: player property name; space]].
! !

!CAssignmentTileCostume methodsFor: 'events' stamp: 'bf 2/17/2005 18:27'!
onExpressionChanged
	<on: expressionChanged in: player>
	self signal: #updateEverything! !

!CAssignmentTileCostume methodsFor: 'events' stamp: 'bf 5/10/2005 16:19'!
onOperatorChanged
	<on: operatorChanged in: player>
	self signal: #updateEverything! !

!CAssignmentTileCostume methodsFor: 'events' stamp: 'bf 3/11/2005 19:22'!
onPlayerChanged
	super onPlayerChanged.
	self onPropertyChanged.
	self signal: #updateEverything! !

!CAssignmentTileCostume methodsFor: 'events' stamp: 'bf 2/17/2005 18:39'!
onPropertyChanged
	<on: propertyChanged in: player>

	player property borderStyle: #none.
	self signal: #updateEverything! !

!CAssignmentTileCostume methodsFor: 'events' stamp: 'bf 5/10/2005 18:34'!
onUpdateEverything
	| labels |
	<on: updateEverything>
	labels := self labelsForOperator: player operator.
	self removeAll.
	self add: (self makeLabel: labels first).
	self operatorList isEmpty ifFalse: [self add: expandArrow].
	self add: player property.
	self add: (self makeLabel: labels second).
	self add: player expression.
! !

!CAssignmentTileCostume methodsFor: 'private' stamp: 'bf 5/10/2005 17:24'!
operatorList
	^ player property valueType assignmentOperators! !

!CAssignmentTileCostume methodsFor: 'accessing' stamp: 'bf 2/15/2005 15:32'!
propertyTile
	"Answer the propertyTile of the receiver"
	<bewareOf: #propertyTileChanged>
	^self propertyValueAt: #propertyTile! !

!CAssignmentTileCostume methodsFor: 'accessing' stamp: 'bf 2/15/2005 15:32'!
propertyTile: aValue
	"Modify the receiver's propertyTile"
	^self propertyValueAt: #propertyTile put: aValue with: #propertyTileChanged! !

!CAssignmentTileCostume methodsFor: 'initialize' stamp: 'bf 5/10/2005 18:24'!
setupCostume
	super setupCostume.
	self color: CWidgetLibrary default paneColor.
	self borderStyle: #raised.
	self borderWidth: 1.

	self layout: CTableLayout new.
	self layout listDirection: #leftToRight.
	self layout rubberBandCells: true.
	self hResizing: #shrinkWrap.
	self vResizing: #shrinkWrap.

	expandArrow text: 'd'.
	expandArrow textInset: 0@0.
	expandArrow textColor: (Color black alpha: 0.3).
	expandArrow textFont: (TextStyle named:'WindowFont') defaultFont.
	expandArrow hResizing: #shrinkWrap; vResizing: #shrinkWrap.
! !

CTileCostume subclass: #CColorTileCostume
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tweak-Tiles-ValueCostumes'!

CColorTileCostume defineFields: '<?xml  version="1.0" ?>
<fields>
	<field toGet="swatch" toSet="swatch:" changeEvent="swatchChanged">swatch</field>
</fields>'!

!CColorTileCostume methodsFor: 'initialize' stamp: 'ar 3/21/2004 19:26'!
initialize
	super initialize.
	swatch := CPlayer new.! !

!CColorTileCostume methodsFor: 'events' stamp: 'ar 3/21/2004 19:28'!
onExtentChanged
	<on: extentChanged>
	swatch center: extent // 2.! !

!CColorTileCostume methodsFor: 'events' stamp: 'ar 3/21/2004 19:42'!
onMouseDown
	| picker |
	<on: mouseDown in: swatch>
	picker := CColorPicker new.
	picker player: self.
	picker openAt: hand position.
	picker pickColor.! !

!CColorTileCostume methodsFor: 'events' stamp: 'ar 3/21/2004 19:28'!
onPlayerChanged
	super onPlayerChanged.
	self onValueChanged.! !

!CColorTileCostume methodsFor: 'events' stamp: 'ar 3/23/2004 18:18'!
onValueChanged
	<on: valueChanged>
	value ifNotNil:[swatch color: value].! !

!CColorTileCostume methodsFor: 'initialize' stamp: 'ar 6/21/2005 14:58'!
setupCostume
	super setupCostume.

	self extent: 50@22.
	self hResizing: #rigid; vResizing: #spaceFill.

	swatch extent: 14@14.
	swatch borderWidth: 1; borderStyle: #inset.
	self add: swatch.
! !

!CColorTileCostume methodsFor: 'accessing' stamp: 'ar 3/21/2004 19:26'!
swatch
	"Answer the swatch of the receiver"
	<bewareOf: #swatchChanged>
	^self propertyValueAt: #swatch! !

!CColorTileCostume methodsFor: 'accessing' stamp: 'ar 3/21/2004 19:26'!
swatch: aValue
	"Modify the receiver's swatch"
	^self propertyValueAt: #swatch put: aValue with: #swatchChanged! !

CTileCostume subclass: #CEnumListCostume
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tweak-Tiles-ValueCostumes'!

CEnumListCostume defineFields: '<?xml  version="1.0" ?>
<fields>
	<field toGet="list" toSet="list:" changeEvent="listChanged">list</field>
</fields>'!

!CEnumListCostume methodsFor: 'initialize' stamp: 'ar 3/30/2004 03:03'!
initialize
	super initialize.
	self define: #list as: CDropDownList new.! !

!CEnumListCostume methodsFor: 'accessing' stamp: 'ar 3/30/2004 03:03'!
list
	"Answer the list of the receiver"
	<bewareOf: #listChanged>
	^self propertyValueAt: #list! !

!CEnumListCostume methodsFor: 'accessing' stamp: 'ar 3/30/2004 03:03'!
list: aValue
	"Modify the receiver's list"
	^self propertyValueAt: #list put: aValue with: #listChanged! !

!CEnumListCostume methodsFor: 'events' stamp: 'ar 3/30/2004 03:05'!
onCursorChanged
	<on: cursorChanged in: list>
	player value: list selection.! !

!CEnumListCostume methodsFor: 'events' stamp: 'bf 10/11/2004 15:21'!
onMouseEnter
	<on: mouseEnter>
	list costume button visible: true.! !

!CEnumListCostume methodsFor: 'events' stamp: 'bf 10/11/2004 15:22'!
onMouseLeave
	<on: mouseLeave>
	list costume button visible: false.! !

!CEnumListCostume methodsFor: 'events' stamp: 'ar 3/30/2004 03:04'!
onPlayerChanged
	super onPlayerChanged.
	list contents: player valueType enumValues.
	self onValueChanged.! !

!CEnumListCostume methodsFor: 'events' stamp: 'ar 3/30/2004 03:04'!
onValueChanged
	<on: valueChanged>
	list selection: value.! !

!CEnumListCostume methodsFor: 'initialize' stamp: 'ar 12/10/2005 20:51'!
setupCostume
	super setupCostume.

	self layout: CTableLayout new.
	self layout listDirection: #leftToRight.
	self hResizing: #shrinkWrap; vResizing: #shrinkWrap.
	self height: 22.

	list borderWidth: 0.
	list color: Color transparent.
	list font: CTileLibrary default tileFont.
	list clipping: false.
	list costume button visible: false.
	self add: list.
! !

CTileCostume subclass: #CEnumTileCostume
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tweak-Tiles-ValueCostumes'!

CEnumTileCostume defineFields: '<?xml  version="1.0" ?>
<fields>
	<field toGet="label" toSet="label:" changeEvent="labelChanged">label</field>
	<field toGet="spinner" toSet="spinner:" changeEvent="spinnerChanged">spinner</field>
	<field toGet="valueList" toSet="valueList:" changeEvent="valueListChanged">valueList</field>
</fields>'!

!CEnumTileCostume methodsFor: 'initialize' stamp: 'ar 3/21/2004 20:34'!
initialize
	super initialize.
	self define: #spinner as: CSpinnerButtons new.
	self define: #label as: CLabel new.
	self define: #valueList as: CCollection new.
	valueList wrapCursor: true.! !

!CEnumTileCostume methodsFor: 'accessing' stamp: 'ar 3/21/2004 20:31'!
label
	"Answer the label of the receiver"
	<bewareOf: #labelChanged>
	^self propertyValueAt: #label! !

!CEnumTileCostume methodsFor: 'accessing' stamp: 'ar 3/21/2004 20:31'!
label: aValue
	"Modify the receiver's label"
	^self propertyValueAt: #label put: aValue with: #labelChanged! !

!CEnumTileCostume methodsFor: 'events' stamp: 'ar 3/21/2004 20:34'!
onContextMenu
	| newValue |
	<on: yellowButtonDown>
	newValue := CMenu new chooseFromValues: valueList.
	newValue ifNotNil:[value := newValue].! !

!CEnumTileCostume methodsFor: 'events' stamp: 'ar 3/21/2004 20:34'!
onPlayerChanged
	super onPlayerChanged.
	valueList contents: player valueType enumValues.
	self onValueChanged.! !

!CEnumTileCostume methodsFor: 'events' stamp: 'ar 3/21/2004 20:39'!
onSpinDown
	<on: spinDown in: spinner>
	value := valueList next.! !

!CEnumTileCostume methodsFor: 'events' stamp: 'ar 3/21/2004 20:39'!
onSpinUp
	<on: spinUp in: spinner>
	value := valueList prev.! !

!CEnumTileCostume methodsFor: 'events' stamp: 'ar 3/21/2004 20:35'!
onValueChanged
	<on: valueChanged>
	valueList selection: value.
	label value: value.! !

!CEnumTileCostume methodsFor: 'initialize' stamp: 'ar 6/21/2005 14:59'!
setupCostume
	super setupCostume.

	self layout: CTableLayout new.
	self layout listDirection: #leftToRight.
	self hResizing: #shrinkWrap; vResizing: #spaceFill.
	self height: 22.

	self add: spinner.

	label hResizing: #shrinkWrap; vResizing: #spaceFill.
	label font: CTileLibrary default tileFont.
	label clipping: false.
	self add: label.! !

!CEnumTileCostume methodsFor: 'accessing' stamp: 'ar 3/21/2004 20:31'!
spinner
	"Answer the spinner of the receiver"
	<bewareOf: #spinnerChanged>
	^self propertyValueAt: #spinner! !

!CEnumTileCostume methodsFor: 'accessing' stamp: 'ar 3/21/2004 20:31'!
spinner: aValue
	"Modify the receiver's spinner"
	^self propertyValueAt: #spinner put: aValue with: #spinnerChanged! !

!CEnumTileCostume methodsFor: 'accessing' stamp: 'ar 3/21/2004 20:33'!
valueList
	"Answer the valueList of the receiver"
	<bewareOf: #valueListChanged>
	^self propertyValueAt: #valueList! !

!CEnumTileCostume methodsFor: 'accessing' stamp: 'ar 3/21/2004 20:33'!
valueList: aValue
	"Modify the receiver's valueList"
	^self propertyValueAt: #valueList put: aValue with: #valueListChanged! !

CTileCostume subclass: #CImageResourceTileCostume
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tweak-Tiles-ValueCostumes'!

CImageResourceTileCostume defineFields: '<?xml  version="1.0" ?>
<fields>
	<field toGet="editor" toSet="editor:" changeEvent="editorChanged">editor</field>
	<field toGet="button" toSet="button:" changeEvent="buttonChanged">button</field>
</fields>'!

!CImageResourceTileCostume methodsFor: 'accessing' stamp: 'ar 4/3/2004 14:12'!
button
	"Answer the button of the receiver"
	<bewareOf: #buttonChanged>
	^self propertyValueAt: #button! !

!CImageResourceTileCostume methodsFor: 'accessing' stamp: 'ar 4/3/2004 14:12'!
button: aValue
	"Modify the receiver's button"
	^self propertyValueAt: #button put: aValue with: #buttonChanged! !

!CImageResourceTileCostume methodsFor: 'accessing' stamp: 'ar 4/3/2004 14:12'!
editor
	"Answer the editor of the receiver"
	<bewareOf: #editorChanged>
	^self propertyValueAt: #editor! !

!CImageResourceTileCostume methodsFor: 'accessing' stamp: 'ar 4/3/2004 14:12'!
editor: aValue
	"Modify the receiver's editor"
	^self propertyValueAt: #editor put: aValue with: #editorChanged! !

!CImageResourceTileCostume methodsFor: 'initialize' stamp: 'ar 4/3/2004 14:09'!
initialize
	super initialize.
	self define: #editor as: CInputField new.
	self define: #button as: CPushButton new.! !

!CImageResourceTileCostume methodsFor: 'events' stamp: 'ar 4/3/2004 14:19'!
onButtonPressed
	| fileName file aGraphic |
	<on: fire in: button>
	fileName := CFileOpenDialog openFile: {
		{'Pictures'. #('*.gif' '*.jpg' '*.bmp' '*.png' '*.jpeg')}.
	} label: 'Choose picture'.
	fileName ifNil:[^self].
	file := FileStream readOnlyFileNamed: fileName.
	[aGraphic := CImageResource readFrom: file] ensure:[file close].
	aGraphic ifNotNil:[value := aGraphic].! !

!CImageResourceTileCostume methodsFor: 'events' stamp: 'ar 4/3/2004 14:25'!
onEditorAccept
	| url aStream aGraphic |
	<on: accept in: editor>
	editor contents isEmpty ifTrue:[
		value := nil.
	] ifFalse:[
		url := editor contents asUrl.
		aStream := url contentStream.
		aGraphic := [CImageResource readFrom: aStream] ensure:[aStream close].
		value := aGraphic.
	].! !

!CImageResourceTileCostume methodsFor: 'events' stamp: 'ar 4/3/2004 14:12'!
onPlayerChanged
	super onPlayerChanged.
	self onValueChanged.! !

!CImageResourceTileCostume methodsFor: 'events' stamp: 'ar 4/3/2004 14:16'!
onValueChanged
	<on: valueChanged>
	editor value: (value ifNil:[''] ifNotNil:[value resourceUrl]).! !

!CImageResourceTileCostume methodsFor: 'initialize' stamp: 'bf 7/29/2005 11:52'!
setupCostume
	super setupCostume.

	self layout: CTableLayout new.
	self layout listDirection: #leftToRight.
	self hResizing: #shrinkWrap; vResizing: #spaceFill.
	self height: 1.

	editor hResizing: #rigid; vResizing: #rigid.
	editor extent: 80@20.
	editor borderWidth: 1.
	editor borderColor: CWidgetLibrary default frameColor.
	editor color: Color transparent.
	self add: editor.

	button font: (TextStyle named: 'WindowFont') defaultFont.
	button label: 'r'.
	button extent: 20@20.
	self add: button.
! !

CTileCostume subclass: #CMessageAndArgsCostume
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tweak-Tiles-Costumes'!

CMessageAndArgsCostume defineFields: '<?xml  version="1.0" ?>
<fields>
</fields>'!

!CMessageAndArgsCostume methodsFor: 'initialize' stamp: 'ar 3/7/2004 22:02'!
defaultPlayer
	^CMessageTile! !

!CMessageAndArgsCostume methodsFor: 'private' stamp: 'bf 5/18/2005 18:59'!
newRow
	^CPlayer new
		layout: (CTableLayout new
			listDirection: #leftToRight;
			yourself);
		vResizing: #shrinkWrap;
		hResizing: #shrinkWrap;
		layoutInset: 2;
		yourself! !

!CMessageAndArgsCostume methodsFor: 'events' stamp: 'ar 3/7/2004 18:48'!
onArgumentsChanged
	"Update everything when the player's argument list changes"
	<on: argumentsChanged in: player>
	self onUpdateEverything.! !

!CMessageAndArgsCostume methodsFor: 'events' stamp: 'ar 5/12/2005 20:05'!
onDisplayNameChanged
	"Update everything when the player's display name changes"
	<on: displayNameChanged in: player>
	self onUpdateEverything.! !

!CMessageAndArgsCostume methodsFor: 'events' stamp: 'ar 3/7/2004 18:47'!
onPlayerChanged
	super onPlayerChanged.
	self onUpdateEverything.! !

!CMessageAndArgsCostume methodsFor: 'events' stamp: 'ar 3/7/2004 18:48'!
onSelectorChanged
	"Update everything when the player's selector changes"
	<on: selectorChanged in: player>
	self onUpdateEverything.! !

!CMessageAndArgsCostume methodsFor: 'events' stamp: 'bf 5/18/2005 18:48'!
onUpdateEverything
	| selector nArgs selectorParts displayName |
	self removeAll.
	selector := player selector ifNil:[^self].
	displayName := player displayName ifNil:[selector].
	nArgs := selector numArgs.
	nArgs = 0 ifTrue:[^self add: (self tileForMessagePart: displayName)].
	"remove once we have proper statement tiles"
	(CTileDefinition isStatement: player selector)
		ifTrue: [layout listDirection: #topToBottom; cellPositioning: #leftCenter]
		ifFalse: [layout listDirection: #leftToRight].
	selectorParts := displayName findTokens: $:.
	selectorParts with: player arguments do:[:msgPart :msgArg|
		| row |
		row := layout listDirection == #leftToRight
			ifTrue: [self] 
			ifFalse: [self newRow].
		row add: (self tileForMessagePart: msgPart).
		row add: (self tileForArgument: msgArg).
		row == self ifFalse: [self add: row].
	].


! !

!CMessageAndArgsCostume methodsFor: 'initialize' stamp: 'ar 4/30/2004 15:52'!
setupCostume
	super setupCostume.
	self layout: CTableLayout new.
	self layout listDirection: #leftToRight.
	self hResizing: #spaceFill; vResizing: #shrinkWrap.! !

!CMessageAndArgsCostume methodsFor: 'private' stamp: 'ar 3/7/2004 21:09'!
tileForArgument: arg
	"Answer an appropriate costume for the given argument"
	^arg costume! !

!CMessageAndArgsCostume methodsFor: 'private' stamp: 'ar 3/7/2004 18:55'!
tileForMessagePart: msgPart
	"Answer an appropriate costume for the given message part"
	^CMessagePartCostume on: msgPart! !

CTileCostume subclass: #CMessagePartCostume
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tweak-Tiles-Costumes'!

CMessagePartCostume defineFields: '<?xml  version="1.0" ?>
<fields>
</fields>'!

!CMessagePartCostume methodsFor: 'initialize' stamp: 'ar 3/7/2004 22:03'!
defaultPlayer
	^nil! !

!CMessagePartCostume methodsFor: 'events' stamp: 'bf 4/26/2005 15:50'!
onValueChanged
	<on: valueChanged>
	self text: self value asText
! !

!CMessagePartCostume methodsFor: 'initialize' stamp: 'ar 4/30/2004 15:52'!
setupCostume
	super setupCostume.
	text := ''.
	self textFont: CTileLibrary default tileFont.
	self hResizing: #shrinkWrap.
	self vResizing: #shrinkWrap.
	"self textInset: 3@3."
	self borderWidth: 1.
	self borderColor: CWidgetLibrary default frameColor.
	self borderStyle: #simple.! !

CTileCostume subclass: #CMessageTileCostume
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tweak-Tiles-Costumes'!

CMessageTileCostume defineFields: '<?xml  version="1.0" ?>
<fields>
	<field toGet="fireButton" toSet="fireButton:" changeEvent="fireButtonChanged">fireButton</field>
	<field toGet="receiverTile" toSet="receiverTile:" changeEvent="receiverTileChanged">receiverTile</field>
	<field toGet="argumentsTile" toSet="argumentsTile:" changeEvent="argumentsTileChanged">argumentsTile</field>
	<field toGet="expandArrow" toSet="expandArrow:" changeEvent="expandArrowChanged">expandArrow</field>
</fields>'!

!CMessageTileCostume methodsFor: 'accessing' stamp: 'ar 3/12/2004 16:17'!
argumentsTile
	"Answer the argumentsTile of the receiver"
	<bewareOf: #argumentsTileChanged>
	^self propertyValueAt: #argumentsTile! !

!CMessageTileCostume methodsFor: 'accessing' stamp: 'ar 3/12/2004 16:17'!
argumentsTile: aValue
	"Modify the receiver's argumentsTile"
	^self propertyValueAt: #argumentsTile put: aValue with: #argumentsTileChanged! !

!CMessageTileCostume methodsFor: 'initialize' stamp: 'ar 3/7/2004 22:03'!
defaultPlayer
	^CMessageTile! !

!CMessageTileCostume methodsFor: 'initialize' stamp: 'bf 7/27/2005 17:22'!
disableDrop
	player arguments do: [:ea |
		ea costume dropEnabled: false ]! !

!CMessageTileCostume methodsFor: 'accessing' stamp: 'ar 3/12/2004 16:17'!
expandArrow
	"Answer the expandArrow of the receiver"
	<bewareOf: #expandArrowChanged>
	^self propertyValueAt: #expandArrow! !

!CMessageTileCostume methodsFor: 'accessing' stamp: 'ar 3/12/2004 16:17'!
expandArrow: aValue
	"Modify the receiver's expandArrow"
	^self propertyValueAt: #expandArrow put: aValue with: #expandArrowChanged! !

!CMessageTileCostume methodsFor: 'accessing' stamp: 'ar 3/12/2004 16:16'!
fireButton
	"Answer the fireButton of the receiver"
	<bewareOf: #fireButtonChanged>
	^self propertyValueAt: #fireButton! !

!CMessageTileCostume methodsFor: 'accessing' stamp: 'ar 3/12/2004 16:16'!
fireButton: aValue
	"Modify the receiver's fireButton"
	^self propertyValueAt: #fireButton put: aValue with: #fireButtonChanged! !

!CMessageTileCostume methodsFor: 'initialize' stamp: 'bf 4/22/2005 19:55'!
initialize
	super initialize.
	self define: #fireButton as: CTileButton new.
	self define: #receiverTile as: CReceiverTileCostume new.
	self define: #argumentsTile as: CMessageAndArgsCostume new.
	self define: #expandArrow as: CTileButton new.! !

!CMessageTileCostume methodsFor: 'events' stamp: 'ar 3/7/2004 21:14'!
onFireButtonPressed
	| result |
	<on: fire in: fireButton>
	result := player eval.
	player valueType ifNotNil:[
		(CValueTile on: result) openInHand.
	].! !

!CMessageTileCostume methodsFor: 'events' stamp: 'bf 4/26/2005 15:41'!
onPlayerChanged
	self onReceiverChanged.
	self onValueTypeChanged.
	argumentsTile player: self player.! !

!CMessageTileCostume methodsFor: 'events' stamp: 'bf 4/26/2005 15:49'!
onReceiverChanged
	<on: receiverChanged in: player>
	receiverTile value: player receiver.
	receiverTile reflexive: player isStatementLike not.! !

!CMessageTileCostume methodsFor: 'events' stamp: 'bf 9/27/2004 15:53'!
onTilePadDragged: aTilePad
	super onTilePadDragged: aTilePad.
	self color: CWidgetLibrary default paneColor.
	self borderWidth: 1.! !

!CMessageTileCostume methodsFor: 'events' stamp: 'bf 9/27/2004 15:54'!
onTilePadDropped: aTilePad
	"The receiver -a tile being dragged around- is notified that it just successfully dropped on aTilePad."
	super onTilePadDropped: aTilePad.
	self color: Color transparent.
	self borderWidth: 0.! !

!CMessageTileCostume methodsFor: 'events' stamp: 'ar 3/7/2004 21:12'!
onValueTypeChanged
	<on: valueTypeChanged in: player>
	player valueType 
		ifNil:[fireButton label: '!!']
		ifNotNil:[fireButton label: '?'].! !

!CMessageTileCostume methodsFor: 'accessing' stamp: 'ar 3/12/2004 16:17'!
receiverTile
	"Answer the receiverTile of the receiver"
	<bewareOf: #receiverTileChanged>
	^self propertyValueAt: #receiverTile! !

!CMessageTileCostume methodsFor: 'accessing' stamp: 'ar 3/12/2004 16:17'!
receiverTile: aValue
	"Modify the receiver's receiverTile"
	^self propertyValueAt: #receiverTile put: aValue with: #receiverTileChanged! !

!CMessageTileCostume methodsFor: 'initialize' stamp: 'bf 2/21/2005 17:16'!
setupCostume
	super setupCostume.
	self color: CWidgetLibrary default paneColor.
	self borderStyle: #raised.
	self borderWidth: 1.

	"self color: (Color r: 0.8 g: 1.0 b: 0.8)."
	self layout: CTableLayout new.
	self layout listDirection: #leftToRight.
	self layout rubberBandCells: true.
	self hResizing: #shrinkWrap.
	self vResizing: #shrinkWrap.

	fireButton hResizing: #shrinkWrap.
	fireButton visible: false.
	receiverTile hResizing: #shrinkWrap.
	receiverTile vResizing: #shrinkWrap.
	expandArrow label: '...'.
	expandArrow hResizing: #shrinkWrap; vResizing: #spaceFill.
	expandArrow visible: false.
	self add: fireButton.
	self add: receiverTile.
	self add: argumentsTile.
	self add: expandArrow.! !

CTileCostume subclass: #CNumberTileCostume
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tweak-Tiles-ValueCostumes'!

CNumberTileCostume defineFields: '<?xml  version="1.0" ?>
<fields>
	<field toGet="editor" toSet="editor:" changeEvent="editorChanged">editor</field>
	<field toGet="spinner" toSet="spinner:" changeEvent="spinnerChanged">spinner</field>
</fields>'!

!CNumberTileCostume methodsFor: 'private' stamp: 'bf 5/13/2005 19:13'!
changeValueBy: delta

	editor hasKeyboardFocus
		ifTrue: [self editorValue: self editorValue + delta. 
			editor modified: true]
		ifFalse: [player value: player value + delta].
! !

!CNumberTileCostume methodsFor: 'accessing' stamp: 'ar 3/12/2004 16:15'!
editor
	"Answer the editor of the receiver"
	<bewareOf: #editorChanged>
	^self propertyValueAt: #editor! !

!CNumberTileCostume methodsFor: 'accessing' stamp: 'ar 3/12/2004 16:15'!
editor: aValue
	"Modify the receiver's editor"
	^self propertyValueAt: #editor put: aValue with: #editorChanged! !

!CNumberTileCostume methodsFor: 'private' stamp: 'bf 5/13/2005 18:54'!
editorValue

	^[editor contents asString asNumber] 
		on: Error do:[:ex| ex return: player value].! !

!CNumberTileCostume methodsFor: 'private' stamp: 'bf 5/13/2005 18:57'!
editorValue: aNumber

	editor value: (aNumber roundTo: 0.001)! !

!CNumberTileCostume methodsFor: 'initialize' stamp: 'ar 3/17/2004 03:14'!
initialize
	super initialize.
	self define: #editor as: CInputField new.
	self define: #spinner as: CSpinnerButtons new.! !

!CNumberTileCostume methodsFor: 'events' stamp: 'bf 5/13/2005 18:59'!
onAccept
	| newValue |
	<on: accept in: editor>
	newValue := self editorValue.
	player value: newValue.
	editor modified: false.
	self editorValue: newValue.
! !

!CNumberTileCostume methodsFor: 'events' stamp: 'ar 6/18/2005 18:40'!
onClose
	"Make sure spinner is gone when I go away"
	<on: close>
	spinner close.! !

!CNumberTileCostume methodsFor: 'events' stamp: 'bf 5/13/2005 17:58'!
onFocusLost
	<on: keyboardFocusLeave in: editor>
	editor modified
		ifTrue: [self onAccept]
		ifFalse: [self onValueChanged]! !

!CNumberTileCostume methodsFor: 'events' stamp: 'bf 2/22/2005 15:33'!
onGeometryChanged
	<on: geometryChanged>

	spinner asPrimCostume	
		align: spinner globalBounds leftCenter with: self globalBounds rightCenter.
! !

!CNumberTileCostume methodsFor: 'events' stamp: 'ar 6/18/2005 18:38'!
onMouseEnter
	"Show the spinner when the mouse enters the receiver"
	| box |
	<on: mouseEnter>
	spinner open.
	box := self worldBounds.
	spinner left: box right.
	spinner top: box top + (box height - spinner height // 2).! !

!CNumberTileCostume methodsFor: 'events' stamp: 'ar 6/18/2005 18:39'!
onMouseLeave
	"Hide the spinner unless the user moves mouse into it"
	<on: mouseLeave>
	(spinner containsPoint: spinner cursorPoint) ifFalse:[spinner close].! !

!CNumberTileCostume methodsFor: 'events' stamp: 'ar 6/18/2005 18:14'!
onMouseLeaveSpinner
	"Hide the spinner when the mouse leaves the receiver"
	<on: mouseLeave in: spinner>
	spinner close.! !

!CNumberTileCostume methodsFor: 'events' stamp: 'ar 3/7/2004 22:23'!
onPlayerChanged
	super onPlayerChanged.
	self onValueChanged.! !

!CNumberTileCostume methodsFor: 'events' stamp: 'bf 5/13/2005 19:04'!
onSpinDown
	<on: spinDown in: spinner>
	self changeValueBy: -1! !

!CNumberTileCostume methodsFor: 'events' stamp: 'bf 5/13/2005 19:04'!
onSpinUp
	<on: spinUp in: spinner>
	self changeValueBy: 1! !

!CNumberTileCostume methodsFor: 'events' stamp: 'bf 5/13/2005 18:58'!
onValueChanged
	<on: valueChanged>
	editor hasKeyboardFocus
		ifFalse: [self editorValue: player value].! !

!CNumberTileCostume methodsFor: 'initialize' stamp: 'bf 7/29/2005 11:53'!
setupCostume
	super setupCostume.

	self layout: CTableLayout new.
	self layout listDirection: #leftToRight.
	self hResizing: #shrinkWrap; vResizing: #shrinkWrap.
	self height: 1.

	editor hResizing: #shrinkWrap; vResizing: #shrinkWrap.
	editor fillStyle: #none.
	editor borderWidth: 0.
	self add: editor.

	spinner borderStyle: #simple.
	spinner borderWidth: 1.
	spinner borderColor: CWidgetLibrary default frameColor.! !

!CNumberTileCostume methodsFor: 'accessing' stamp: 'ar 3/12/2004 16:15'!
spinner
	"Answer the spinner of the receiver"
	<bewareOf: #spinnerChanged>
	^self propertyValueAt: #spinner! !

!CNumberTileCostume methodsFor: 'accessing' stamp: 'ar 3/12/2004 16:15'!
spinner: aValue
	"Modify the receiver's spinner"
	^self propertyValueAt: #spinner put: aValue with: #spinnerChanged! !

CTileCostume subclass: #COperatorTileCostume
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tweak-Tiles-Costumes'!

COperatorTileCostume defineFields: '<?xml  version="1.0" ?>
<fields>
</fields>'!

!COperatorTileCostume methodsFor: 'initialize' stamp: 'bf 7/27/2005 17:22'!
disableDrop
	{player receiver}, player arguments do: [:ea |
		ea costume dropEnabled: false ]! !

!COperatorTileCostume methodsFor: 'highlighting' stamp: 'bf 7/26/2005 17:00'!
highlightReceiver
	| overlay |
	overlay := CFillingHighlight new.
	overlay name: #replaceOverlay.
	overlay fill: (Color green alpha: 0.5).

	player receiver addHighlight: overlay
! !

!COperatorTileCostume methodsFor: 'events' stamp: 'ar 3/22/2004 16:19'!
onPlayerChanged
	super onPlayerChanged.
	self onUpdateEverything.! !

!COperatorTileCostume methodsFor: 'events' stamp: 'bf 9/27/2004 15:53'!
onTilePadDragged: aTilePad
	super onTilePadDragged: aTilePad.
	self color: CWidgetLibrary default paneColor.
	self borderWidth: 1.! !

!COperatorTileCostume methodsFor: 'events' stamp: 'bf 7/26/2005 17:00'!
onTilePadDropped: aTilePad
	"The receiver -a tile being dragged around- is notified that it just successfully dropped on aTilePad."
	super onTilePadDropped: aTilePad.
	self color: Color transparent.
	self borderWidth: 0.
	self unhighlightReceiver.
! !

!COperatorTileCostume methodsFor: 'events' stamp: 'bf 7/31/2005 17:25'!
onTilePadEnter: aTilePad
	"The receiver -a tile being dragged around- is notified that it just entered aTilePad"
	<on: tilePadEnter in: player>

	(player willReplaceReceiverWith: aTilePad player expression)
		ifTrue: [self highlightReceiver]
		ifFalse: [self unhighlightReceiver]
! !

!COperatorTileCostume methodsFor: 'events' stamp: 'bf 7/26/2005 17:00'!
onTilePadLeave: aTilePad
	"The receiver -a tile being dragged around- is notified that it just left aTilePad"
	<on: tilePadLeave in: player>

	self unhighlightReceiver.
! !

!COperatorTileCostume methodsFor: 'events' stamp: 'bf 5/18/2005 18:21'!
onUpdateEverything
	self removeAll.
	"remove once we have proper statement tiles"
	(CTileDefinition isStatement: player selector)
		ifTrue: [layout listDirection: #topToBottom; cellPositioning: #leftCenter]
		ifFalse: [layout listDirection: #leftToRight].
	self add: player receiver.
	self add: (CMessageAndArgsCostume new player: player; yourself).
	! !

!COperatorTileCostume methodsFor: 'initialize' stamp: 'ar 4/30/2004 15:52'!
setupCostume
	super setupCostume.
	self color: CWidgetLibrary default paneColor.
	self borderStyle: #raised.
	self borderWidth: 1.

	self layout: CTableLayout new.
	self layout listDirection: #leftToRight.
	self layout rubberBandCells: true.
	self hResizing: #shrinkWrap.
	self vResizing: #shrinkWrap.! !

!COperatorTileCostume methodsFor: 'highlighting' stamp: 'bf 7/26/2005 16:59'!
unhighlightReceiver
	player receiver removeHighlight: #replaceOverlay! !

CTileCostume subclass: #CPropertyTileCostume
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tweak-Tiles-Costumes'!

CPropertyTileCostume defineFields: '<?xml  version="1.0" ?>
<fields>
	<field toGet="receiverTile" toSet="receiverTile:" changeEvent="receiverTileChanged">receiverTile</field>
	<field toGet="propertyName" toSet="propertyName:" changeEvent="propertyNameChanged">propertyName</field>
</fields>'!

!CPropertyTileCostume methodsFor: 'initialize' stamp: 'ar 3/21/2004 18:15'!
defaultPlayer
	^CPropertyTile! !

!CPropertyTileCostume methodsFor: 'initialize' stamp: 'ar 3/22/2004 17:45'!
initialize
	super initialize.
	self define: #fireButton as: CTileButton new.
	self define: #receiverTile as: CReceiverTileCostume new.
	self define: #propertyName as: CLabel new.
	receiverTile reflexive: true.! !

!CPropertyTileCostume methodsFor: 'events' stamp: 'ar 5/12/2005 20:06'!
onDisplayNameChanged
	<on: displayNameChanged in: player>
	self updatePropertyName.! !

!CPropertyTileCostume methodsFor: 'events' stamp: 'ar 5/12/2005 20:07'!
onPlayerChanged
	super onPlayerChanged.
	self onReceiverChanged.
	self updatePropertyName.! !

!CPropertyTileCostume methodsFor: 'events' stamp: 'ar 5/12/2005 20:07'!
onPropertyChanged
	<on: propertyChanged in: player>
	self updatePropertyName.! !

!CPropertyTileCostume methodsFor: 'events' stamp: 'ar 3/21/2004 21:15'!
onReceiverChanged
	<on: receiverChanged in: player>
	receiverTile value: player receiver.! !

!CPropertyTileCostume methodsFor: 'events' stamp: 'bf 9/27/2004 15:53'!
onTilePadDragged: aTilePad
	super onTilePadDragged: aTilePad.
	self color: CWidgetLibrary default paneColor.
	self borderWidth: 1.! !

!CPropertyTileCostume methodsFor: 'events' stamp: 'bf 9/27/2004 15:54'!
onTilePadDropped: aTilePad
	"The receiver -a tile being dragged around- is notified that it just successfully dropped on aTilePad."
	super onTilePadDropped: aTilePad.
	self color: Color transparent.
	self borderWidth: 0.! !

!CPropertyTileCostume methodsFor: 'accessing' stamp: 'ar 3/21/2004 18:17'!
propertyName
	"Answer the propertyName of the receiver"
	<bewareOf: #propertyNameChanged>
	^self propertyValueAt: #propertyName! !

!CPropertyTileCostume methodsFor: 'accessing' stamp: 'ar 3/21/2004 18:17'!
propertyName: aValue
	"Modify the receiver's propertyName"
	^self propertyValueAt: #propertyName put: aValue with: #propertyNameChanged! !

!CPropertyTileCostume methodsFor: 'accessing' stamp: 'ar 3/21/2004 18:16'!
receiverTile
	"Answer the receiverTile of the receiver"
	<bewareOf: #receiverTileChanged>
	^self propertyValueAt: #receiverTile! !

!CPropertyTileCostume methodsFor: 'accessing' stamp: 'ar 3/21/2004 18:16'!
receiverTile: aValue
	"Modify the receiver's receiverTile"
	^self propertyValueAt: #receiverTile put: aValue with: #receiverTileChanged! !

!CPropertyTileCostume methodsFor: 'initialize' stamp: 'ar 4/30/2004 15:53'!
setupCostume
	super setupCostume.
	self color: CWidgetLibrary default paneColor.
	self borderStyle: #raised.
	self borderWidth: 1.

	self layout: CTableLayout new.
	self layout listDirection: #leftToRight.
	self layout rubberBandCells: true.
	self hResizing: #shrinkWrap.
	self vResizing: #shrinkWrap.

	self add: receiverTile.

	propertyName font: CTileLibrary default tileFont.
	propertyName clipping: false.
	propertyName borderWidth: 1.
	propertyName borderColor: CWidgetLibrary default frameColor.
	propertyName borderStyle: #simple.
	self add: propertyName.! !

!CPropertyTileCostume methodsFor: 'events' stamp: 'ar 5/12/2005 20:07'!
updatePropertyName
	propertyName value: (player displayName ifNil:[player property])
! !

CTileCostume subclass: #CReceiverTileCostume
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tweak-Tiles-Costumes'!

CReceiverTileCostume defineFields: '<?xml  version="1.0" ?>
<fields>
	<field toGet="reflexive" toSet="reflexive:" changeEvent="reflexiveChanged">reflexive</field>
</fields>'!

!CReceiverTileCostume methodsFor: 'initialize' stamp: 'ar 3/21/2004 21:15'!
defaultPlayer
	^nil! !

!CReceiverTileCostume methodsFor: 'initialize' stamp: 'ar 3/22/2004 17:44'!
initialize
	super initialize.
	reflexive := false.! !

!CReceiverTileCostume methodsFor: 'events' stamp: 'ar 3/22/2004 17:45'!
onNameChanged
	<on: nameChanged in: value>
	reflexive 
		ifTrue:[self text: value name,'''s']
		ifFalse:[self text: value name].! !

!CReceiverTileCostume methodsFor: 'events' stamp: 'ar 3/21/2004 21:21'!
onValueChanged
	<on: valueChanged>
	self onNameChanged.! !

!CReceiverTileCostume methodsFor: 'accessing' stamp: 'ar 3/22/2004 17:44'!
reflexive
	"Answer the reflexive of the receiver"
	<bewareOf: #reflexiveChanged>
	^self propertyValueAt: #reflexive! !

!CReceiverTileCostume methodsFor: 'accessing' stamp: 'ar 3/22/2004 17:44'!
reflexive: aValue
	"Modify the receiver's reflexive"
	^self propertyValueAt: #reflexive put: aValue with: #reflexiveChanged! !

!CReceiverTileCostume methodsFor: 'initialize' stamp: 'ar 4/30/2004 15:53'!
setupCostume
	super setupCostume.
	text := ''.
	self textFont: CTileLibrary default tileFont.
	self hResizing: #shrinkWrap; vResizing: #shrinkWrap.
	"self textInset: 3@3."
	self borderWidth: 1.
	self borderColor: CWidgetLibrary default frameColor.
	self borderStyle: #simple.! !

CTileCostume subclass: #CScriptNameCostume
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tweak-Tiles-ValueCostumes'!

CScriptNameCostume defineFields: '<?xml  version="1.0" ?>
<fields>
	<field toGet="editor" toSet="editor:" changeEvent="editorChanged">editor</field>
</fields>'!

!CScriptNameCostume methodsFor: 'accessing' stamp: 'ar 4/20/2005 13:34'!
editor
	"Answer the editor of the receiver"
	<bewareOf: #editorChanged>
	^self propertyValueAt: #editor! !

!CScriptNameCostume methodsFor: 'accessing' stamp: 'ar 4/20/2005 13:34'!
editor: aValue
	"Modify the receiver's editor"
	^self propertyValueAt: #editor put: aValue with: #editorChanged! !

!CScriptNameCostume methodsFor: 'initialize' stamp: 'ar 4/20/2005 13:34'!
initialize
	super initialize.
	self define: #editor as: CInputField new.! !

!CScriptNameCostume methodsFor: 'events' stamp: 'ar 4/20/2005 13:51'!
onAcceptValue
	<on: accept in: editor>
	value := editor value asSymbol.! !

!CScriptNameCostume methodsFor: 'events' stamp: 'ar 4/20/2005 13:34'!
onPlayerChanged
	super onPlayerChanged.
	self onValueChanged.! !

!CScriptNameCostume methodsFor: 'events' stamp: 'ar 4/20/2005 13:34'!
onValueChanged
	<on: valueChanged>
	editor value: value.! !

!CScriptNameCostume methodsFor: 'initialize' stamp: 'ar 6/21/2005 15:08'!
setupCostume
	super setupCostume.

	self layout: CTableLayout new.
	self layout listDirection: #leftToRight.
	self hResizing: #shrinkWrap; vResizing: #spaceFill.
	self height: 22.

	editor width: 60.
	editor color: Color transparent.
	editor hResizing: #shrinkWrap; vResizing: #spaceFill.
	editor font: CTileLibrary default tileFont.
	editor borderWidth: 0.
	self add: editor.! !

CTileCostume subclass: #CSequenceTileCostume
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tweak-Tiles-Costumes'!

CSequenceTileCostume subclass: #CScriptorTileCostume
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tweak-Tiles-Costumes'!

CScriptorTileCostume defineFields: '<?xml  version="1.0" ?>
<fields>
	<field toGet="header" toSet="header:" changeEvent="headerChanged">header</field>
</fields>'!

!CScriptorTileCostume methodsFor: 'actions' stamp: 'bf 5/17/2005 17:41'!
adjustTicking

	| answer |
	answer := CMenu new
				label: 'Frequency (now ', player frequency asString, ')';
				value: player frequency;
				chooseFromValues: #(1 2 5 8 10 25 50 100 'other...').

	[answer isNumber] whileFalse: [
		answer ifNil: [^self].
		answer := CDialog request: 'Frequency' initialAnswer: answer.
		[answer := answer asNumber] on: Error do: [:ignore | ].
	].

	player frequency: answer
! !

!CScriptorTileCostume methodsFor: 'actions' stamp: 'bf 6/21/2004 16:49'!
doEval

	| oldFill |
	oldFill := header fireButton fill colors.
	header fireButton fill colors: {Color yellow. Color white}.
	player eval.
	self waitTick.
	header fireButton fill colors := oldFill. 
! !

!CScriptorTileCostume methodsFor: 'accessing' stamp: 'ar 3/12/2004 16:17'!
header
	"Answer the header of the receiver"
	<bewareOf: #headerChanged>
	^self propertyValueAt: #header! !

!CScriptorTileCostume methodsFor: 'accessing' stamp: 'ar 3/12/2004 16:17'!
header: aValue
	"Modify the receiver's header"
	^self propertyValueAt: #header put: aValue with: #headerChanged! !

!CScriptorTileCostume methodsFor: 'initialize' stamp: 'ar 3/17/2004 03:14'!
initialize
	super initialize.
	self define: #header as: CScriptorHeader new.! !

!CScriptorTileCostume methodsFor: 'events' stamp: 'bf 11/30/2004 17:27'!
onCloseButtonFired
	(self confirm: 'This will permanently destroy this script. Continue?')
		ifTrue: [player remove]! !

!CScriptorTileCostume methodsFor: 'events' stamp: 'bf 11/30/2004 15:45'!
onCollapseButtonFired
	self close.! !

!CScriptorTileCostume methodsFor: 'events' stamp: 'bf 2/21/2005 17:39'!
onDragReceiver
	(CValueTile on: player receiver) openInHand.! !

!CScriptorTileCostume methodsFor: 'events' stamp: 'bf 11/30/2004 15:35'!
onFire
	"Fire once, wait half a second, and then fire repeatedly"
	| startTime repeat |

	self doEval.

	startTime := Time millisecondClockValue.
	repeat := false.
	[	repeat
			ifTrue: [self doEval]
			ifFalse: [repeat := (Time millisecondsSince: startTime) > 500].
		hand anyButtonPressed]
	whileTrue: [self waitTick]! !

!CScriptorTileCostume methodsFor: 'events' stamp: 'bf 10/4/2004 17:14'!
onPlayerChanged
	super onPlayerChanged.
	self onSelectorChanged.
	self onReceiverChanged.
	self onTriggerChanged.
! !

!CScriptorTileCostume methodsFor: 'events' stamp: 'bf 9/24/2004 17:56'!
onReceiverChanged
	<on: receiverChanged in: player>
	self onReceiverNameChanged.
	self startScript: #onReceiverNameChanged when: {player receiver. #nameChanged}.
! !

!CScriptorTileCostume methodsFor: 'events' stamp: 'bf 9/24/2004 17:53'!
onReceiverNameChanged
	header rcvrLabel value: player receiver name.! !

!CScriptorTileCostume methodsFor: 'events' stamp: 'bf 11/29/2004 18:36'!
onRenameReceiver
	
	| answer |
	answer := CDialog request: 'Rename object' initialAnswer: player receiver name centerAt: hand position.
	answer isEmptyOrNil
		ifFalse: [player receiver name: answer]! !

!CScriptorTileCostume methodsFor: 'events' stamp: 'ar 12/19/2004 19:35'!
onRenameScript
	player selector: header scriptName value asSymbol.! !

!CScriptorTileCostume methodsFor: 'events' stamp: 'bf 10/19/2005 10:20'!
onRenameTrigger
	
	| answer list menu |
	list := #(
		(normal normal)
		-
		(paused paused)
		(ticking (
			('1' ticking1)
			('2' ticking2)
			('5' ticking5)
			('8' ticking8)
			('10' ticking10)
			('25' ticking25)
			('50' ticking50)
			('100' ticking100)
			('other...' tickingRate)
		))
		-
		(mouseDown mouseDown)
		(mouseStillDown mouseStillDown)
		(mouseUp mouseUp)
		-
		(mouseEnter mouseEnter)
		(mouseLeave mouseLeave)
		(mouseEnterDragging dragEnter)
		(mouseLeaveDragging dragLeave)
		-
		(opening open)
		(closing close)
		-
		('more...' more)
	).
	menu := CMenu new.
	menu label: 'Choose Trigger'.
	answer := menu chooseFrom: list.
	answer ifNil:[^self].

	(answer beginsWith: 'ticking') ifTrue:[
		answer := answer allButFirst: 7.
		answer first isDigit ifFalse:[
			answer := CDialog request: 'Tick rate?' initialAnswer:''.
			answer isEmptyOrNil ifTrue:[^self].
		].
		answer := [Number readFrom: answer] on: Error do:[:ex| ex return: nil].
		answer ifNil:[^self].
		player frequency: answer.
		^player start.
	].
	answer == #normal ifTrue: [^player stop].
	answer == #paused ifTrue: [^player pause].
	answer == #ticking ifTrue: [^player start].
	answer == #more ifTrue:[
		answer := (player trigger ifNil: [#on->'']) value.
		answer isString ifFalse: [answer := ''].
		[
			answer := CDialog request: 'Which event?' initialAnswer: answer centerAt: hand position.
			answer ifNil: [^self].
			answer := answer withBlanksTrimmed.
			answer isEmpty ifTrue: [^player trigger: nil].
			(answer first isLetter and: [answer allSatisfy: [:c | c isAlphaNumeric]])
				ifTrue: [^player trigger: #on->answer asSymbol]
				ifFalse: [self inform: 'Event names should consist of letters and digits only!!'].
		] repeat
	].
	"otherwise"
	^player trigger: #on->answer.

! !

!CScriptorTileCostume methodsFor: 'events' stamp: 'ar 3/8/2004 23:35'!
onSelectorChanged
	<on: selectorChanged in: player>
	header scriptName value: player selector! !

!CScriptorTileCostume methodsFor: 'events' stamp: 'bf 5/17/2005 18:42'!
onTickerMouseDown
	"Mouse went down in ticker"
	| c evt |
	c := header ticker color.
	header ticker color: Color veryLightGray.
	
	self startScript: [
		self wait: 0.5.
		self signal: #mouseUpTimeOut].

	evt := self waitUntilAnyOf: {header ticker. #mouseUp. self. #mouseUpTimeOut}.

	evt selector == #mouseUp
		ifTrue: [
			self toggleTicking]
		ifFalse: [
			header ticker color: c.
			self adjustTicking]
! !

!CScriptorTileCostume methodsFor: 'events' stamp: 'bf 5/4/2005 19:48'!
onTickingChanged
	| ticking |
	<on: tickingChanged in: player>

	ticking := player ticking.

	header ticker color: (ticking
		ifNil: [header ticker defaultColor]
		ifNotNil: [
			ticking
				ifTrue: [Color lightGreen]
				ifFalse: [Color lightRed]]).

	header ticker ticking: ticking == true.

	self onTriggerChanged.


! !

!CScriptorTileCostume methodsFor: 'events' stamp: 'bf 5/4/2005 19:42'!
onTriggerChanged
	<on: triggerChanged in: player>

	header trigger value: (player trigger
		ifNil: ['normal']
		ifNotNil: [
			player trigger key = #on
				ifTrue: [player trigger value]
				ifFalse: [
					player paused
						ifTrue: ['paused']
						ifFalse: ['ticking']
				]]).
! !

!CScriptorTileCostume methodsFor: 'initialize' stamp: 'bf 5/11/2005 17:18'!
setupCostume
	super setupCostume.

	self borderStyle: #raised.
	self borderWidth: 4.
	self borderColor: (Color r: 0.995 g: 0.804 b: 0.404).
	self roundCornerSpec: #(1 4).

	target borderWidth: 2.
	target borderStyle: #inset.
	target borderColor: (Color r: 0.995 g: 0.804 b: 0.404).
	target layout layoutInset: 2; cellInset: 1.
	target color: Color white.

	self layout: CTableLayout new.
	self hResizing: #shrinkWrap.
	self vResizing: #shrinkWrap.
	self layout rubberBandCells: true.

	self addFirst: header.

	self startScript: #onFire when: {header fireButton. #mouseDown}.
	self startScript: #onCollapseButtonFired when: {header collapseButton. #fire}.
	self startScript: #onCloseButtonFired when: {header closeButton. #fire}.
	self startScript: #onRenameReceiver when: {header rcvrLabel. #mouseDoubleClick}.
	self startScript: #onDragReceiver when: {header rcvrLabel. #mouseDrag}.
	self startScript: #onRenameScript when: {header scriptName. #valueChanged}.
	self startScript: #onRenameTrigger when: {header trigger. #mouseDown}.
	self startScript: #onTickerMouseDown when: {header ticker. #mouseDown}.
! !

!CScriptorTileCostume methodsFor: 'actions' stamp: 'bf 5/17/2005 17:10'!
toggleTicking
	player ticking == true	"may be nil"
		ifTrue: [player pause]
		ifFalse: [player start]! !

CSequenceTileCostume defineFields: '<?xml  version="1.0" ?>
<fields>
	<field toGet="target" toSet="target:" changeEvent="targetChanged">target</field>
	<field toGet="spacer" toSet="spacer:" changeEvent="spacerChanged">spacer</field>
</fields>'!

!CSequenceTileCostume methodsFor: 'initialize' stamp: 'ar 3/8/2004 21:05'!
defaultPlayer
	^CSequenceTile! !

!CSequenceTileCostume methodsFor: 'initialize' stamp: 'ar 3/17/2004 03:15'!
initialize
	super initialize.
	self define: #spacer as: CPlayer new.
	self define: #target as: CPlayer new.! !

!CSequenceTileCostume methodsFor: 'drag and drop' stamp: 'ar 3/14/2004 21:54'!
onDragEnter
	<on: dragEnter in: target>
	self tileSignal: #tileSequenceEnter.
	self onTileEnter.! !

!CSequenceTileCostume methodsFor: 'events' stamp: 'ar 3/1/2005 09:35'!
onDragStart: evt
	"Handle a drag start"
	| dragTile |
	<on: mouseDrag in: target>
	dragTile := ((target findPlayerAt: evt position) ifNil:[^self]) asPlayer.
	player statements remove: dragTile.
	player statements isEmpty 
		ifTrue:[target height: spacer height].
	hand grab: dragTile.
! !

!CSequenceTileCostume methodsFor: 'events' stamp: 'bf 2/17/2005 17:21'!
onPlayerChanged
	super onPlayerChanged.
	self onStatementsModified.! !

!CSequenceTileCostume methodsFor: 'events' stamp: 'bf 2/17/2005 17:07'!
onStatementsModified
	<on: statementsModified in: player>
	target removeAll.
	target addAll: player statements.! !

!CSequenceTileCostume methodsFor: 'drag and drop' stamp: 'ar 6/19/2005 10:40'!
onTileEnter
	| newTile evt |
	newTile := (self hand dragContents ifNil:[^self]) asPlayer.
	newTile == self player ifTrue:[^self]. "spurious enter"
	newTile isTilePlayer ifFalse:[^self].
	newTile isStatementLike ifFalse:[^self].

	self startTicking: #trackDropZones at: 100.
	evt := self waitUntilAnyOf: {
		target. #tileSequenceEnter.
		target. #dragLeave.
		target. #dragDrop.
	}.
	self stopScript: #trackDropZones.

	evt selector == #dragDrop ifTrue:[ | delta steps |
		"animate tile to target location"
		steps := 5.
		delta := spacer worldBounds topLeft - newTile worldBounds topLeft.
		1 to: steps do:[:i|
			newTile moveBy: delta // steps.
			self waitTick.
		].
		self playSoundNamed: 'scritch'.
		player statements add: newTile 
			at: (target elements indexOf: spacer).
		newTile signal: #tileSequenceDrop with: self player.
		newTile signal: #dropAccept with: self player.
	].
	evt selector == #dragLeave ifTrue:[
		self tileSignal: #tileSequenceLeave.
	].
	spacer close. 
! !

!CSequenceTileCostume methodsFor: 'drag and drop' stamp: 'ar 3/25/2004 14:41'!
onTileSequenceLeave
	<on: tileSequenceLeave>
	(target containsPoint: target cursorPoint) 
		ifTrue:[self startScript: #onTileEnter]
		ifFalse:[self tileSignal: #tileSequenceLeave].! !

!CSequenceTileCostume methodsFor: 'initialize' stamp: 'bf 2/21/2005 19:35'!
setupCostume
	super setupCostume.
	self color: Color white.
	self borderColor: CWidgetLibrary default frameColor.
	self borderStyle: #inset.
	self borderWidth: 5.
	self layout: CTableLayout new.
	self hResizing: #spaceFill; vResizing: #shrinkWrap.

	target layout: CTableLayout new.
	target hResizing: #spaceFill; vResizing: #shrinkWrap.
	target height: 18.
	target asPrimCostume minWidth: 64.
	target layout cellPositioning: #topLeft.
	self add: target.

	spacer hResizing: #spaceFill; vResizing: #shrinkWrap.
	spacer fillStyle: #radialGradient.
	spacer fillColors: {Color white. Color r: 1.0 g: 0.677 b: 0.032}.

	spacer borderWidth: 1.
	spacer borderStyle: #simple.
	spacer borderColor: CWidgetLibrary default frameColor.
	spacer text: '<-- drop here -->'.
	spacer textFont: CTileLibrary default tileFont.
	spacer textAnchorPoint: #center.! !

!CSequenceTileCostume methodsFor: 'accessing' stamp: 'ar 3/12/2004 16:18'!
spacer
	"Answer the spacer of the receiver"
	<bewareOf: #spacerChanged>
	^self propertyValueAt: #spacer! !

!CSequenceTileCostume methodsFor: 'accessing' stamp: 'ar 3/12/2004 16:18'!
spacer: aValue
	"Modify the receiver's spacer"
	^self propertyValueAt: #spacer put: aValue with: #spacerChanged! !

!CSequenceTileCostume methodsFor: 'accessing' stamp: 'ar 3/12/2004 16:18'!
target
	"Answer the target of the receiver"
	<bewareOf: #targetChanged>
	^self propertyValueAt: #target! !

!CSequenceTileCostume methodsFor: 'accessing' stamp: 'ar 3/12/2004 16:18'!
target: aValue
	"Modify the receiver's target"
	^self propertyValueAt: #target put: aValue with: #targetChanged! !

!CSequenceTileCostume methodsFor: 'drag and drop' stamp: 'ar 4/2/2004 14:53'!
trackDropZones
	| refPt above |
	spacer close.
	refPt := target cursorPoint.
	"figure out the guy above the cursor point so we know 
	what the reference y for resizing the spacers is"
	above := target elements detect:[:any| 
		any ~~ spacer and:[any center y > refPt y]
	] ifNone:[nil].
	above 
		ifNil:[target addLast: spacer]
		ifNotNil:[target add: spacer before: above].! !

CTileCostume subclass: #CSoundTileCostume
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tweak-Tiles-ValueCostumes'!

CSoundTileCostume defineFields: '<?xml  version="1.0" ?>
<fields>
	<field toGet="label" toSet="label:" changeEvent="labelChanged">label</field>
	<field toGet="spinner" toSet="spinner:" changeEvent="spinnerChanged">spinner</field>
</fields>'!

!CSoundTileCostume methodsFor: 'accessing' stamp: 'ar 3/21/2004 18:46'!
label
	"Answer the label of the receiver"
	<bewareOf: #labelChanged>
	^self propertyValueAt: #label! !

!CSoundTileCostume methodsFor: 'accessing' stamp: 'ar 3/21/2004 18:46'!
label: aValue
	"Modify the receiver's label"
	^self propertyValueAt: #label put: aValue with: #labelChanged! !

!CSoundTileCostume methodsFor: 'accessing' stamp: 'ar 3/21/2004 18:46'!
spinner
	"Answer the spinner of the receiver"
	<bewareOf: #spinnerChanged>
	^self propertyValueAt: #spinner! !

!CSoundTileCostume methodsFor: 'accessing' stamp: 'ar 3/21/2004 18:46'!
spinner: aValue
	"Modify the receiver's spinner"
	^self propertyValueAt: #spinner put: aValue with: #spinnerChanged! !

CTileCostume subclass: #CStringTileCostume
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tweak-Tiles-ValueCostumes'!

CStringTileCostume defineFields: '<?xml  version="1.0" ?>
<fields>
	<field toGet="editor" toSet="editor:" changeEvent="editorChanged">editor</field>
</fields>'!

!CStringTileCostume methodsFor: 'accessing' stamp: 'ar 3/28/2004 22:42'!
editor
	"Answer the editor of the receiver"
	<bewareOf: #editorChanged>
	^self propertyValueAt: #editor! !

!CStringTileCostume methodsFor: 'accessing' stamp: 'ar 3/28/2004 22:42'!
editor: aValue
	"Modify the receiver's editor"
	^self propertyValueAt: #editor put: aValue with: #editorChanged! !

!CStringTileCostume methodsFor: 'initialize' stamp: 'ar 3/28/2004 22:43'!
initialize
	super initialize.
	self define: #editor as: CInputField new.! !

!CStringTileCostume methodsFor: 'events' stamp: 'ar 3/28/2004 22:45'!
onAcceptValue
	<on: accept in: editor>
	value := editor value.! !

!CStringTileCostume methodsFor: 'events' stamp: 'ar 3/28/2004 22:44'!
onPlayerChanged
	super onPlayerChanged.
	self onValueChanged.! !

!CStringTileCostume methodsFor: 'events' stamp: 'ar 3/28/2004 22:44'!
onValueChanged
	<on: valueChanged>
	editor value: value.! !

!CStringTileCostume methodsFor: 'initialize' stamp: 'ar 4/30/2004 15:54'!
setupCostume
	super setupCostume.

	self layout: CTableLayout new.
	self layout listDirection: #leftToRight.
	self hResizing: #shrinkWrap; vResizing: #spaceFill.
	self height: 22.

	editor width: 60.
	editor color: Color transparent.
	editor hResizing: #rigid; vResizing: #spaceFill.
	editor font: CTileLibrary default tileFont.
	self add: editor.! !

CTileCostume defineFields: '<?xml  version="1.0" ?>
<fields>
</fields>'!

!CTileCostume methodsFor: 'events' stamp: 'bf 7/27/2005 17:44'!
aboutToBeDragged
	self top: hand position y + 8.! !

!CTileCostume methodsFor: 'events' stamp: 'bf 9/27/2004 16:01'!
onTilePadDragged: aTilePad
	"The receiver -a tile being dragged around- is notified that it just was dragged from aTilePad"
	<on: tilePadDragged in: player>
! !

!CTileCostume methodsFor: 'events' stamp: 'bf 4/26/2005 17:37'!
onTilePadDropped: aTilePad
	"The receiver -a tile being dragged around- is notified that it just successfully dropped on aTilePad."
	<on: tilePadDropped in: player>
	self color: Color transparent.
! !

!CTileCostume methodsFor: 'initialize' stamp: 'ar 4/30/2004 15:54'!
setupCostume
	super setupCostume.
	self color: Color transparent.
	self clipping: false.
! !

!CTileCostume methodsFor: 'events' stamp: 'ar 3/30/2004 16:32'!
tileSignal: eventName
	"Find a guy in our container list who handles the event and let him know about it"
	self allContainersDo:[:parent|
		(parent handlesEvent: eventName) ifTrue:[^parent signal: eventName with: self].
	].! !

CTileCostume subclass: #CValueTileCostume
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tweak-Tiles-ValueCostumes'!

CValueTileCostume defineFields: '<?xml  version="1.0" ?>
<fields>
</fields>'!

!CValueTileCostume methodsFor: 'initialize' stamp: 'ar 3/7/2004 22:04'!
defaultPlayer
	^CValueTile! !

!CValueTileCostume methodsFor: 'events' stamp: 'bf 2/21/2005 19:46'!
onNameChanged
	<on: nameChanged in: value>
	self onValueChanged! !

!CValueTileCostume methodsFor: 'events' stamp: 'ar 3/7/2004 19:06'!
onPlayerChanged
	super onPlayerChanged.
	self onValueChanged.! !

!CValueTileCostume methodsFor: 'events' stamp: 'ar 3/17/2004 17:34'!
onValueChanged
	<on: valueChanged>
	self text: player value asText.! !

!CValueTileCostume methodsFor: 'initialize' stamp: 'ar 6/21/2005 15:00'!
setupCostume
	super setupCostume.
	self hResizing: #shrinkWrap.
	self vResizing: #shrinkWrap.
	self roundCorners: true.
	self color: Color white.
	self layoutInset: 5@3.! !

CTileCostume subclass: #CVariableTileCostume
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tweak-Tiles-Costumes'!

!CVariableTileCostume commentStamp: 'ar 3/30/2004 17:17' prior: 0!
CVariableTileCostume is the basis for all 'variable-like' tiles, e.g., tiles which allow drag and drop operations. It uses some events to coordinate the DnD activity internally, including:

#tilePadChildEnter - a tile entered a tile pad embedded in the receiver.
#tilePadChildLeave - a tile left a tile pad embedded in the receiver.
#tilePadChildDrop - a tile was dropped on a tile pad embedded in the receiver.
!

CVariableTileCostume defineFields: '<?xml  version="1.0" ?>
<fields>
	<field toGet="dropEnabled" toSet="dropEnabled:" changeEvent="dropEnabledChanged">dropEnabled</field>
</fields>'!

!CVariableTileCostume methodsFor: 'initialize' stamp: 'ar 3/19/2004 18:02'!
defaultPlayer
	^CVariableTile! !

!CVariableTileCostume methodsFor: 'accessing' stamp: 'ar 3/17/2004 15:16'!
dropEnabled
	"This method was automatically generated"
	<bewareOf: #dropEnabledChanged>
	^self propertyValueAt: #dropEnabled! !

!CVariableTileCostume methodsFor: 'accessing' stamp: 'ar 3/11/2004 14:28'!
dropEnabled: aValue
	"This method was automatically generated"
	^self propertyValueAt: #dropEnabled put: aValue with: #dropEnabledChanged! !

!CVariableTileCostume methodsFor: 'initialize' stamp: 'ar 3/7/2004 23:11'!
initialize
	super initialize.
	self dropEnabled: true.! !

!CVariableTileCostume methodsFor: 'drag and drop' stamp: 'bf 7/25/2005 19:12'!
onDragEnter
	"Called when the hand enters the receiver's bounds whilst dragging something"

	| newTile argType newType newExpression |
	<on: dragEnter>
	dropEnabled ifFalse:[^self].
	newTile := self hand dragContents asPlayer.
	newTile isTilePlayer ifFalse:[^self].

	"See if the types match"
	newType := newTile valueType ifNil:[^nil]. "no type, no drop"
	argType := player valueType.
	argType ifNotNil:[
		(argType canCoerceType: newType) ifFalse:[^nil].
	].
	(self runDropLoop: newTile) ifFalse:[^nil].

	newExpression := newTile tileExpression.
	self assert: [newExpression parent == nil].
	newExpression := newExpression replacing: player expression.
	player expression: newExpression.
	self waitTick.  "Without this, the next line seemingly happens too soon..."
	newExpression signal: #tilePadDropped.
	newExpression ~~ newTile ifTrue:
		[newTile destroy].! !

!CVariableTileCostume methodsFor: 'drag and drop' stamp: 'bf 7/27/2005 17:45'!
onDragStart
	"Handle a drag start - e.g., the user trying to drag something from this slot"
	| expr dragTile |
	<on: mouseDrag>
	expr := player expression ifNil:[^self].
	expr isValueTile ifTrue:[
		"If it's just a simple value tile, hand the user its value"
		dragTile := CValueTile on: expr eval.
		dragTile valueType: expr valueType.
	] ifFalse:[
		"It's more complex than that. Hand the user the full expression and 
		substitute the default value for the given type as the new expression."
		dragTile := expr.
		dragTile parent: nil.
		player valueType ifNil:[
			player expression: nil.
		] ifNotNil:[
			expr := CValueTile on: player valueType defaultValue.
			expr valueType: dragTile valueType.
			player expression: expr.
		].
		dragTile signal: #tilePadDragged with: player.
	].
	hand grab: dragTile.
! !

!CVariableTileCostume methodsFor: 'drag and drop' stamp: 'ar 3/7/2004 23:15'!
onDropEnabledChanged
	<on: dropEnabledChanged>
	dropEnabled 
		ifTrue:[self resumeScript: #onDragEnter]
		ifFalse:[self pauseScript: #onDragEnter]! !

!CVariableTileCostume methodsFor: 'events' stamp: 'ar 6/24/2005 21:09'!
onExpressionChanged
	| expr paren |
	<on: expressionChanged in: player>
	self removeAll.
	expr := player expression.
	expr ifNil:[
		text := player name.
	] ifNotNil:[
		text := ''.
		expr isComplex ifTrue:[
			paren := CLabel new.
			paren value: '('.
			paren font: CTileLibrary default tileFont.
			paren textEmphasis: 1.
			self add: paren.
		].
		expr topLeft: 0@0.
		expr color: Color transparent.
		self add: expr.
		expr isComplex ifTrue:[
			paren := CLabel new.
			paren value: ')'.
			paren font: CTileLibrary default tileFont.
			paren textEmphasis: 1.
			self add: paren.
		].
	].! !

!CVariableTileCostume methodsFor: 'events' stamp: 'ar 3/7/2004 21:11'!
onPlayerChanged
	super onPlayerChanged.
	self onExpressionChanged.! !

!CVariableTileCostume methodsFor: 'drag and drop' stamp: 'bf 4/26/2005 18:38'!
runDropLoop: dragTile
	"Run the feedback loop. This is somewhat complicated since we have to deal with pads which might be nested."
	| oldFill evt |
	[self containsPoint: self cursorPoint] whileTrue:[
		"notify all interested parties"

		"tilePadEntering: Allow a container to stop tracking the dragTile"
		self tileSignal: #tilePadChildEnter.

		"tilePadEnter: Allow the dragTile wants to make itself transparent"
		dragTile signal: #tilePadEnter with: self.

		"tilePadHighlight: Allow the expression tile to become transparent if needed"
		(player expression ifNil:[player]) signal: #tilePadHighlight with: self.

		"give drop feedback and wait for user to drop or to leave or to enter another tile pad"
		oldFill := fill copy.
		fill := Color green.
		evt := self waitUntilAnyOf: {
			self. #dragLeave.
			self. #dragDrop.
			player. #tilePadChildEnter.
		}.

		fill := oldFill.

		"Notify the expression tile that the transaction is over"
		(player expression ifNil:[player]) signal: #tilePadUnhighlight with: self.

		evt selector == #dragDrop ifTrue:[
			"Notify the dragTile that it is being dropped"
			dragTile signal: #tilePadDropped with: player.
			dragTile signal: #dropAccept with: player.

			"Notify containers that I got a drop"
			self tileSignal: #tilePadChildDrop.
			^true "accept it"
		].

		evt selector == #tilePadChildEnter ifTrue:[
			"we are entering a child tile pad"
			evt := self waitUntilAnyOf: {
				player. #tilePadChildLeave.
				player. #tilePadChildDrop.
			}.
			(evt selector == #tilePadChildDrop) ifTrue:[^false]. "dropped on child"
		].

		(self containsPoint: self cursorPoint) ifFalse:[
			"Notify dragTile about leaving me"
			dragTile signal: #tilePadLeave with: self.

			"Notify containers about dragTile leaving me"
			self tileSignal: #tilePadChildLeave.
			^false
		].
	].
	^false! !

!CVariableTileCostume methodsFor: 'initialize' stamp: 'bf 2/21/2005 19:34'!
setupCostume
	super setupCostume.
	self layout: CTableLayout new.
	self layout listDirection: #leftToRight.
	self hResizing: #shrinkWrap; vResizing: #shrinkWrap.! !

CTileCostume subclass: #CVectorTileCostume
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tweak-Tiles-ValueCostumes'!

CVectorTileCostume defineFields: '<?xml  version="1.0" ?>
<fields>
	<field toGet="editorList" toSet="editorList:" changeEvent="editorListChanged">editorList</field>
</fields>'!

!CVectorTileCostume methodsFor: 'accessing' stamp: 'ar 5/12/2005 14:12'!
editorList
	"Answer the editorList of the receiver"
	<bewareOf: #editorListChanged>
	^self propertyValueAt: #editorList! !

!CVectorTileCostume methodsFor: 'accessing' stamp: 'ar 5/12/2005 14:12'!
editorList: aValue
	"Modify the receiver's editorList"
	^self propertyValueAt: #editorList put: aValue with: #editorListChanged! !

!CVectorTileCostume methodsFor: 'events' stamp: 'ar 5/12/2005 16:15'!
onElementChanged
	"This is the valueChanged handler for all the elements"
	| clone fields |
	self pauseScript: #onValueChanged.
	clone := value shallowCopy.
	fields := player valueType vectorFieldsOf: clone.
	editorList with: fields do:[:editor :fieldDef|
		clone setField: fieldDef to: editor value.
	].
	value := clone.
	self resumeScript: #onValueChanged.
! !

!CVectorTileCostume methodsFor: 'events' stamp: 'ar 5/12/2005 13:34'!
onPlayerChanged
	self updateAll.! !

!CVectorTileCostume methodsFor: 'events' stamp: 'ar 5/12/2005 15:01'!
onValueChanged
	| vectorFields |
	<on: valueChanged>
	vectorFields := player valueType vectorFieldsOf: self value.
	vectorFields size = editorList size ifFalse:[self updateAll].
	self pauseScript: #onElementChanged.
	vectorFields with: editorList do:[:fieldDef :editor|
		editor value: (value getField: fieldDef).
	].
	self resumeScript: #onElementChanged.
! !

!CVectorTileCostume methodsFor: 'initialize' stamp: 'ar 5/12/2005 14:51'!
setupCostume
	super setupCostume.
	self extent: 40@TextStyle defaultFont height.
	self layout := CTableLayout new.
	self layout listDirection := #leftToRight.
	self hResizing := #shrinkWrap.
	self vResizing := #shrinkWrap.! !

!CVectorTileCostume methodsFor: 'events' stamp: 'ar 6/21/2005 15:12'!
updateAll
	| tile vectorFields itemType |
	self stopScript: #onElementChanged.
	self removeAll.
	itemType := player valueType elementType.
	vectorFields := player valueType vectorFieldsOf: self value.
	editorList := vectorFields collect:[:field|
		tile := CLabel on: (field name copyWith: $:) asText allBold.
		tile hResizing: #shrinkWrap.
		self addLast: tile.
		tile := CValueTile on: (value getField: field).
		tile valueType: (itemType ifNil:[CTileType defaultTileTypeFor: tile value]).
		tile hResizing: #rigid.
		(tile costume get: #editor) ifNotNilDo:[:ed| ed hResizing: #spaceFill]. "@@@@ THIS SUCKS"
		tile width: 30.
		self addLast: tile.
		tile].
	self startScript: #onElementChanged when: (Array streamContents:[:s|
		editorList do:[:aTile| s nextPut: aTile; nextPut: #valueChanged].
	]).
! !
CScriptorTile initialize!
